@@ -318,21 +318,6 @@ m.Template = m.Class({
 		return 1;
 	},
 
-	// returns true if the entity can attack the given class
-	canAttackClass: function(saidClass) {
-		if (!this.get("Attack"))
-			return false;
-
-		for (let type in this.get("Attack"))
-		{
-			let restrictedClasses = this.get("Attack/" + type + "/RestrictedClasses/_string");
-			if (restrictedClasses && !MatchesClassList([saidClass], restrictedClasses))
-				return false;
-		}
-
-		return true;
-	},
-
 	"buildableEntities": function() {
 		let templates = this.get("Builder/Entities/_string");
 		if (!templates)
@@ -351,11 +336,11 @@ m.Template = m.Class({
 	},
 
 	"researchableTechs": function(civ) {
-		if (this.civ() !== civ)     // techs can only be researched in structures from the player civ TODO no more true
-			return undefined;
 		let templates = this.get("ProductionQueue/Technologies/_string");
 		if (!templates)
 			return undefined;
+		if (civ)
+			templates = templates.replace(/\{civ\}/g, civ);
 		return templates.split(/\s+/);
 	},
 
@@ -605,6 +590,7 @@ m.Entity = m.Class({
 		return this._entity.idle;
 	},
 
+	"getStance": function() { return this._entity.stance !== undefined ? this._entity.stance : undefined; },
 	unitAIState: function() { return this._entity.unitAIState !== undefined ? this._entity.unitAIState : undefined; },
 	unitAIOrderData: function() { return this._entity.unitAIOrderData !== undefined ? this._entity.unitAIOrderData : undefined; },
 
@@ -745,6 +731,46 @@ m.Entity = m.Class({
 	garrisoned: function() { return this._entity.garrisoned; },
 	canGarrisonInside: function() { return this._entity.garrisoned.length < this.garrisonMax(); },
 
+	/**
+	 * returns true if the entity can attack (including capture) the given class.
+	 */
+	"canAttackClass": function(aClass)
+	{
+		if (!this.get("Attack"))
+			return false;
+
+		for (let type in this.get("Attack"))
+		{
+			if (type === "Slaughter")
+				continue;
+			let restrictedClasses = this.get("Attack/" + type + "/RestrictedClasses/_string");
+			if (!restrictedClasses || !MatchesClassList([aClass], restrictedClasses))
+				return true;
+		}
+		return false;
+	},
+
+	/**
+	 * returns true if the entity can capture the given target entity
+	 * if no target is given, returns true if the entity has the Capture attack
+	 */
+	"canCapture": function(target)
+	{
+		if (!this.get("Attack/Capture"))
+			return false;
+
+		if (!target)
+			return true;
+		let restrictedClasses = this.get("Attack/Capture/RestrictedClasses/_string");
+		return !restrictedClasses || !MatchesClassList(target.classes(), restrictedClasses);
+	},
+
+	"isCapturable": function() { return this.get("Capturable") !== undefined; },
+
+	"canGuard": function() { return this.get("UnitAI/CanGuard") === "true"; },
+
+	"canGarrison": function() { return this.get("Garrisonable") !== "false"; },
+
 	move: function(x, z, queued = false) {
 		Engine.PostCommand(PlayerID,{"type": "walk", "entities": [this.id()], "x": x, "z": z, "queued": queued });
 		return this;
@@ -762,6 +788,8 @@ m.Entity = m.Class({
 
 	// violent, aggressive, defensive, passive, standground
 	setStance: function(stance, queued = false) {
+		if (this.getStance() === undefined)
+			return undefined;
 		Engine.PostCommand(PlayerID,{"type": "stance", "entities": [this.id()], "name" : stance, "queued": queued });
 		return this;
 	},
@@ -929,6 +957,16 @@ m.Entity = m.Class({
 			if (item.progress < percentToStopAt)
 				Engine.PostCommand(PlayerID,{ "type": "stop-production", "entity": this.id(), "id": item.id });
 		return this;
+	},
+
+	"guard": function(target, queued = false) {
+		Engine.PostCommand(PlayerID, { "type": "guard", "entities": [this.id()], "target": target.id(), "queued": queued });
+		return this;
+	},
+
+	"removeGuard": function() {
+		Engine.PostCommand(PlayerID, { "type": "remove-guard", "entities": [this.id()] });
+		return this;
 	}
 });
 
