@@ -9,6 +9,14 @@ m.warn = function(output)
 		warn("PlayerID " + PlayerID + " |   " + uneval(output));
 };
 
+/**
+ * Useful for simulating consecutive AI matches.
+ */
+m.exit = function()
+{
+	Engine.Exit();
+};
+
 m.VectorDistance = function(a, b)
 {
 	let dx = a[0] - b[0];
@@ -23,44 +31,6 @@ m.SquareVectorDistance = function(a, b)
 	return dx*dx + dz*dz;
 };
 
-/**
- * A is the reference, B must be in "range" of A
- * this supposes the range is already squared
- */
-m.inRange = function(a, b, range)// checks for X distance
-{
-	// will avoid unnecessary checking for position in some rare cases... I'm lazy
-	if (a === undefined || b === undefined || range === undefined)
-		return undefined;
-
-	let dx = a[0] - b[0];
-	let dz = a[1] - b[1];
-	return dx*dx + dz*dz < range;
-};
-
-/** Slower than SquareVectorDistance, faster than VectorDistance but not exactly accurate. */
-m.ManhattanDistance = function(a, b)
-{
-	let dx = a[0] - b[0];
-	let dz = a[1] - b[1];
-	return Math.abs(dx) + Math.abs(dz);
-};
-
-m.AssocArraytoArray = function(assocArray)
-{
-	let endArray = [];
-	for (let i in assocArray)
-		endArray.push(assocArray[i]);
-	return endArray;
-};
-
-/** Picks a random element from an array */
-m.PickRandom = function(list)
-{
-	return list.length ? list[Math.floor(Math.random()*list.length)] : undefined;
-};
-
-
 /** Utility functions for conversions of maps of different sizes */
 
 /**
@@ -97,7 +67,7 @@ m.getMapIndices = function(i, map1, map2)
 };
 
 /**
- * Returns the list of points of map2 contained inside the cell i of map1 
+ * Returns the list of points of map2 contained inside the cell i of map1
  * map1.cellSize must be a multiple of map2.cellSize
  */
 m.getMapPoints = function(i, map1, map2)
