@@ -3,10 +3,6 @@ Engine.IncludeModule("common-api");
 var PETRA = (function() {
 var m = {};
 
-// "local" global variables for stuffs that will need a unique ID
-// Note that since order of loading is alphabetic, this means this file must go before any other file using them.
-m.playerGlobals = [];
-
 m.PetraBot = function PetraBot(settings)
 {
 	API3.BaseAI.call(this, settings);
@@ -15,7 +11,7 @@ m.PetraBot = function PetraBot(settings)
 	this.elapsedTime = 0;
 
 	this.uniqueIDs = {
-		"armies": 0,
+		"armies": 1,	// starts at 1 to allow easier tests on armies ID existence
 		"bases": 1,	// base manager ID starts at one because "0" means "no base" on the map
 		"plans": 0,	// training/building/research plans
 		"transports": 1	// transport plans start at 1 because 0 might be used as none
@@ -28,7 +24,7 @@ m.PetraBot = function PetraBot(settings)
 
 m.PetraBot.prototype = new API3.BaseAI();
 
-m.PetraBot.prototype.CustomInit = function(gameState, sharedScript)
+m.PetraBot.prototype.CustomInit = function(gameState)
 {
 	if (this.isDeserialized)
 	{
@@ -77,7 +73,7 @@ m.PetraBot.prototype.CustomInit = function(gameState, sharedScript)
 
 		// this.queues can only be modified by the queue manager or things will go awry.
 		this.queues = {};
-		for (var i in this.Config.priorities)
+		for (let i in this.Config.priorities)
 			this.queues[i] = new m.Queue();
 
 		this.queueManager = new m.QueueManager(this.Config, this.queues);
@@ -109,7 +105,7 @@ m.PetraBot.prototype.OnUpdate = function(sharedScript)
 	// Run the update every n turns, offset depending on player ID to balance the load
 	this.elapsedTime = this.gameState.getTimeElapsed() / 1000;
 	if (!this.playedTurn || (this.turn + this.player) % 8 == 5)
-	{		
+	{
 		Engine.ProfileStart("PetraBot bot (player " + this.player +")");
 
 		this.playedTurn++;
@@ -117,19 +113,19 @@ m.PetraBot.prototype.OnUpdate = function(sharedScript)
 		if (this.gameState.getOwnEntities().length === 0)
 		{
 			Engine.ProfileStop();
-			return; // With no entities to control the AI cannot do anything 
+			return; // With no entities to control the AI cannot do anything
 		}
 
 		this.HQ.update(this.gameState, this.queues, this.savedEvents);
 
 		this.queueManager.update(this.gameState);
-		
+
 		for (let i in this.savedEvents)
 			this.savedEvents[i] = [];
 
 		Engine.ProfileStop();
 	}
-	
+
 	this.turn++;
 };
 
