@@ -130,7 +130,7 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 	// TODO the 30 is to take roughly into account the structure size in following checks. Can be improved
 	if (entity.attackTypes().indexOf("Ranged") !== -1)
 		dist2Min = (entity.attackRange("Ranged").max + 30) * (entity.attackRange("Ranged").max + 30);
-    
+
 	for (let targetId of this.targetList)
 	{
 		let target = gameState.getEntityById(targetId);
@@ -187,7 +187,7 @@ m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 	{
 		if (!this.armies.length)
 		{
-			// check if we can recover capture points from any of our notdecaying structures		
+			// check if we can recover capture points from any of our notdecaying structures
 			for (let ent of gameState.getOwnStructures().values())
 			{
 				if (ent.decaying())
@@ -266,6 +266,7 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 
 		if (army.getState() === 0)
 		{
+			this.switchToAttack(gameState, army);
 			army.clear(gameState);
 			this.armies.splice(i--,1);
 			continue;
@@ -347,6 +348,7 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 		if (stillDangerous)
 			continue;
 
+		this.switchToAttack(gameState, army);
 		army.clear(gameState);
 		this.armies.splice(i--,1);
 	}
@@ -356,7 +358,7 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 {
 	if (this.armies.length === 0)
 		return;
-	
+
 	let armiesNeeding = [];
 	// let's add defenders
 	for (let army of this.armies)
@@ -364,7 +366,7 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 		let needsDef = army.needsDefenders(gameState);
 		if (needsDef === false)
 			continue;
-		
+
 		// Okay for now needsDef is the total needed strength.
 		// we're dumb so we don't choose if we have a defender shortage.
 		armiesNeeding.push( {"army": army, "need": needsDef} );
@@ -374,7 +376,7 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 		return;
 
 	// let's get our potential units
-	let potentialDefenders = []; 
+	let potentialDefenders = [];
 	gameState.getOwnUnits().forEach(function(ent) {
 		if (!ent.position())
 			return;
@@ -388,7 +390,7 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 			return;
 		if (ent.getMetadata(PlayerID, "transport") !== undefined || ent.getMetadata(PlayerID, "transporter") !== undefined)
 			return;
-		if (gameState.getGameType() === "regicide" && ent.hasClass("Hero") && ent.healthLevel() < 0.8)
+		if (gameState.ai.HQ.gameTypeManager.criticalEnts.has(ent.id()))
 			return;
 		if (ent.getMetadata(PlayerID, "plan") !== undefined && ent.getMetadata(PlayerID, "plan") !== -1)
 		{
@@ -398,7 +400,7 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 		}
 		potentialDefenders.push(ent.id());
 	});
-	
+
 	for (let a = 0; a < armiesNeeding.length; ++a)
 		armiesNeeding[a].army.recalculatePosition(gameState);
 
@@ -457,6 +459,7 @@ m.DefenseManager.prototype.abortArmy = function(gameState, army)
  * If our defense structures are attacked, garrison soldiers inside when possible
  * and if a support unit is attacked and has less than 55% health, garrison it inside the nearest healing structure
  * and if a ranged siege unit (not used for defense) is attacked, garrison it in the nearest fortress
+ * If our hero is attacked in regicide game mode, the gameTypeManager will handle it
  */
 m.DefenseManager.prototype.checkEvents = function(gameState, events)
 {
@@ -464,54 +467,44 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 	for (let army of this.armies)
 		army.checkEvents(gameState, events);
 
+	for (let evt of events.OwnershipChanged)   // capture events
+	{
+		if (gameState.isPlayerMutualAlly(evt.from) && evt.to > 0)
+		{
+			let ent = gameState.getEntityById(evt.entity);
+			if (ent && ent.hasClass("CivCentre")) // one of our cc has been captured
+				gameState.ai.HQ.attackManager.switchDefenseToAttack(gameState, ent, { "range": 150 });
+		}
+	}
+
 	for (let evt of events.Attacked)
 	{
 		let target = gameState.getEntityById(evt.target);
-		if (!target || !gameState.isEntityOwn(target) || !target.position())
+		if (!target || !target.position())
 			continue;
-		// If attacked by one of our allies (he must trying to recover capture points), do not react
+
 		let attacker = gameState.getEntityById(evt.attacker);
+		if (attacker && gameState.isEntityOwn(attacker) && gameState.isEntityEnemy(target) && !attacker.hasClass("Ship") &&
+		   (!target.hasClass("Structure") || target.attackRange("Ranged")))
+		{
+			// If enemies are in range of one of our defensive structures, garrison it for arrow multiplier
+			// (enemy non-defensive structure are not considered to stay in sync with garrisonManager)
+			if (attacker.position() && attacker.isGarrisonHolder() && attacker.getArrowMultiplier())
+				this.garrisonUnitsInside(gameState, attacker, {"attacker": target});
+		}
+
+		if (!gameState.isEntityOwn(target))
+			continue;
+
+		// If attacked by one of our allies (he must trying to recover capture points), do not react
 		if (attacker && gameState.isEntityAlly(attacker))
 			continue;
 
 		if (target.hasClass("Ship"))    // TODO integrate ships later   need to be sure it is accessible
 			continue;
 
-		let plan = target.getMetadata(PlayerID, "plan");
-
-		// Retreat the hero in regicide mode if wounded
-		if (gameState.getGameType() == "regicide" && target.hasClass("Hero") && target.healthLevel() < 0.7)
-		{
-			target.stopMoving();
-
-			if (plan >= 0)
-			{
-				let attackPlan = gameState.ai.HQ.attackManager.getPlan(target.getMetadata(PlayerID, "plan"));
-				if (attackPlan)
-					attackPlan.removeUnit(target, true);
-			}
-
-			if (target.getMetadata(PlayerID, "PartOfArmy"))
-			{
-				let army = gameState.ai.HQ.defenseManager.getArmy(target.getMetadata(PlayerID, "PartOfArmy"));
-				if (army)
-					army.removeOwn(gameState, target.id());
-			}
-
-			this.garrisonUnitForHealing(gameState, target);
-
-			if (plan >= 0) // couldn't find a place to garrison, so the hero will flee from attacks
-			{
-				target.setStance("passive");
-				let accessIndex = gameState.ai.accessibility.getAccessValue(target.position());
-				let basePos = m.getBestBase(gameState, target);
-				if (basePos && basePos.accessIndex == accessIndex)
-					target.move(basePos.anchor.position()[0], basePos.anchor.position()[1]);
-			}
-			continue;
-		}
-
 		// If inside a started attack plan, let the plan deal with this unit
+		let plan = target.getMetadata(PlayerID, "plan");
 		if (plan !== undefined && plan >= 0)
 		{
 			let attack = gameState.ai.HQ.attackManager.getPlan(plan);
@@ -559,14 +552,14 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 		}
 
 		// try to garrison any attacked support unit if low healthlevel
-		if (target.hasClass("Support") && target.healthLevel() < 0.55 &&
+		if (target.hasClass("Support") && target.healthLevel() < this.Config.garrisonHealthLevel.medium &&
 			!target.getMetadata(PlayerID, "transport") && plan !== -2 && plan !== -3)
 		{
-			this.garrisonUnitForHealing(gameState, target);
+			this.garrisonAttackedUnit(gameState, target);
 			continue;
 		}
 
-		// try to garrison any attacked range siege unit 
+		// try to garrison any attacked range siege unit
 		if (target.hasClass("Siege") && !target.hasClass("Melee") &&
 			!target.getMetadata(PlayerID, "transport") && plan !== -2 && plan !== -3)
 		{
@@ -578,52 +571,76 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 			continue;
 
 		if (target.isGarrisonHolder() && target.getArrowMultiplier())
-			this.garrisonRangedUnitsInside(gameState, target, {"attacker": attacker});
+			this.garrisonUnitsInside(gameState, target, {"attacker": attacker});
 	}
 };
 
-m.DefenseManager.prototype.garrisonRangedUnitsInside = function(gameState, target, data)
+m.DefenseManager.prototype.garrisonUnitsInside = function(gameState, target, data)
 {
 	let minGarrison = data.min ? data.min : target.garrisonMax();
 	let typeGarrison = data.type ? data.type : "protection";
 	if (gameState.ai.HQ.garrisonManager.numberOfGarrisonedUnits(target) >= minGarrison)
-		return;
+		return false;
 	if (target.hitpoints() < target.garrisonEjectHealth() * target.maxHitpoints())
-		return;
+		return false;
 	if (data.attacker)
 	{
 		let attackTypes = target.attackTypes();
 		if (!attackTypes || attackTypes.indexOf("Ranged") === -1)
-			return;
+			return false;
 		let dist = API3.SquareVectorDistance(data.attacker.position(), target.position());
 		let range = target.attackRange("Ranged").max;
 		if (dist >= range*range)
-			return;
+			return false;
 	}
 	let index = gameState.ai.accessibility.getAccessValue(target.position());
 	let garrisonManager = gameState.ai.HQ.garrisonManager;
 	let garrisonArrowClasses = target.getGarrisonArrowClasses();
-	let units = gameState.getOwnUnits().filter(ent => MatchesClassList(garrisonArrowClasses, ent.classes())).filterNearest(target.position());
+	let units = gameState.getOwnUnits().filter(ent => MatchesClassList(ent.classes(), garrisonArrowClasses)).filterNearest(target.position());
+	let allowMelee = gameState.ai.HQ.garrisonManager.allowMelee(target);
+	if (allowMelee === undefined)
+	{
+		// Should be kept in sync with garrisonManager to avoid garrisoning-ungarrisoning some units
+		if (data.attacker)
+			allowMelee = data.attacker.hasClass("Structure") ? data.attacker.attackRange("Ranged") : !m.isSiegeUnit(data.attacker);
+		else
+			allowMelee = true;
+	}
+	let ret = false;
 	for (let ent of units.values())
 	{
 		if (garrisonManager.numberOfGarrisonedUnits(target) >= minGarrison)
 			break;
 		if (!ent.position())
 			continue;
+		if (typeGarrison !== "decay" && !allowMelee && ent.attackTypes().indexOf("Melee") !== -1)
+			continue;
 		if (ent.getMetadata(PlayerID, "transport") !== undefined)
 			continue;
-		if (ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3)
+		let army = ent.getMetadata(PlayerID, "PartOfArmy") ? this.getArmy(ent.getMetadata(PlayerID, "PartOfArmy")) : undefined;
+		if (!army && (ent.getMetadata(PlayerID, "plan") === -2 || ent.getMetadata(PlayerID, "plan") === -3))
 			continue;
-		if (ent.getMetadata(PlayerID, "plan") !== undefined && ent.getMetadata(PlayerID, "plan") !== -1)
+		if (ent.getMetadata(PlayerID, "plan") !== undefined && ent.getMetadata(PlayerID, "plan") >= 0)
 		{
 			let subrole = ent.getMetadata(PlayerID, "subrole");
-			if (subrole && (subrole === "completing" || subrole === "walking" || subrole === "attacking")) 
+			// when structure decaying (usually because we've just captured it in enemy territory), also allow units from an attack plan
+			if (typeGarrison !== "decay" && subrole && (subrole === "completing" || subrole === "walking" || subrole === "attacking"))
 				continue;
 		}
 		if (gameState.ai.accessibility.getAccessValue(ent.position()) !== index)
 			continue;
+		if (ent.getMetadata(PlayerID, "plan") !== undefined && ent.getMetadata(PlayerID, "plan") >= 0)
+		{
+			let attackPlan = gameState.ai.HQ.attackManager.getPlan(ent.getMetadata(PlayerID, "plan"));
+			if (attackPlan)
+				attackPlan.removeUnit(ent, true);
+		}
+		if (army)
+			army.removeOwn(gameState, ent.id());
 		garrisonManager.garrison(gameState, ent, target, typeGarrison);
+		ret = true;
 	}
+	return ret;
 };
 
 /** garrison a attacked siege ranged unit inside the nearest fortress */
@@ -634,19 +651,13 @@ m.DefenseManager.prototype.garrisonSiegeUnit = function(gameState, unit)
 	let unitAccess = gameState.ai.accessibility.getAccessValue(unit.position());
 	let garrisonManager = gameState.ai.HQ.garrisonManager;
 	gameState.getAllyStructures().forEach(function(ent) {
-		if (!MatchesClassList(ent.garrisonableClasses(), unit.classes()))
+		if (!MatchesClassList(unit.classes(), ent.garrisonableClasses()))
 			return;
 		if (garrisonManager.numberOfGarrisonedUnits(ent) >= ent.garrisonMax())
 			return;
 		if (ent.hitpoints() < ent.garrisonEjectHealth() * ent.maxHitpoints())
 			return;
-		let entAccess = ent.getMetadata(PlayerID, "access");
-		if (!entAccess)
-		{
-			entAccess = gameState.ai.accessibility.getAccessValue(ent.position());
-			ent.setMetadata(PlayerID, "access", entAccess);
-		}
-		if (entAccess !== unitAccess)
+		if (m.getLandAccess(gameState, ent) !== unitAccess)
 			return;
 		let dist = API3.SquareVectorDistance(ent.position(), unit.position());
 		if (dist > distmin)
@@ -656,40 +667,52 @@ m.DefenseManager.prototype.garrisonSiegeUnit = function(gameState, unit)
 	});
 	if (nearest)
 		garrisonManager.garrison(gameState, unit, nearest, "protection");
+	return nearest !== undefined;
 };
 
-/** garrison a hurt unit inside the nearest healing structure */
-m.DefenseManager.prototype.garrisonUnitForHealing = function(gameState, unit)
+/**
+ * Garrison a hurt unit inside a player-owned or allied structure
+ * If emergency is true, the unit will be garrisoned in the closest possible structure
+ * Otherwise, it will garrison in the closest healing structure
+ */
+m.DefenseManager.prototype.garrisonAttackedUnit = function(gameState, unit, emergency = false)
 {
 	let distmin = Math.min();
 	let nearest;
 	let unitAccess = gameState.ai.accessibility.getAccessValue(unit.position());
 	let garrisonManager = gameState.ai.HQ.garrisonManager;
-	gameState.getAllyStructures().forEach(function(ent) {
-		if (!ent.buffHeal())
-			return;
-		if (!MatchesClassList(ent.garrisonableClasses(), unit.classes()))
-			return;
-		if (garrisonManager.numberOfGarrisonedUnits(ent) >= ent.garrisonMax())
-			return;
+	for (let ent of gameState.getAllyStructures().values())
+	{
+		if (!emergency && !ent.buffHeal())
+			continue;
+		if (!MatchesClassList(unit.classes(), ent.garrisonableClasses()))
+			continue;
+		if (garrisonManager.numberOfGarrisonedUnits(ent) >= ent.garrisonMax() &&
+		    (!emergency || !ent.garrisoned().length))
+			continue;
 		if (ent.hitpoints() < ent.garrisonEjectHealth() * ent.maxHitpoints())
-			return;
-		let entAccess = ent.getMetadata(PlayerID, "access");
-		if (!entAccess)
-		{
-			entAccess = gameState.ai.accessibility.getAccessValue(ent.position());
-			ent.setMetadata(PlayerID, "access", entAccess);
-		}
-		if (entAccess !== unitAccess)
-			return;
+			continue;
+		if (m.getLandAccess(gameState, ent) !== unitAccess)
+			continue;
 		let dist = API3.SquareVectorDistance(ent.position(), unit.position());
 		if (dist > distmin)
-			return;
+			continue;
 		distmin = dist;
 		nearest = ent;
-	});
-	if (nearest)
+	}
+	if (!nearest)
+		return false;
+
+	if (!emergency)
+	{
 		garrisonManager.garrison(gameState, unit, nearest, "protection");
+		return true;
+	}
+	if (garrisonManager.numberOfGarrisonedUnits(nearest) >= nearest.garrisonMax()) // make room for this ent
+		nearest.unload(nearest.garrisoned()[0]);
+
+	garrisonManager.garrison(gameState, unit, nearest, nearest.buffHeal() ? "protection" : "emergency");
+	return true;
 };
 
 /**
@@ -704,6 +727,36 @@ m.DefenseManager.prototype.GetCooperationLevel = function(ent)
 	return cooperation;
 };
 
+/**
+ * Switch a defense army into an attack if needed
+ */
+m.DefenseManager.prototype.switchToAttack = function(gameState, army)
+{
+	if (!army)
+		return;
+	for (let targetId of this.targetList)
+	{
+		let target = gameState.getEntityById(targetId);
+		if (!target || !target.position() || !gameState.isPlayerEnemy(target.owner()))
+			continue;
+		let targetAccess = m.getLandAccess(gameState, target);
+		let targetPos = target.position();
+		for (let entId of army.ownEntities)
+		{
+			let ent = gameState.getEntityById(entId);
+			if (!ent)
+				continue;
+			let pos = ent.position();
+			if (!pos || gameState.ai.accessibility.getAccessValue(pos) !== targetAccess)
+				continue;
+			if (API3.SquareVectorDistance(targetPos, pos) > 14400)
+				continue;
+			gameState.ai.HQ.attackManager.switchDefenseToAttack(gameState, target, { "armyID": army.ID, "uniqueTarget": true });
+			return;
+		}
+	}
+};
+
 m.DefenseManager.prototype.Serialize = function()
 {
 	let properties = {
