@@ -1,21 +1,20 @@
 var PETRA = function(m)
 {
 
-/* Naval Manager
- Will deal with anything ships.
- -Basically trade over water (with fleets and goals commissioned by the economy manager)
- -Defense over water (commissioned by the defense manager)
-	-subtask being patrols, escort, naval superiority.
- -Transport of units over water (a few units).
- -Scouting, ultimately.
- Also deals with handling docks, making sure we have access and stuffs like that.
- Does not build them though, that's for the base manager to handle.
+/**
+ * Naval Manager
+ * Will deal with anything ships.
+ * -Basically trade over water (with fleets and goals commissioned by the economy manager)
+ * -Defense over water (commissioned by the defense manager)
+ * -Transport of units over water (a few units).
+ * -Scouting, ultimately.
+ * Also deals with handling docks, making sure we have access and stuffs like that.
  */
 
 m.NavalManager = function(Config)
 {
 	this.Config = Config;
-	
+
 	// ship subCollections. Also exist for land zones, idem, not caring.
 	this.seaShips = [];
 	this.seaTransportShips = [];
@@ -29,21 +28,20 @@ m.NavalManager = function(Config)
 	// needed NB per zone.
 	this.neededTransportShips = [];
 	this.neededWarShips = [];
-	
+
 	this.transportPlans = [];
 
 	// shore-line regions where we can load and unload units
 	this.landingZones = {};
 };
 
-// More initialisation for stuff that needs the gameState
+/** More initialisation for stuff that needs the gameState */
 m.NavalManager.prototype.init = function(gameState, deserializing)
 {
-	// finished docks
-	this.docks = gameState.getOwnStructures().filter(API3.Filters.and(API3.Filters.byClassesOr(["Dock", "Shipyard"]),
-		API3.Filters.not(API3.Filters.isFoundation())));
+	// docks
+	this.docks = gameState.getOwnStructures().filter(API3.Filters.byClassesOr(["Dock", "Shipyard"]));
 	this.docks.registerUpdates();
-	
+
 	this.ships = gameState.getOwnUnits().filter(API3.Filters.and(API3.Filters.byClass("Ship"), API3.Filters.not(API3.Filters.byMetadata(PlayerID, "role", "trader"))));
 	// note: those two can overlap (some transport ships are warships too and vice-versa).
 	this.transportShips = this.ships.filter(API3.Filters.and(API3.Filters.byCanGarrison(), API3.Filters.not(API3.Filters.byClass("FishingBoat"))));
@@ -54,16 +52,14 @@ m.NavalManager.prototype.init = function(gameState, deserializing)
 	this.transportShips.registerUpdates();
 	this.warShips.registerUpdates();
 	this.fishShips.registerUpdates();
-	
-	var fishes = gameState.getFishableSupplies();
-	var availableFishes = {};
-	for (let fish of fishes.values())
+
+	let availableFishes = {};
+	for (let fish of gameState.getFishableSupplies().values())
 	{
-		let sea = gameState.ai.accessibility.getAccessValue(fish.position(), true);
-		fish.setMetadata(PlayerID, "sea", sea);
-		if (availableFishes[sea])
+		let sea = this.getFishSea(gameState, fish);
+		if (sea && availableFishes[sea])
 			availableFishes[sea] += fish.resourceSupplyAmount();
-		else
+		else if (sea)
 			availableFishes[sea] = fish.resourceSupplyAmount();
 	}
 
@@ -107,22 +103,12 @@ m.NavalManager.prototype.init = function(gameState, deserializing)
 		}
 	}
 
-	// load units and buildings from the config files
-	let civ = gameState.civ();
-	if (civ in this.Config.buildings.naval)
-		this.bNaval = this.Config.buildings.naval[civ];
-	else
-		this.bNaval = this.Config.buildings.naval['default'];
-
-	for (let i in this.bNaval)
-		this.bNaval[i] = gameState.applyCiv(this.bNaval[i]);
-
 	if (deserializing)
 		return;
 
 	// determination of the possible landing zones
-	var width = gameState.getMap().width;
-	var length = width * gameState.getMap().height;
+	let width = gameState.getPassabilityMap().width;
+	let length = width * gameState.getPassabilityMap().height;
 	for (let i = 0; i < length; ++i)
 	{
 		let land = gameState.ai.accessibility.landPassMap[i];
@@ -172,7 +158,7 @@ m.NavalManager.prototype.init = function(gameState, deserializing)
 	for (let ship of this.ships.values())
 		this.setShipIndex(gameState, ship);
 	for (let dock of this.docks.values())
-		this.setDockIndex(gameState, dock);
+		this.setAccessIndices(gameState, dock);
 };
 
 m.NavalManager.prototype.updateFishingBoats = function(sea, num)
@@ -186,8 +172,13 @@ m.NavalManager.prototype.resetFishingBoats = function(gameState, sea)
 	if (sea !== undefined)
 		this.wantedFishShips[sea] = 0;
 	else
-		for (let i = 0; i < gameState.ai.accessibility.regionSize.length; ++i)
-			this.wantedFishShips[i] = 0;
+		this.wantedFishShips.fill(0);
+};
+
+m.NavalManager.prototype.setAccessIndices = function(gameState, ent)
+{
+	m.getLandAccess(gameState, ent);
+	m.getSeaAccess(gameState, ent);
 };
 
 m.NavalManager.prototype.setShipIndex = function(gameState, ship)
@@ -196,53 +187,77 @@ m.NavalManager.prototype.setShipIndex = function(gameState, ship)
 	ship.setMetadata(PlayerID, "sea", sea);
 };
 
-m.NavalManager.prototype.setDockIndex = function(gameState, dock)
+/** Get the sea, cache it if not yet done and check if in opensea */
+m.NavalManager.prototype.getFishSea = function(gameState, fish)
 {
-	var land = dock.getMetadata(PlayerID, "access");
-	if (land === undefined)
-	{
-		land = this.getDockIndex(gameState, dock, false);
-		dock.setMetadata(PlayerID, "access", land);
-	}
-	var sea = dock.getMetadata(PlayerID, "sea");
-	if (sea === undefined)
-	{
-		sea = this.getDockIndex(gameState, dock, true);
-		dock.setMetadata(PlayerID, "sea", sea);
-	}
-};
-
-// get the indices for our starting docks and those of our allies
-// land index when onWater=false, sea indes when true
-m.NavalManager.prototype.getDockIndex = function(gameState, dock, onWater)
-{
-	var index = gameState.ai.accessibility.getAccessValue(dock.position(), onWater);
-	if (index < 2)
-	{
-		// pre-positioned docks are sometimes not well positionned
-		var dockPos = dock.position();
-		var radius = dock.footprintRadius();
-		for (let i = 0; i < 16; i++)
+	let sea = fish.getMetadata(PlayerID, "sea");
+	if (sea)
+		return sea;
+	const ntry = 4;
+	const around = [ [-0.7,0.7], [0,1], [0.7,0.7], [1,0], [0.7,-0.7], [0,-1], [-0.7,-0.7], [-1,0] ];
+	let pos = gameState.ai.accessibility.gamePosToMapPos(fish.position());
+	let width = gameState.ai.accessibility.width;
+	let k = pos[0] + pos[1]*width;
+	sea = gameState.ai.accessibility.navalPassMap[k];
+	fish.setMetadata(PlayerID, "sea", sea);
+	let radius = 120 / gameState.ai.accessibility.cellSize / ntry;
+	if (around.every(a =>
 		{
-			let pos = [ dockPos[0] + radius*Math.cos(i*Math.PI/8), dockPos[1] + radius*Math.sin(i*Math.PI/8)];
-
-			index = gameState.ai.accessibility.getAccessValue(pos, onWater);
-			if (index >= 2)
-				break;
-		}
-	}
-	if (index < 2)
-		API3.warn("ERROR in Petra navalManager because of dock position (onWater=" + onWater + ") index " + index);
-	return index;
+			for (let t = 0; t < ntry; ++t)
+			{
+				let i = pos[0] + Math.round(a[0]*radius*(ntry-t));
+				let j = pos[1] + Math.round(a[1]*radius*(ntry-t));
+				if (i < 0 || i >= width || j < 0 || j >= width)
+					continue;
+				if (gameState.ai.accessibility.landPassMap[i + j*width] === 1)
+				{
+					let navalPass = gameState.ai.accessibility.navalPassMap[i + j*width];
+					if (navalPass === sea)
+						return true;
+					else if (navalPass === 1)  // we could be outside the map
+						continue;
+				}
+				return false;
+			}
+			return true;
+		}))
+		fish.setMetadata(PlayerID, "opensea", true);
+	return sea;
 };
 
-// get the list of seas (or lands) around this region not connected by a dock
+/** check if we can safely fish at the fish position */
+m.NavalManager.prototype.canFishSafely = function(gameState, fish)
+{
+	if (fish.getMetadata(PlayerID, "opensea"))
+		return true;
+	const ntry = 4;
+	const around = [ [-0.7,0.7], [0,1], [0.7,0.7], [1,0], [0.7,-0.7], [0,-1], [-0.7,-0.7], [-1,0] ];
+	let territoryMap = gameState.ai.HQ.territoryMap;
+	let width = territoryMap.width;
+	let radius = 140 / territoryMap.cellSize / ntry;
+	let pos = territoryMap.gamePosToMapPos(fish.position());
+	return around.every(a =>
+		{
+			for (let t = 0; t < ntry; ++t)
+			{
+				let i = pos[0] + Math.round(a[0]*radius*t);
+				let j = pos[1] + Math.round(a[1]*radius*t);
+				if (i < 0 || i >= width || j < 0 || j >= width)
+					break;
+				let owner = territoryMap.getOwnerIndex(i + j*width);
+				if (owner !== 0 && gameState.isPlayerEnemy(owner))
+					return false;
+			}
+			return true;
+		});
+};
+
+/** get the list of seas (or lands) around this region not connected by a dock */
 m.NavalManager.prototype.getUnconnectedSeas = function(gameState, region)
 {
-	var seas = gameState.ai.accessibility.regionLinks[region].slice();
-	var docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock"));
-	docks.forEach(function (dock) {
-		if (dock.getMetadata(PlayerID, "access") !== region)
+	let seas = gameState.ai.accessibility.regionLinks[region].slice();
+	this.docks.forEach(function (dock) {
+		if (!dock.hasClass("Dock") || dock.getMetadata(PlayerID, "access") !== region)
 			return;
 		let i = seas.indexOf(dock.getMetadata(PlayerID, "sea"));
 		if (i !== -1)
@@ -253,25 +268,25 @@ m.NavalManager.prototype.getUnconnectedSeas = function(gameState, region)
 
 m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 {
-	for (let evt of events.ConstructionFinished)
+	for (let evt of events.Create)
 	{
-		if (!evt || !evt.newentity)
+		if (!evt.entity)
 			continue;
-		let entity = gameState.getEntityById(evt.newentity);
-		if (entity && entity.hasClass("Dock") && entity.isOwn(PlayerID))
-			this.setDockIndex(gameState, entity);
+		let ent = gameState.getEntityById(evt.entity);
+		if (ent && ent.isOwn(PlayerID) && ent.foundationProgress() !== undefined && (ent.hasClass("Dock") || ent.hasClass("Shipyard")))
+			this.setAccessIndices(gameState, ent);
 	}
 
 	for (let evt of events.TrainingFinished)
 	{
-		if (!evt || !evt.entities)
+		if (!evt.entities)
 			continue;
 		for (let entId of evt.entities)
 		{
-			let entity = gameState.getEntityById(entId);
-			if (!entity || !entity.hasClass("Ship") || !entity.isOwn(PlayerID))
+			let ent = gameState.getEntityById(entId);
+			if (!ent || !ent.hasClass("Ship") || !ent.isOwn(PlayerID))
 				continue;
-			this.setShipIndex(gameState, entity);
+			this.setShipIndex(gameState, ent);
 		}
 	}
 
@@ -281,27 +296,27 @@ m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 			continue;
 		if (!evt.entityObj.hasClass("Ship") || !evt.metadata[PlayerID].transporter)
 			continue;
-		var plan = this.getPlan(evt.metadata[PlayerID].transporter);
+		let plan = this.getPlan(evt.metadata[PlayerID].transporter);
 		if (!plan)
 			continue;
 
-		var shipId = evt.entityObj.id();
+		let shipId = evt.entityObj.id();
 		if (this.Config.debug > 1)
 			API3.warn("one ship " + shipId + " from plan " + plan.ID + " destroyed during " + plan.state);
 		if (plan.state === "boarding")
 		{
 			// just reset the units onBoard metadata and wait for a new ship to be assigned to this plan
 			plan.units.forEach(function (ent) {
-				if ((ent.getMetadata(PlayerID, "onBoard") === "onBoard" && ent.position()) ||
-					ent.getMetadata(PlayerID, "onBoard") === shipId)
+				if (ent.getMetadata(PlayerID, "onBoard") === "onBoard" && ent.position() ||
+				    ent.getMetadata(PlayerID, "onBoard") === shipId)
 					ent.setMetadata(PlayerID, "onBoard", undefined);
 			});
-			plan.needTransportShips = (plan.transportShips.length === 0);
+			plan.needTransportShips = !plan.transportShips.hasEntities();
 		}
 		else if (plan.state === "sailing")
 		{
-			var endIndex = plan.endIndex;
-			var self = this;
+			let endIndex = plan.endIndex;
+			let self = this;
 			plan.units.forEach(function (ent) {
 				if (!ent.position())  // unit from another ship of this plan ... do nothing
 					return;
@@ -322,12 +337,11 @@ m.NavalManager.prototype.checkEvents = function(gameState, queues, events)
 
 	for (let evt of events.OwnershipChanged)	// capture events
 	{
-		if (evt.to === PlayerID)
-		{
-			let ent = gameState.getEntityById(evt.entity);
-			if (ent && ent.hasClass("Dock"))
-				this.setDockIndex(gameState, ent);
-		}
+		if (evt.to !== PlayerID)
+			continue;
+		let ent = gameState.getEntityById(evt.entity);
+		if (ent && (ent.hasClass("Dock") || ent.hasClass("Shipyard")))
+			this.setAccessIndices(gameState, ent);
 	}
 };
 
@@ -345,15 +359,20 @@ m.NavalManager.prototype.addPlan = function(plan)
 	this.transportPlans.push(plan);
 };
 
-// complete already existing plan or create a new one for this requirement
-// (many units can then call this separately and end up in the same plan)
-// TODO  check garrison classes
+/**
+ * complete already existing plan or create a new one for this requirement
+ * (many units can then call this separately and end up in the same plan)
+ * TODO  check garrison classes
+ */
 m.NavalManager.prototype.requireTransport = function(gameState, entity, startIndex, endIndex, endPos)
 {
+	if (!entity.canGarrison())
+		return false;
+
 	if (entity.getMetadata(PlayerID, "transport") !== undefined)
 	{
 		if (this.Config.debug > 0)
-			API3.warn("Petra naval manager error: unit " + entity.id() +  " has already required a transport");
+			API3.warn("Petra naval manager error: unit " + entity.id() + " has already required a transport");
 		return false;
 	}
 
@@ -378,12 +397,12 @@ m.NavalManager.prototype.requireTransport = function(gameState, entity, startInd
 	return true;
 };
 
-// split a transport plan in two, moving all entities not yet affected to a ship in the new plan
+/** split a transport plan in two, moving all entities not yet affected to a ship in the new plan */
 m.NavalManager.prototype.splitTransport = function(gameState, plan)
 {
 	if (this.Config.debug > 1)
 		API3.warn(">>>> split of transport plan started <<<<");
-	var newplan = new m.TransportPlan(gameState, [], plan.startIndex, plan.endIndex, plan.endPos);
+	let newplan = new m.TransportPlan(gameState, [], plan.startIndex, plan.endIndex, plan.endPos);
 	if (newplan.failed)
 	{
 		if (this.Config.debug > 1)
@@ -392,7 +411,7 @@ m.NavalManager.prototype.splitTransport = function(gameState, plan)
 	}
 	newplan.init(gameState);
 
-	var nbUnits = 0;
+	let nbUnits = 0;
 	plan.units.forEach(function (ent) {
 		if (ent.getMetadata(PlayerID, "onBoard"))
 			return;
@@ -403,7 +422,7 @@ m.NavalManager.prototype.splitTransport = function(gameState, plan)
 		API3.warn(">>>> previous plan left with units " + plan.units.length);
 	if (nbUnits)
 		this.transportPlans.push(newplan);
-	return (nbUnits !== 0);
+	return nbUnits !== 0;
 };
 
 /**
@@ -448,7 +467,7 @@ m.NavalManager.prototype.setMinimalTransportShips = function(gameState, sea, num
 // bumps up the number of ships we want if we need more.
 m.NavalManager.prototype.checkLevels = function(gameState, queues)
 {
-	if (queues.ships.length() !== 0)
+	if (queues.ships.hasQueuedUnits())
 		return;
 
 	for (let sea = 0; sea < this.neededTransportShips.length; sea++)
@@ -477,13 +496,12 @@ m.NavalManager.prototype.checkLevels = function(gameState, queues)
 
 m.NavalManager.prototype.maintainFleet = function(gameState, queues)
 {
-	if (queues.ships.length() > 0)
+	if (queues.ships.hasQueuedUnits())
 		return;
-	if (gameState.getOwnEntitiesByClass("Dock", true).filter(API3.Filters.isBuilt()).length +
-		gameState.getOwnEntitiesByClass("Shipyard", true).filter(API3.Filters.isBuilt()).length === 0)
+	if (!this.docks.filter(API3.Filters.isBuilt()).hasEntities())
 		return;
 	// check if we have enough transport ships per region.
-	for (var sea = 0; sea < this.seaShips.length; ++sea)
+	for (let sea = 0; sea < this.seaShips.length; ++sea)
 	{
 		if (this.seaShips[sea] === undefined)
 			continue;
@@ -492,7 +510,7 @@ m.NavalManager.prototype.maintainFleet = function(gameState, queues)
 
 		if (this.seaTransportShips[sea].length < this.wantedTransportShips[sea])
 		{
-			var template = this.getBestShip(gameState, sea, "transport");
+			let template = this.getBestShip(gameState, sea, "transport");
 			if (template)
 			{
 				queues.ships.addPlan(new m.TrainingPlan(gameState, template, { "sea": sea }, 1, 1));
@@ -513,7 +531,7 @@ m.NavalManager.prototype.maintainFleet = function(gameState, queues)
 	}
 };
 
-// assigns free ships to plans that need some
+/** assigns free ships to plans that need some */
 m.NavalManager.prototype.assignShipsToPlans = function(gameState)
 {
 	for (let plan of this.transportPlans)
@@ -521,15 +539,15 @@ m.NavalManager.prototype.assignShipsToPlans = function(gameState)
 			plan.assignShip(gameState);
 };
 
-// let blocking ships move apart from active ships (waiting for a better pathfinder)
+/** let blocking ships move apart from active ships (waiting for a better pathfinder) */
 m.NavalManager.prototype.moveApart = function(gameState)
 {
-	var self = this;
+	let self = this;
 
 	this.ships.forEach(function(ship) {
 		if (ship.hasClass("FishingBoat"))   // small ships should not be a problem
 			return;
-		var sea = ship.getMetadata(PlayerID, "sea");
+		let sea = ship.getMetadata(PlayerID, "sea");
 		if (ship.getMetadata(PlayerID, "transporter") === undefined)
 		{
 			if (ship.isIdle())	// do not stay idle near a dock to not disturb other ships
@@ -560,7 +578,7 @@ m.NavalManager.prototype.moveApart = function(gameState)
 	gameState.ai.HQ.tradeManager.traders.filter(API3.Filters.byClass("Ship")).forEach(function(ship) {
 		if (ship.getMetadata(PlayerID, "route") === undefined)
 			return;
-		var sea = ship.getMetadata(PlayerID, "sea");
+		let sea = ship.getMetadata(PlayerID, "sea");
 		self.seaShips[sea].forEach(function(blockingShip) {
 			if (blockingShip === ship || !blockingShip.isIdle())
 				return;
@@ -582,7 +600,7 @@ m.NavalManager.prototype.buildNavalStructures = function(gameState, queues)
 	if (gameState.getPopulation() > this.Config.Economy.popForDock)
 	{
 		if (queues.dock.countQueuedUnitsWithClass("NavalMarket") === 0 &&
-			gameState.getOwnStructures().filter(API3.Filters.and(API3.Filters.byClass("NavalMarket"), API3.Filters.isFoundation())).length === 0 &&
+			!gameState.getOwnStructures().filter(API3.Filters.and(API3.Filters.byClass("NavalMarket"), API3.Filters.isFoundation())).hasEntities() &&
 			gameState.ai.HQ.canBuild(gameState, "structures/{civ}_dock"))
 		{
 			let dockStarted = false;
@@ -608,43 +626,36 @@ m.NavalManager.prototype.buildNavalStructures = function(gameState, queues)
 	}
 
 	if (gameState.currentPhase() < 2 || gameState.getPopulation() < this.Config.Economy.popForTown + 15 ||
-		queues.militaryBuilding.length() !== 0 || this.bNaval.length === 0)
+	    queues.militaryBuilding.hasQueuedUnits())
 		return;
-	var docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock"));
-	if (!docks.length)
+	if (!this.docks.filter(API3.Filters.byClass("Dock")).hasEntities() ||
+	     this.docks.filter(API3.Filters.byClass("Shipyard")).hasEntities())
 		return;
-	var nNaval = 0;
-	for (let naval of this.bNaval)
-		nNaval += gameState.countEntitiesAndQueuedByType(naval, true);
-
-	if (nNaval === 0 || (nNaval < this.bNaval.length && gameState.getPopulation() > 120))
-	{
-		for (let naval of this.bNaval)
-		{
-			if (gameState.countEntitiesAndQueuedByType(naval, true) < 1 && gameState.ai.HQ.canBuild(gameState, naval))
-			{
-				let wantedLand = {};
-				for (let base of gameState.ai.HQ.baseManagers)
-					if (base.anchor)
-						wantedLand[base.accessIndex] = true;
-				let sea = docks.toEntityArray()[0].getMetadata(PlayerID, "sea");
-				queues.militaryBuilding.addPlan(new m.ConstructionPlan(gameState, naval, { "land": wantedLand, "sea": sea }));
-				break;
-			}
-		}
-	}
+	let template;
+	if (gameState.ai.HQ.canBuild(gameState, "structures/{civ}_super_dock"))
+		template = "structures/{civ}_super_dock";
+	else if (gameState.ai.HQ.canBuild(gameState, "structures/{civ}_shipyard"))
+		template = "structures/{civ}_shipyard";
+	else
+		return;
+	let wantedLand = {};
+	for (let base of gameState.ai.HQ.baseManagers)
+		if (base.anchor)
+			wantedLand[base.accessIndex] = true;
+	let sea = this.docks.toEntityArray()[0].getMetadata(PlayerID, "sea");
+	queues.militaryBuilding.addPlan(new m.ConstructionPlan(gameState, template, { "land": wantedLand, "sea": sea }));
 };
 
-// goal can be either attack (choose ship with best arrowCount) or transport (choose ship with best capacity)
+/** goal can be either attack (choose ship with best arrowCount) or transport (choose ship with best capacity) */
 m.NavalManager.prototype.getBestShip = function(gameState, sea, goal)
 {
-	var civ = gameState.civ();
-	var trainableShips = [];
+	let civ = gameState.getPlayerCiv();
+	let trainableShips = [];
 	gameState.getOwnTrainingFacilities().filter(API3.Filters.byMetadata(PlayerID, "sea", sea)).forEach(function(ent) {
-		var trainables = ent.trainableEntities(civ);
+		let trainables = ent.trainableEntities(civ);
 		for (let trainable of trainables)
 		{
-			if (gameState.isDisabledTemplates(trainable))
+			if (gameState.isTemplateDisabled(trainable))
 				continue;
 			let template = gameState.getTemplate(trainable);
 			if (template && template.hasClass("Ship") && trainableShips.indexOf(trainable) === -1)
@@ -652,10 +663,10 @@ m.NavalManager.prototype.getBestShip = function(gameState, sea, goal)
 		}
 	});
 
-	var best = 0;
-	var bestShip;
-	var limits = gameState.getEntityLimits();
-	var current = gameState.getEntityCounts();
+	let best = 0;
+	let bestShip;
+	let limits = gameState.getEntityLimits();
+	let current = gameState.getEntityCounts();
 	for (let trainable of trainableShips)
 	{
 		let template = gameState.getTemplate(trainable);
