@@ -84,8 +84,8 @@ m.QueueManager.prototype.wantedGatherRates = function(gameState)
 	if (gameState.ai.playedTurn === 0)
 	{
 		let ret = {};
-		for (let res of gameState.sharedScript.resourceList)
-			ret[res] = (res === "food" || res === "wood" ) ? 10 : 0;
+		for (let res of gameState.sharedScript.resourceInfo.codes)
+			ret[res] = this.Config.queues.firstTurn[res] || this.Config.queues.firstTurn.default;
 		return ret;
 	}
 
@@ -97,11 +97,11 @@ m.QueueManager.prototype.wantedGatherRates = function(gameState)
 	let totalShort = {};
 	let totalMedium = {};
 	let totalLong = {};
-	for (let res of gameState.sharedScript.resourceList)
+	for (let res of gameState.sharedScript.resourceInfo.codes)
 	{
-		totalShort[res] = (res === "food" || res === "wood" ) ? 200 : 100;
-		totalMedium[res] = 0;
-		totalLong[res] = 0;
+		totalShort[res] = this.Config.queues.short[res] || this.Config.queues.short.default;
+		totalMedium[res] = this.Config.queues.medium[res] || this.Config.queues.medium.default;
+		totalLong[res] = this.Config.queues.long[res] || this.Config.queues.long.default;
 	}
 	let total;
 	//queueArrays because it's faster.
@@ -133,7 +133,7 @@ m.QueueManager.prototype.wantedGatherRates = function(gameState)
 	// global rates
 	let rates = {};
 	let diff;
-	for (let res of gameState.sharedScript.resourceList)
+	for (let res of gameState.sharedScript.resourceInfo.codes)
 	{
 		if (current[res] > 0)
 		{
@@ -282,7 +282,7 @@ m.QueueManager.prototype.distributeResources = function(gameState)
 			}
 			else if (this.accounts[j][res] > queueCost[res])
 			{
-				availableRes[res] += (this.accounts[j][res] - queueCost[res]);
+				availableRes[res] += this.accounts[j][res] - queueCost[res];
 				this.accounts[j][res] = queueCost[res];
 			}
 		}
@@ -408,7 +408,7 @@ m.QueueManager.prototype.update = function(gameState)
 
 	if (this.Config.debug > 1 && gameState.ai.playedTurn%50 === 0)
 		this.printQueues(gameState);
-	
+
 	Engine.ProfileStop();
 };
 
@@ -421,16 +421,16 @@ m.QueueManager.prototype.checkPausedQueues = function(gameState)
 	{
 		let toBePaused = false;
 		if (gameState.ai.HQ.numActiveBase() === 0)
-			toBePaused = (q !== "dock" && q !== "civilCentre");
+			toBePaused = q !== "dock" && q !== "civilCentre";
 		else if (numWorkers < workersMin / 3)
-			toBePaused = (q !== "citizenSoldier" && q !== "villager" && q !== "emergency");
+			toBePaused = q !== "citizenSoldier" && q !== "villager" && q !== "emergency";
 		else if (numWorkers < workersMin * 2 / 3)
-			toBePaused = (q === "civilCentre" || q === "economicBuilding" ||
-				q === "militaryBuilding" || q === "defenseBuilding" ||
-				q === "majorTech" || q === "minorTech" || q.indexOf("plan_") !== -1);
+			toBePaused = q === "civilCentre" || q === "economicBuilding" ||
+				q === "militaryBuilding" || q === "defenseBuilding" || q === "healer" ||
+				q === "majorTech" || q === "minorTech" || q.indexOf("plan_") !== -1;
 		else if (numWorkers < workersMin)
-			toBePaused = (q === "civilCentre" || q === "defenseBuilding" ||
-				q == "majorTech" || q.indexOf("_siege") != -1 || q.indexOf("_champ") != -1);
+			toBePaused = q === "civilCentre" || q === "defenseBuilding" ||
+				q == "majorTech" || q.indexOf("_siege") != -1 || q.indexOf("_champ") != -1;
 
 		if (toBePaused)
 		{
@@ -533,7 +533,7 @@ m.QueueManager.prototype.removeQueue = function(queueName)
 	delete this.queues[queueName];
 	delete this.priorities[queueName];
 	delete this.accounts[queueName];
-		
+
 	this.queueArrays = [];
 	for (let q in this.queues)
 		this.queueArrays.push([q, this.queues[q]]);
