@@ -35,7 +35,7 @@ m.TradeManager.prototype.assignTrader = function(ent)
 
 m.TradeManager.prototype.trainMoreTraders = function(gameState, queues)
 {
-	if (!this.tradeRoute || queues.trader.hasQueuedUnits())
+	if (!this.hasTradeRoute() || queues.trader.hasQueuedUnits())
 		return;
 
 	let numTraders = this.traders.length;
@@ -104,7 +104,8 @@ m.TradeManager.prototype.trainMoreTraders = function(gameState, queues)
 	if (!gameState.getTemplate(template))
 	{
 		if (this.Config.debug > 0)
-			API3.warn("Petra error: trying to train " + template + " for civ " + gameState.civ() + " but no template found.");
+			API3.warn("Petra error: trying to train " + template + " for civ " +
+			          gameState.getPlayerCiv() + " but no template found.");
 		return;
 	}
 	queues.trader.addPlan(new m.TrainingPlan(gameState, template, metadata, 1, 1));
@@ -112,7 +113,7 @@ m.TradeManager.prototype.trainMoreTraders = function(gameState, queues)
 
 m.TradeManager.prototype.updateTrader = function(gameState, ent)
 {
-	if (!this.tradeRoute || !ent.isIdle() || !ent.position())
+	if (!this.hasTradeRoute() || !ent.isIdle() || !ent.position())
 		return;
 	if (ent.getMetadata(PlayerID, "transport") !== undefined)
 		return;
@@ -156,7 +157,7 @@ m.TradeManager.prototype.setTradingGoods = function(gameState)
 	let tradingGoods = {};
 	for (let res in gameState.ai.HQ.wantedRates)
 		tradingGoods[res] = 0;
-	// first, try to anticipate future needs 
+	// first, try to anticipate future needs
 	let stocks = gameState.ai.HQ.getTotalResourceLevel(gameState);
 	let mostNeeded = gameState.ai.HQ.pickMostNeededResources(gameState);
 	let remaining = 100;
@@ -216,9 +217,9 @@ m.TradeManager.prototype.performBarter = function(gameState)
 
 	let rates = gameState.ai.HQ.GetCurrentGatherRates(gameState);
 
-	let prices = gameState.getBarterPrices();
+	let barterPrices = gameState.getBarterPrices();
 	// calculates conversion rates
-	let getBarterRate = function (prices,buy,sell) { return Math.round(100 * prices.sell[sell] / prices.buy[buy]); };
+	let getBarterRate = (prices, buy, sell) => Math.round(100 * prices.sell[sell] / prices.buy[buy]);
 
 	// loop through each missing resource checking if we could barter and help finishing a queue quickly.
 	for (let buy of needs.types)
@@ -247,7 +248,7 @@ m.TradeManager.prototype.performBarter = function(gameState)
 				else if (available[sell] > 1000)
 					barterRateMin = 10;
 			}
-			else 
+			else
 			{
 				barterRateMin = 70;
 				if (available[sell] > 1000)
@@ -256,7 +257,7 @@ m.TradeManager.prototype.performBarter = function(gameState)
 					barterRateMin += 20;
 			}
 
-			let barterRate = getBarterRate(prices, buy, sell);
+			let barterRate = getBarterRate(barterPrices, buy, sell);
 			if (barterRate > bestRate && barterRate > barterRateMin)
 			{
 				bestRate = barterRate;
@@ -286,7 +287,7 @@ m.TradeManager.prototype.performBarter = function(gameState)
 		let barterRateMin = 80;
 		if (available[buy] < 5000 && available.food > 5000)
 			barterRateMin -= 20 - Math.floor(available[buy]/250);
-		let barterRate = getBarterRate(prices, buy, "food");
+		let barterRate = getBarterRate(barterPrices, buy, "food");
 		if (barterRate < barterRateMin)
 			continue;
 		let choice = barterRate / (100 + available[buy]);
@@ -301,7 +302,7 @@ m.TradeManager.prototype.performBarter = function(gameState)
 		barterers[0].barter(bestToBuy, "food", 100);
 		if (this.Config.debug > 2)
 			API3.warn("Contingency bartering: sold food for " + bestToBuy + " available sell " + available.food +
-				  " available buy " + available[bestToBuy] + " barterRate " + getBarterRate(prices, bestToBuy, "food"));
+				  " available buy " + available[bestToBuy] + " barterRate " + getBarterRate(barterPrices, bestToBuy, "food"));
 		return true;
 	}
 
@@ -331,13 +332,13 @@ m.TradeManager.prototype.checkEvents = function(gameState, events)
 		}
 	}
 
-	// if one market is destroyed, we should look for a better route
+	// if one market (or market-foundation) is destroyed, we should look for a better route
 	for (let evt of events.Destroy)
 	{
 		if (!evt.entityObj)
 			continue;
 		let ent = evt.entityObj;
-		if (!ent || ent.foundationProgress() !== undefined || !ent.hasClass("Market") || !gameState.isPlayerAlly(ent.owner()))
+		if (!ent || !ent.hasClass("Market") || !gameState.isPlayerAlly(ent.owner()))
 			continue;
 		this.routeProspection = true;
 		gameState.ai.HQ.restartBuild(gameState, "structures/{civ}_market");
@@ -372,17 +373,26 @@ m.TradeManager.prototype.checkEvents = function(gameState, events)
 		return true;
 	}
 
+	// or if diplomacy changed
+	if (events.DiplomacyChanged.length)
+	{
+		this.routeProspection = true;
+		gameState.ai.HQ.restartBuild(gameState, "structures/{civ}_market");
+		gameState.ai.HQ.restartBuild(gameState, "structures/{civ}_dock");
+		return true;
+	}
+
 	return false;
 };
 
 /**
  * fills the best trade route in this.tradeRoute and the best potential route in this.potentialTradeRoute
- * If an index is given, it returns the best route with this index or the best land route if index is a land index 
+ * If an index is given, it returns the best route with this index or the best land route if index is a land index
  */
 m.TradeManager.prototype.checkRoutes = function(gameState, accessIndex)
 {
-	let market1 = gameState.updatingCollection("OwnMarkets", API3.Filters.byClass("Market"), gameState.getOwnStructures()).toEntityArray();
-	let market2 = gameState.updatingCollection("ExclusiveAllyMarkets", API3.Filters.byClass("Market"), gameState.getExclusiveAllyEntities()).toEntityArray();
+	let market1 = gameState.updatingCollection("OwnMarkets", API3.Filters.byClass("Market"), gameState.getOwnStructures());
+	let market2 = gameState.updatingCollection("ExclusiveAllyMarkets", API3.Filters.byClass("Market"), gameState.getExclusiveAllyEntities());
 	if (market1.length + market2.length < 2)  // We have to wait  ... markets will be built soon
 	{
 		this.tradeRoute = undefined;
@@ -390,7 +400,8 @@ m.TradeManager.prototype.checkRoutes = function(gameState, accessIndex)
 		return false;
 	}
 
-	if (!market2.length)
+	let onlyOurs = !market2.hasEntities();
+	if (onlyOurs)
 		market2 = market1;
 	let candidate = { "gain": 0 };
 	let potential = { "gain": 0 };
@@ -399,24 +410,26 @@ m.TradeManager.prototype.checkRoutes = function(gameState, accessIndex)
 
 	let traderTemplatesGains = gameState.getTraderTemplatesGains();
 
-	for (let m1 of market1)
+	for (let m1 of market1.values())
 	{
 		if (!m1.position())
 			continue;
-		let access1 = gameState.ai.accessibility.getAccessValue(m1.position());
-		let sea1 = m1.hasClass("NavalMarket") ? gameState.ai.HQ.navalManager.getDockIndex(gameState, m1, true) : undefined;
-		for (let m2 of market2)
+		let access1 = m.getLandAccess(gameState, m1);
+		let sea1 = m1.hasClass("NavalMarket") ? m.getSeaAccess(gameState, m1) : undefined;
+		for (let m2 of market2.values())
 		{
-			if (m1.id() === m2.id())
+			if (onlyOurs && m1.id() >= m2.id())
 				continue;
 			if (!m2.position())
 				continue;
-			let access2 = gameState.ai.accessibility.getAccessValue(m2.position());
-			let sea2 = m2.hasClass("NavalMarket") ? gameState.ai.HQ.navalManager.getDockIndex(gameState, m2, true) : undefined;
+			let access2 = m.getLandAccess(gameState, m2);
+			let sea2 = m2.hasClass("NavalMarket") ? m.getSeaAccess(gameState, m2) : undefined;
 			let land = access1 == access2 ? access1 : undefined;
-			let sea = (sea1 && sea1 == sea2) ? sea1 : undefined;
+			let sea = sea1 && sea1 == sea2 ? sea1 : undefined;
 			if (!land && !sea)
 				continue;
+			if (land && m.isLineInsideEnemyTerritory(gameState, m1.position(), m2.position()))
+				continue;
 			let gainMultiplier;
 			if (land && traderTemplatesGains.landGainMultiplier)
 				gainMultiplier = traderTemplatesGains.landGainMultiplier;
@@ -527,8 +540,18 @@ m.TradeManager.prototype.checkTrader = function(gameState, ent)
 		(possibleRoute.target.id() != presentRoute.source && possibleRoute.target.id() != presentRoute.target))
 	{
 		// Trader will be assigned in updateTrader
-		ent.stopMoving();
 		ent.setMetadata(PlayerID, "route", undefined);
+		if (!possibleRoute && !ent.hasClass("Ship"))
+		{
+			let closestBase = m.getBestBase(gameState, ent, true);
+			if (closestBase.accessIndex == access)
+			{
+				let closestBasePos = closestBase.anchor.position();
+				ent.moveToRange(closestBasePos[0], closestBasePos[1], 0, 15);
+				return;
+			}
+		}
+		ent.stopMoving();
 	}
 };
 
@@ -569,7 +592,7 @@ m.TradeManager.prototype.prospectForNewMarket = function(gameState, queues)
 		gameState.ai.queueManager.changePriority("economicBuilding", 2*this.Config.priorities.economicBuilding);
 	let plan = new m.ConstructionPlan(gameState, "structures/{civ}_market");
 	if (!this.tradeRoute)
-		plan.onStart = function(gameState) { gameState.ai.queueManager.changePriority("economicBuilding", gameState.ai.Config.priorities.economicBuilding); };
+		plan.queueToReset = "economicBuilding";
 	queues.economicBuilding.addPlan(plan);
 };
 
@@ -583,22 +606,24 @@ m.TradeManager.prototype.isNewMarketWorth = function(expectedGain)
 
 m.TradeManager.prototype.update = function(gameState, events, queues)
 {
-	this.performBarter(gameState);
+	if (gameState.ai.HQ.canBarter)
+		this.performBarter(gameState);
+
+	if (this.Config.difficulty <= 1)
+		return;
 
 	if (this.routeProspection)
 		this.prospectForNewMarket(gameState, queues);
 
-	let self = this;
-
 	if (this.checkEvents(gameState, events))  // true if one market was built or destroyed
 	{
-		this.traders.forEach(function(ent) { self.checkTrader(gameState, ent); });
+		this.traders.forEach(ent => { this.checkTrader(gameState, ent); });
 		this.checkRoutes(gameState);
 	}
 
 	if (this.tradeRoute)
 	{
-		this.traders.forEach(function(ent) { self.updateTrader(gameState, ent); });
+		this.traders.forEach(ent => { this.updateTrader(gameState, ent); });
 		if (gameState.ai.playedTurn % 5 === 0)
 			this.trainMoreTraders(gameState, queues);
 		if (gameState.ai.playedTurn % 20 === 0 && this.traders.length >= 2)
@@ -614,7 +639,7 @@ m.TradeManager.prototype.routeEntToId = function(route)
 		return route;
 	let ret = {};
 	for (let key in route)
-		ret[key] = (key == "source" || key == "target") ? route[key].id() : route[key];
+		ret[key] = key == "source" || key == "target" ? route[key].id() : route[key];
 	return ret;
 };
 
@@ -624,7 +649,7 @@ m.TradeManager.prototype.routeIdToEnt = function(gameState, route)
 		return route;
 	let ret = {};
 	for (let key in route)
-		ret[key] = (key == "source" || key == "target") ? gameState.getEntityById(route[key]) : route[key];
+		ret[key] = key == "source" || key == "target" ? gameState.getEntityById(route[key]) : route[key];
 	return ret;
 };
 
