@@ -39,23 +39,82 @@ m.Worker.prototype.update = function(gameState, ent)
 	this.ent = ent;
 
 	let unitAIState = ent.unitAIState();
-	if (unitAIState === "INDIVIDUAL.GATHER.GATHERING" || unitAIState === "INDIVIDUAL.GATHER.APPROACHING" ||
-	    unitAIState === "INDIVIDUAL.COMBAT.APPROACHING")
+	if ((subrole === "hunter" || subrole === "gatherer") &&
+	    (unitAIState === "INDIVIDUAL.GATHER.GATHERING" || unitAIState === "INDIVIDUAL.GATHER.APPROACHING" ||
+	     unitAIState === "INDIVIDUAL.COMBAT.APPROACHING"))
 	{
-		if (this.isInaccessibleSupply(gameState) && ((subrole === "hunter" && !this.startHunting(gameState)) ||
-			(subrole === "gatherer" && !this.startGathering(gameState))))
+		if (this.isInaccessibleSupply(gameState) && !this.retryGathering(gameState, subrole))
 			ent.stopMoving();
+
+		// Check that we have not drifted too far
+		if (unitAIState === "INDIVIDUAL.COMBAT.APPROACHING" && ent.unitAIOrderData().length)
+		{
+			let orderData = ent.unitAIOrderData()[0];
+			if (orderData && orderData.target)
+			{
+				let supply = gameState.getEntityById(orderData.target);
+				if (supply && supply.resourceSupplyType() && supply.resourceSupplyType().generic === "food")
+				{
+					let territoryOwner = gameState.ai.HQ.territoryMap.getOwner(supply.position());
+					if (gameState.isPlayerEnemy(territoryOwner) && !this.retryGathering(gameState, subrole))
+						ent.stopMoving();
+					else if (!gameState.isPlayerAlly(territoryOwner))
+					{
+						let distanceSquare = ent.hasClass("Cavalry") ? 90000 : 30000;
+						let supplyAccess = gameState.ai.accessibility.getAccessValue(supply.position());
+						let foodDropsites = gameState.playerData.hasSharedDropsites ?
+						                    gameState.getAnyDropsites("food") : gameState.getOwnDropsites("food");
+						let hasFoodDropsiteWithinDistance = false;
+						for (let dropsite of foodDropsites.values())
+						{
+							if (!dropsite.position())
+								continue;
+							let owner = dropsite.owner();
+							// owner !== PlayerID can only happen when hasSharedDropsites === true, so no need to test it again
+							if (owner !== PlayerID && (!dropsite.isSharedDropsite() || !gameState.isPlayerMutualAlly(owner)))
+								continue;
+							if (supplyAccess !== m.getLandAccess(gameState, dropsite))
+								continue;
+							if (API3.SquareVectorDistance(supply.position(), dropsite.position()) < distanceSquare)
+							{
+								hasFoodDropsiteWithinDistance = true;
+								break;
+							}
+						}
+						if (!hasFoodDropsiteWithinDistance && !this.retryGathering(gameState, subrole))
+							ent.stopMoving();
+					}
+				}
+			}
+		}
 	}
 	else if (ent.getMetadata(PlayerID, "approachingTarget"))
+	{
 		ent.setMetadata(PlayerID, "approachingTarget", undefined);
+		ent.setMetadata(PlayerID, "alreadyTried", undefined);
+	}
 
 	let unitAIStateOrder = unitAIState.split(".")[1];
 	// If we're fighting or hunting, let's not start gathering
 	// but for fishers where UnitAI must have made us target a moving whale.
+	// Also, if we are attacking, do not capture
 	if (unitAIStateOrder === "COMBAT")
 	{
 		if (subrole === "fisher")
 			this.startFishing(gameState);
+		else if (unitAIState === "INDIVIDUAL.COMBAT.ATTACKING" && ent.unitAIOrderData().length &&
+			!ent.getMetadata(PlayerID, "PartOfArmy"))
+		{
+			let orderData = ent.unitAIOrderData()[0];
+			if (orderData && orderData.target && orderData.attackType && orderData.attackType === "Capture")
+			{
+				// If we are here, an enemy structure must have targeted one of our workers
+				// and UnitAI sent it fight back with allowCapture=true
+				let target = gameState.getEntityById(orderData.target);
+				if (target && target.owner() > 0 && !gameState.isPlayerAlly(target.owner()))
+					ent.attack(orderData.target, m.allowCapture(gameState, ent, target));
+			}
+		}
 		return;
 	}
 
@@ -90,6 +149,7 @@ m.Worker.prototype.update = function(gameState, ent)
 				let supplyId = ent.unitAIOrderData()[0].target;
 				let supply = gameState.getEntityById(supplyId);
 				if (supply && !supply.hasClass("Field") && !supply.hasClass("Animal") &&
+					supply.resourceSupplyType().generic !== "treasure" &&
 					supplyId !== ent.getMetadata(PlayerID, "supply"))
 				{
 					let nbGatherers = supply.resourceSupplyNumGatherers() + gameState.ai.HQ.GetTCGatherer(supplyId);
@@ -136,7 +196,7 @@ m.Worker.prototype.update = function(gameState, ent)
 	{
 		if (unitAIStateOrder === "REPAIR")
 		{
-			// update our target in case UnitAI sent us to a different foundation because of autocontinue	
+			// update our target in case UnitAI sent us to a different foundation because of autocontinue
 			if (ent.unitAIOrderData()[0] && ent.unitAIOrderData()[0].target &&
 				ent.getMetadata(PlayerID, "target-foundation") !== ent.unitAIOrderData()[0].target)
 				ent.setMetadata(PlayerID, "target-foundation", ent.unitAIOrderData()[0].target);
@@ -167,14 +227,10 @@ m.Worker.prototype.update = function(gameState, ent)
 		else
 		{
 			let access = gameState.ai.accessibility.getAccessValue(ent.position());
-			let goalAccess = target.getMetadata(PlayerID, "access");
-			if (!goalAccess)
-			{
-				goalAccess = gameState.ai.accessibility.getAccessValue(target.position());
-				target.setMetadata(PlayerID, "access", goalAccess);
-			}
+			let goalAccess = m.getLandAccess(gameState, target);
+			let queued = m.returnResources(gameState, ent);
 			if (access === goalAccess)
-				ent.repair(target, target.hasClass("House"));  // autocontinue=true for houses
+				ent.repair(target, target.hasClass("House"), queued);  // autocontinue=true for houses
 			else
 				gameState.ai.HQ.navalManager.requireTransport(gameState, ent, access, goalAccess, target.position());
 		}
@@ -250,6 +306,21 @@ m.Worker.prototype.update = function(gameState, ent)
 	}
 };
 
+m.Worker.prototype.retryGathering = function(gameState, subrole)
+{
+	switch (subrole)
+	{
+	case "gatherer":
+		return this.startGathering(gameState);
+	case "hunter":
+		return this.startHunting(gameState);
+	case "fisher":
+		return this.startFishing(gameState);
+	default:
+		return false;
+	}
+};
+
 m.Worker.prototype.startGathering = function(gameState)
 {
 	let access = gameState.ai.accessibility.getAccessValue(this.ent.position());
@@ -302,7 +373,8 @@ m.Worker.prototype.startGathering = function(gameState)
 	// first look in our own base if accessible from our present position
 	if (this.accessIndex === access)
 	{
-		if ((supply = findSupply(this.ent, this.base.dropsiteSupplies[resource].nearby)))
+		supply = findSupply(this.ent, this.base.dropsiteSupplies[resource].nearby);
+		if (supply)
 		{
 			this.ent.gather(supply);
 			return true;
@@ -310,18 +382,21 @@ m.Worker.prototype.startGathering = function(gameState)
 		// --> for food, try to gather from fields if any, otherwise build one if any
 		if (resource === "food")
 		{
-			if ((supply = this.gatherNearestField(gameState, this.baseID)))
+			supply = this.gatherNearestField(gameState, this.baseID);
+			if (supply)
 			{
 				this.ent.gather(supply);
 				return true;
 			}
-			else if (this.ent.isBuilder() && (supply = this.buildAnyField(gameState, this.baseID)))
+			supply = this.buildAnyField(gameState, this.baseID);
+			if (supply)
 			{
 				this.ent.repair(supply);
 				return true;
 			}
 		}
-		if ((supply = findSupply(this.ent, this.base.dropsiteSupplies[resource].medium)))
+		supply = findSupply(this.ent, this.base.dropsiteSupplies[resource].medium);
+		if (supply)
 		{
 			this.ent.gather(supply);
 			return true;
@@ -335,7 +410,8 @@ m.Worker.prototype.startGathering = function(gameState)
 			continue;
 		if (base.accessIndex !== access)
 			continue;
-		if ((supply = findSupply(this.ent, base.dropsiteSupplies[resource].nearby)))
+		supply = findSupply(this.ent, base.dropsiteSupplies[resource].nearby);
+		if (supply)
 		{
 			this.ent.setMetadata(PlayerID, "base", base.ID);
 			this.ent.gather(supply);
@@ -350,13 +426,15 @@ m.Worker.prototype.startGathering = function(gameState)
 				continue;
 			if (base.accessIndex !== access)
 				continue;
-			if ((supply = this.gatherNearestField(gameState, base.ID)))
+			supply = this.gatherNearestField(gameState, base.ID);
+			if (supply)
 			{
 				this.ent.setMetadata(PlayerID, "base", base.ID);
 				this.ent.gather(supply);
 				return true;
 			}
-			if (this.ent.isBuilder() && (supply = this.buildAnyField(gameState, base.ID)))
+			supply = this.buildAnyField(gameState, base.ID);
+			if (supply)
 			{
 				this.ent.setMetadata(PlayerID, "base", base.ID);
 				this.ent.repair(supply);
@@ -370,7 +448,8 @@ m.Worker.prototype.startGathering = function(gameState)
 			continue;
 		if (base.accessIndex !== access)
 			continue;
-		if ((supply = findSupply(this.ent, base.dropsiteSupplies[resource].medium)))
+		supply = findSupply(this.ent, base.dropsiteSupplies[resource].medium);
+		if (supply)
 		{
 			this.ent.setMetadata(PlayerID, "base", base.ID);
 			this.ent.gather(supply);
@@ -402,14 +481,12 @@ m.Worker.prototype.startGathering = function(gameState)
 	{
 		if (base.accessIndex === access)
 			continue;
-		if ((supply = findSupply(this.ent, base.dropsiteSupplies[resource].nearby)))
+		supply = findSupply(this.ent, base.dropsiteSupplies[resource].nearby);
+		if (supply && navalManager.requireTransport(gameState, this.ent, access, base.accessIndex, supply.position()))
 		{
-			if (navalManager.requireTransport(gameState, this.ent, access, base.accessIndex, supply.position()))
-			{
-				if (base.ID !== this.baseID)
-					this.ent.setMetadata(PlayerID, "base", base.ID);
-				return true;
-			}
+			if (base.ID !== this.baseID)
+				this.ent.setMetadata(PlayerID, "base", base.ID);
+			return true;
 		}
 	}
 	if (resource === "food")	// --> for food, try to gather from fields if any, otherwise build one if any
@@ -418,23 +495,19 @@ m.Worker.prototype.startGathering = function(gameState)
 		{
 			if (base.accessIndex === access)
 				continue;
-			if ((supply = this.gatherNearestField(gameState, base.ID)))
+			supply = this.gatherNearestField(gameState, base.ID);
+			if (supply && navalManager.requireTransport(gameState, this.ent, access, base.accessIndex, supply.position()))
 			{
-				if (navalManager.requireTransport(gameState, this.ent, access, base.accessIndex, supply.position()))
-				{
-					if (base.ID !== this.baseID)
-						this.ent.setMetadata(PlayerID, "base", base.ID);
-					return true;
-				}
+				if (base.ID !== this.baseID)
+					this.ent.setMetadata(PlayerID, "base", base.ID);
+				return true;
 			}
-			if ((supply = this.buildAnyField(gameState, base.ID)))
+			supply = this.buildAnyField(gameState, base.ID);
+			if (supply && navalManager.requireTransport(gameState, this.ent, access, base.accessIndex, supply.position()))
 			{
-				if (navalManager.requireTransport(gameState, this.ent, access, base.accessIndex, supply.position()))
-				{
-					if (base.ID !== this.baseID)
-						this.ent.setMetadata(PlayerID, "base", base.ID);
-					return true;
-				}
+				if (base.ID !== this.baseID)
+					this.ent.setMetadata(PlayerID, "base", base.ID);
+				return true;
 			}
 		}
 	}
@@ -442,14 +515,12 @@ m.Worker.prototype.startGathering = function(gameState)
 	{
 		if (base.accessIndex === access)
 			continue;
-		if ((supply = findSupply(this.ent, base.dropsiteSupplies[resource].medium)))
+		supply = findSupply(this.ent, base.dropsiteSupplies[resource].medium);
+		if (supply && navalManager.requireTransport(gameState, this.ent, access, base.accessIndex, supply.position()))
 		{
-			if (navalManager.requireTransport(gameState, this.ent, access, base.accessIndex, supply.position()))
-			{
-				if (base.ID !== this.baseID)
-					this.ent.setMetadata(PlayerID, "base", base.ID);
-				return true;
-			}
+			if (base.ID !== this.baseID)
+				this.ent.setMetadata(PlayerID, "base", base.ID);
+			return true;
 		}
 	}
 	// Okay so we haven't found any appropriate dropsite anywhere.
@@ -459,12 +530,7 @@ m.Worker.prototype.startGathering = function(gameState)
 			return false;
 		if (foundation.resourceDropsiteTypes() && foundation.resourceDropsiteTypes().indexOf(resource) !== -1)
 		{
-			let foundationAccess = foundation.getMetadata(PlayerID, "access");
-			if (!foundationAccess)
-			{
-				foundationAccess = gameState.ai.accessibility.getAccessValue(foundation.position());
-				foundation.setMetadata(PlayerID, "access", foundationAccess);
-			}
+			let foundationAccess = m.getLandAccess(gameState, foundation);
 			if (navalManager.requireTransport(gameState, this.ent, access, foundationAccess, foundation.position()))
 			{
 				if (foundation.getMetadata(PlayerID, "base") !== this.baseID)
@@ -479,34 +545,47 @@ m.Worker.prototype.startGathering = function(gameState)
 		return true;
 
 	// Still nothing, we look now for faraway resources, first in the accessible ones, then in the others
-	if (this.accessIndex === access)
+	// except for food when farms or corrals can be used
+	let allowDistant = true;
+	if (resource === "food")
 	{
-		if ((supply = findSupply(this.ent, this.base.dropsiteSupplies[resource].faraway)))
-		{
-			this.ent.gather(supply);
-			return true;
-		}
+		if (gameState.ai.HQ.turnCache.allowDistantFood === undefined)
+			gameState.ai.HQ.turnCache.allowDistantFood =
+				!gameState.ai.HQ.canBuild(gameState, "structures/{civ}_field") &&
+				!gameState.ai.HQ.canBuild(gameState, "structures/{civ}_corral");
+		allowDistant = gameState.ai.HQ.turnCache.allowDistantFood;
 	}
-	for (let base of gameState.ai.HQ.baseManagers)
+	if (allowDistant)
 	{
-		if (base.ID === this.baseID)
-			continue;
-		if (base.accessIndex !== access)
-			continue;
-		if ((supply = findSupply(this.ent, base.dropsiteSupplies[resource].faraway)))
+		if (this.accessIndex === access)
 		{
-			this.ent.setMetadata(PlayerID, "base", base.ID);
-			this.ent.gather(supply);
-			return true;
+			supply = findSupply(this.ent, this.base.dropsiteSupplies[resource].faraway);
+			if (supply)
+			{
+				this.ent.gather(supply);
+				return true;
+			}
 		}
-	}
-	for (let base of gameState.ai.HQ.baseManagers)
-	{
-		if (base.accessIndex === access)
-			continue;
-		if ((supply = findSupply(this.ent, base.dropsiteSupplies[resource].faraway)))
+		for (let base of gameState.ai.HQ.baseManagers)
 		{
-			if (navalManager.requireTransport(gameState, this.ent, access, base.accessIndex, supply.position()))
+			if (base.ID === this.baseID)
+				continue;
+			if (base.accessIndex !== access)
+				continue;
+			supply = findSupply(this.ent, base.dropsiteSupplies[resource].faraway);
+			if (supply)
+			{
+				this.ent.setMetadata(PlayerID, "base", base.ID);
+				this.ent.gather(supply);
+				return true;
+			}
+		}
+		for (let base of gameState.ai.HQ.baseManagers)
+		{
+			if (base.accessIndex === access)
+				continue;
+			supply = findSupply(this.ent, base.dropsiteSupplies[resource].faraway);
+			if (supply && navalManager.requireTransport(gameState, this.ent, access, base.accessIndex, supply.position()))
 			{
 				if (base.ID !== this.baseID)
 					this.ent.setMetadata(PlayerID, "base", base.ID);
@@ -533,7 +612,7 @@ m.Worker.prototype.startHunting = function(gameState, position)
 	if (!position && this.gatherTreasure(gameState))
 		return true;
 
-	let resources = gameState.getHuntableSupplies();	
+	let resources = gameState.getHuntableSupplies();
 	if (!resources.hasEntities())
 		return false;
 
@@ -544,12 +623,12 @@ m.Worker.prototype.startHunting = function(gameState, position)
 	let isRanged = this.ent.hasClass("Ranged");
 	let entPosition = position ? position : this.ent.position();
 	let access = gameState.ai.accessibility.getAccessValue(entPosition);
-	let foodDropsites = (gameState.playerData.hasSharedDropsites ? gameState.getAnyDropsites("food") : gameState.getOwnDropsites("food")).toEntityArray();
+	let foodDropsites = gameState.playerData.hasSharedDropsites ?
+	                    gameState.getAnyDropsites("food") : gameState.getOwnDropsites("food");
 
-	let nearestDropsiteDist = function(supply) {
-		let distMin = 1000000;
-		let pos = supply.position();
-		for (let dropsite of foodDropsites)
+	let hasFoodDropsiteWithinDistance = function(supplyPosition, supplyAccess, distSquare)
+	{
+		for (let dropsite of foodDropsites.values())
 		{
 			if (!dropsite.position())
 				continue;
@@ -557,17 +636,12 @@ m.Worker.prototype.startHunting = function(gameState, position)
 			// owner !== PlayerID can only happen when hasSharedDropsites === true, so no need to test it again
 			if (owner !== PlayerID && (!dropsite.isSharedDropsite() || !gameState.isPlayerMutualAlly(owner)))
 				continue;
-			let dropsiteAccess = dropsite.getMetadata(PlayerID, "access");
-			if (!dropsiteAccess)
-			{
-				dropsiteAccess = gameState.ai.accessibility.getAccessValue(dropsite.position());
-				dropsite.setMetadata(PlayerID, "access", dropsiteAccess);
-			}
-			if (dropsiteAccess !== access)
+			if (supplyAccess !== m.getLandAccess(gameState, dropsite))
 				continue;
-			distMin = Math.min(distMin, API3.SquareVectorDistance(pos, dropsite.position()));
+			if (API3.SquareVectorDistance(supplyPosition, dropsite.position()) < distSquare)
+				return true;
 		}
-		return distMin;
+		return false;
 	};
 
 	resources.forEach(function(supply)
@@ -587,7 +661,7 @@ m.Worker.prototype.startHunting = function(gameState, position)
 			return;
 
 		let canFlee = !supply.hasClass("Domestic") && supply.templateName().indexOf("resource|") == -1;
-		// Only cavalry and range units should hunt fleeing animals 
+		// Only cavalry and range units should hunt fleeing animals
 		if (canFlee && !isCavalry && !isRanged)
 			return;
 
@@ -608,12 +682,15 @@ m.Worker.prototype.startHunting = function(gameState, position)
 		let territoryOwner = gameState.ai.HQ.territoryMap.getOwner(supply.position());
 		if (territoryOwner !== 0 && !gameState.isPlayerAlly(territoryOwner))  // player is its own ally
 			return;
-
-		let dropsiteDist = nearestDropsiteDist(supply);
-		if (dropsiteDist > 35000)
+		// And if in ally territory, don't hunt this ally's cattle
+		if (territoryOwner !== 0 && territoryOwner !== PlayerID && supply.owner() === territoryOwner)
 			return;
+
 		// Only cavalry should hunt far from dropsite (specially for non domestic animals which flee)
-		if (!isCavalry && (dropsiteDist > 12000 || ((dropsiteDist > 7000 || territoryOwner === 0 ) && canFlee)))
+ 		if (!isCavalry && canFlee && territoryOwner === 0)
+			return;
+		let distanceSquare = isCavalry ? 35000 : ( canFlee ? 7000 : 12000);
+		if (!hasFoodDropsiteWithinDistance(supply.position(), supplyAccess, distanceSquare))
 			return;
 
 		nearestSupplyDist = dist;
@@ -649,7 +726,6 @@ m.Worker.prototype.startFishing = function(gameState)
 	let nearestSupplyDist = Math.min();
 	let nearestSupply;
 
-	let entPosition = this.ent.position();
 	let fisherSea = this.ent.getMetadata(PlayerID, "sea");
 	let fishDropsites = (gameState.playerData.hasSharedDropsites ? gameState.getAnyDropsites("food") : gameState.getOwnDropsites("food")).filter(API3.Filters.byClass("Dock")).toEntityArray();
 
@@ -664,13 +740,7 @@ m.Worker.prototype.startFishing = function(gameState)
 			// owner !== PlayerID can only happen when hasSharedDropsites === true, so no need to test it again
 			if (owner !== PlayerID && (!dropsite.isSharedDropsite() || !gameState.isPlayerMutualAlly(owner)))
 				continue;
-			let dropsiteSea = dropsite.getMetadata(PlayerID, "sea");
-			if (!dropsiteSea)
-			{
-				dropsiteSea = gameState.ai.accessibility.getAccessValue(dropsite.position(), true);
-				dropsite.setMetadata(PlayerID, "sea", dropsiteSea);
-			}
-			if (dropsiteSea !== fisherSea)
+			if (fisherSea !== m.getSeaAccess(gameState, dropsite))
 				continue;
 			distMin = Math.min(distMin, API3.SquareVectorDistance(pos, dropsite.position()));
 		}
@@ -684,9 +754,7 @@ m.Worker.prototype.startFishing = function(gameState)
 			return;
 
 		// check that it is accessible
-		if (!supply.getMetadata(PlayerID, "sea"))
-			supply.setMetadata(PlayerID, "sea", gameState.ai.accessibility.getAccessValue(supply.position(), true));
-		if (supply.getMetadata(PlayerID, "sea") !== fisherSea)
+		if (gameState.ai.HQ.navalManager.getFishSea(gameState, supply) !== fisherSea)
 			return;
 
 		exhausted = false;
@@ -698,18 +766,13 @@ m.Worker.prototype.startFishing = function(gameState)
 		if (nbGatherers > 0 && supply.resourceSupplyAmount()/(1+nbGatherers) < 30)
 			return;
 
-		// measure the distance to the resource
-		let dist = API3.SquareVectorDistance(entPosition, supply.position());
-		if (dist > nearestSupplyDist)
-			return;
-
 		// Avoid ennemy territory
-		let territoryOwner = gameState.ai.HQ.territoryMap.getOwner(supply.position());
-		if (territoryOwner !== 0 && !gameState.isPlayerAlly(territoryOwner))  // player is its own ally
+		if (!gameState.ai.HQ.navalManager.canFishSafely(gameState, supply))
 			return;
 
-		let dropsiteDist = nearestDropsiteDist(supply);
-		if (dropsiteDist > 35000)
+		// measure the distance from the resource to the nearest dropsite
+		let dist = nearestDropsiteDist(supply);
+		if (dist > nearestSupplyDist)
 			return;
 
 		nearestSupplyDist = dist;
@@ -763,11 +826,13 @@ m.Worker.prototype.gatherNearestField = function(gameState, baseID)
 
 /**
  * WARNING with the present options of AI orders, the unit will not gather after building the farm.
- * This is done by calling the gatherNearestField function when construction is completed. 
+ * This is done by calling the gatherNearestField function when construction is completed.
  */
 m.Worker.prototype.buildAnyField = function(gameState, baseID)
 {
-	let baseFoundations = gameState.getOwnFoundations().filter(API3.Filters.byMetadata(PlayerID, "base", baseID));	
+	if (!this.ent.isBuilder())
+		return false;
+	let baseFoundations = gameState.getOwnFoundations().filter(API3.Filters.byMetadata(PlayerID, "base", baseID));
 	let maxGatherers = gameState.getTemplate(gameState.applyCiv("structures/{civ}_field")).maxGatherers();
 	let bestFarmEnt = false;
 	let bestFarmDist = 10000000;
@@ -801,17 +866,13 @@ m.Worker.prototype.gatherTreasure = function(gameState)
 	let access = gameState.ai.accessibility.getAccessValue(this.ent.position());
 	for (let treasure of gameState.ai.HQ.treasures.values())
 	{
+		if (m.IsSupplyFull(gameState, treasure))
+			continue;
 		// let some time for the previous gatherer to reach the treasure befor trying again
 		let lastGathered = treasure.getMetadata(PlayerID, "lastGathered");
 		if (lastGathered && gameState.ai.elapsedTime - lastGathered < 20)
 			continue;
-		let treasureAccess = treasure.getMetadata(PlayerID, "access");
-		if (!treasureAccess)
-		{
-			treasureAccess = gameState.ai.accessibility.getAccessValue(treasure.position());
-			treasure.setMetadata(PlayerID, "access", treasureAccess);
-		}
-		if (treasureAccess !== access)
+		if (access !== m.getLandAccess(gameState, treasure))
 			continue;
 		let territoryOwner = gameState.ai.HQ.territoryMap.getOwner(treasure.position());
 		if (territoryOwner !== 0 && !gameState.isPlayerAlly(territoryOwner))
@@ -872,22 +933,29 @@ m.Worker.prototype.isInaccessibleSupply = function(gameState)
 	if (!target)
 		return true;
 
+	if (!target.resourceSupplyType())
+		return false;
+
 	let approachingTarget = this.ent.getMetadata(PlayerID, "approachingTarget");
+	let carriedAmount = this.ent.resourceCarrying().length ? this.ent.resourceCarrying()[0].amount : 0;
 	if (!approachingTarget || approachingTarget !== targetId)
 	{
 		this.ent.setMetadata(PlayerID, "approachingTarget", targetId);
 		this.ent.setMetadata(PlayerID, "approachingTime", undefined);
 		this.ent.setMetadata(PlayerID, "approachingPos", undefined);
-		this.ent.setMetadata(PlayerID, "carriedAmount", undefined);
+		this.ent.setMetadata(PlayerID, "carriedBefore", carriedAmount);
+		let alreadyTried = this.ent.getMetadata(PlayerID, "alreadyTried");
+		if (alreadyTried && alreadyTried !== targetId)
+			this.ent.setMetadata(PlayerID, "alreadyTried", undefined);
 	}
 
-	let carriedAmount = this.ent.resourceCarrying().length ? this.ent.resourceCarrying()[0].amount : 0;
-	if (this.ent.getMetadata(PlayerID, "carriedAmount") === undefined ||
-		this.ent.getMetadata(PlayerID, "carriedAmount") !== carriedAmount)
+	let carriedBefore = this.ent.getMetadata(PlayerID, "carriedBefore");
+	if (carriedBefore !== carriedAmount)
 	{
-		this.ent.setMetadata(PlayerID, "carriedAmount", carriedAmount);
-		this.ent.setMetadata(PlayerID, "approachingTime", undefined);
-		this.ent.setMetadata(PlayerID, "approachingPos", undefined);
+		this.ent.setMetadata(PlayerID, "approachingTarget", undefined);
+		this.ent.setMetadata(PlayerID, "alreadyTried", undefined);
+		if (target.getMetadata(PlayerID, "inaccessibleTime"))
+			target.setMetadata(PlayerID, "inaccessibleTime", 0);
 		return false;
 	}
 
@@ -904,12 +972,20 @@ m.Worker.prototype.isInaccessibleSupply = function(gameState)
 		{
 			this.ent.setMetadata(PlayerID, "approachingTime", gameState.ai.elapsedTime);
 			this.ent.setMetadata(PlayerID, "approachingPos", presentPos);
+			return false;
 		}
-		else if (gameState.ai.elapsedTime - approachingTime > 10)
+		if (gameState.ai.elapsedTime - approachingTime > 10)
 		{
-
-			target.setMetadata(PlayerID, "inaccessibleTime", gameState.ai.elapsedTime + 600);
-			return true;
+			if (this.ent.getMetadata(PlayerID, "alreadyTried"))
+			{
+				target.setMetadata(PlayerID, "inaccessibleTime", gameState.ai.elapsedTime + 600);
+				return true;
+			}
+			// let's try again to reach it
+			this.ent.setMetadata(PlayerID, "alreadyTried", targetId);
+			this.ent.setMetadata(PlayerID, "approachingTarget", undefined);
+			this.ent.gather(target);
+			return false;
 		}
 	}
 	return false;
