@@ -1,27 +1,91 @@
-const g_TooltipTextFormats = {
+var g_TooltipTextFormats = {
 	"unit": ['[font="sans-10"][color="orange"]', '[/color][/font]'],
 	"header": ['[font="sans-bold-13"]', '[/font]'],
 	"body": ['[font="sans-13"]', '[/font]'],
 	"comma": ['[font="sans-12"]', '[/font]']
 };
 
-const g_AttackTypes = {
+var g_AttackTypes = {
 	"Melee": translate("Melee Attack:"),
 	"Ranged": translate("Ranged Attack:"),
 	"Capture": translate("Capture Attack:")
 };
 
-const g_DamageTypes = {
+var g_DamageTypes = {
 	"hack": translate("Hack"),
 	"pierce": translate("Pierce"),
 	"crush": translate("Crush"),
 };
 
-function costIcon(resource)
+var g_SplashDamageTypes = {
+	"Circular": translate("Circular Splash Damage"),
+	"Linear": translate("Linear Splash Damage")
+};
+
+var g_RangeTooltipString = {
+	"relative": {
+		// Translation: For example: Ranged Attack: 12.0 Pierce, Range: 2 to 10 (+2) meters, Interval: 3 arrows / 2 seconds
+		"minRange": translate("%(attackLabel)s %(damageTypes)s, %(rangeLabel)s %(minRange)s to %(maxRange)s (%(relativeRange)s) %(rangeUnit)s, %(rate)s"),
+		// Translation: For example: Ranged Attack: 12.0 Pierce, Range: 10 (+2) meters, Interval: 3 arrows / 2 seconds
+		"no-minRange": translate("%(attackLabel)s %(damageTypes)s, %(rangeLabel)s %(maxRange)s (%(relativeRange)s) %(rangeUnit)s, %(rate)s"),
+	},
+	"non-relative": {
+		// Translation: For example: Ranged Attack: 12.0 Pierce, Range: 2 to 10 meters, Interval: 3 arrows / 2 seconds
+		"minRange": translate("%(attackLabel)s %(damageTypes)s, %(rangeLabel)s %(minRange)s to %(maxRange)s %(rangeUnit)s, %(rate)s"),
+		// Translation: For example: Ranged Attack: 12.0 Pierce, Range: 10 meters, Interval: 3 arrows / 2 seconds
+		"no-minRange": translate("%(attackLabel)s %(damageTypes)s, %(rangeLabel)s %(maxRange)s %(rangeUnit)s, %(rate)s"),
+	}
+};
+
+function getCostTypes()
+{
+	return g_ResourceData.GetCodes().concat(["population", "populationBonus", "time"]);
+}
+
+/**
+ * If true, always shows whether the splash damage deals friendly fire.
+ * Otherwise display the friendly fire tooltip only if it does.
+ */
+var g_AlwaysDisplayFriendlyFire = false;
+
+function resourceIcon(resource)
 {
 	return '[icon="icon_' + resource + '"]';
 }
 
+function resourceNameFirstWord(type)
+{
+	return translateWithContext("firstWord", g_ResourceData.GetNames()[type]);
+}
+
+function resourceNameWithinSentence(type)
+{
+	return translateWithContext("withinSentence", g_ResourceData.GetNames()[type]);
+}
+
+/**
+ * Format resource amounts to proper english and translate (for example: "200 food, 100 wood and 300 metal").
+ */
+function getLocalizedResourceAmounts(resources)
+{
+	let amounts = g_ResourceData.GetCodes()
+		.filter(type => !!resources[type])
+		.map(type => sprintf(translate("%(amount)s %(resourceType)s"), {
+			"amount": resources[type],
+			"resourceType": resourceNameWithinSentence(type)
+		}));
+
+	if (amounts.length < 2)
+		return amounts.join();
+
+	let lastAmount = amounts.pop();
+	return sprintf(translate("%(previousAmounts)s and %(lastAmount)s"), {
+		// Translation: This comma is used for separating first to penultimate elements in an enumeration.
+		"previousAmounts": amounts.join(translate(", ")),
+		"lastAmount": lastAmount
+	});
+}
+
 function bodyFont(text)
 {
 	return g_TooltipTextFormats.body[0] + text + g_TooltipTextFormats.body[1];
@@ -42,6 +106,14 @@ function commaFont(text)
 	return g_TooltipTextFormats.comma[0] + text + g_TooltipTextFormats.comma[1];
 }
 
+function getSecondsString(seconds)
+{
+	return sprintf(translatePlural("%(time)s %(second)s", "%(time)s %(second)s", seconds), {
+		"time": seconds,
+		"second": unitFont(translatePlural("second", "seconds", seconds))
+	});
+}
+
 function getEntityTooltip(template)
 {
 	if (!template.tooltip)
@@ -61,14 +133,22 @@ function getHealthTooltip(template)
 	});
 }
 
+function getCurrentHealthTooltip(entState, label)
+{
+	if (!entState.maxHitpoints)
+		return "";
+
+	return sprintf(translate("%(healthLabel)s %(current)s / %(max)s"), {
+		"healthLabel": headerFont(label || translate("Health:")),
+		"current": Math.round(entState.hitpoints),
+		"max": Math.round(entState.maxHitpoints)
+	});
+}
+
 function attackRateDetails(template, type)
 {
 	// Either one arrow shot by UnitAI,
-	let time = template.attack[type].repeatTime / 1000;
-	let timeString = sprintf(translatePlural("%(time)s %(second)s", "%(time)s %(second)s", time), {
-		"time": time,
-		"second": unitFont(translatePlural("second", "seconds", time))
-	});
+	let timeString = getSecondsString(template.attack[type].repeatTime / 1000);
 
 	// or multiple arrows shot by BuildingAI
 	if (!template.buildingAI || type != "Ranged")
@@ -132,13 +212,12 @@ function damageTypesToText(dmg)
 		})).join(commaFont(translate(", ")));
 }
 
-// TODO: should also show minRange
 function getAttackTooltip(template)
 {
 	if (!template.attack)
 		return "";
 
-	let attacks = [];
+	let tooltips = [];
 	for (let type in template.attack)
 	{
 		if (type == "Slaughter")
@@ -156,7 +235,7 @@ function getAttackTooltip(template)
 		let attackLabel = headerFont(g_AttackTypes[type]);
 		if (type == "Capture" || type != "Ranged")
 		{
-			attacks.push(sprintf(translate("%(attackLabel)s %(details)s, %(rate)s"), {
+			tooltips.push(sprintf(translate("%(attackLabel)s %(details)s, %(rate)s"), {
 				"attackLabel": attackLabel,
 				"details":
 					type == "Capture" ?
@@ -167,28 +246,27 @@ function getAttackTooltip(template)
 			continue;
 		}
 
+		let minRange = Math.round(template.attack[type].minRange);
+		let maxRange = Math.round(template.attack[type].maxRange);
 		let realRange = template.attack[type].elevationAdaptedRange;
-		let range = Math.round(template.attack[type].maxRange);
-		let relativeRange = realRange ? Math.round(realRange - range) : 0;
+		let relativeRange = realRange ? Math.round(realRange - maxRange) : 0;
 
-		let rangeString = relativeRange ?
-			translate("%(attackLabel)s %(damageTypes)s, %(rangeLabel)s %(rangeString)s (%(relative)s), %(rate)s") :
-			translate("%(attackLabel)s %(damageTypes)s, %(rangeLabel)s %(rangeString)s, %(rate)s");
-
-		attacks.push(sprintf(rangeString, {
+		tooltips.push(sprintf(g_RangeTooltipString[relativeRange ? "relative" : "non-relative"][minRange ? "minRange" : "no-minRange"], {
 			"attackLabel": attackLabel,
 			"damageTypes": damageTypesToText(template.attack[type]),
-			"rangeLabel": translate("Range:"),
-			"rangeString": sprintf(
-				translatePlural("%(range)s %(meters)s", "%(range)s %(meters)s", range), {
-					"range": range,
-					"meters": unitFont(translatePlural("meter", "meters", range))
-				}),
+			"rangeLabel": headerFont(translate("Range:")),
+			"minRange": minRange,
+			"maxRange": maxRange,
+			"relativeRange": relativeRange > 0 ? sprintf(translate("+%(number)s"), { "number": relativeRange }) : relativeRange,
+			"rangeUnit":
+				unitFont(minRange || relativeRange ?
+					// Translation: For example "0.5 to 1 meters", "1 (+1) meters" or "1 to 2 (+3) meters"
+					translate("meters") :
+					translatePlural("meter", "meters", maxRange)),
 			"rate": rate,
-			"relative": relativeRange > 0 ? "+" + relativeRange : relativeRange,
 		}));
 	}
-	return attacks.join("\n");
+	return tooltips.join("\n");
 }
 
 function getSplashDamageTooltip(template)
@@ -203,15 +281,17 @@ function getSplashDamageTooltip(template)
 		if (!splash)
 			continue;
 
-		tooltips.push([
-			sprintf(translate("%(attackLabel)s %(damageTypes)s"), {
-				"attackLabel": headerFont(translate("Splash Damage:")),
-				"damageTypes": damageTypesToText(splash)
-			}),
-			sprintf(translate("Friendly Fire: %(enabled)s"), {
+		let splashDamageTooltip = sprintf(translate("%(label)s: %(value)s"), {
+			"label": headerFont(g_SplashDamageTypes[splash.shape]),
+			"value": damageTypesToText(splash)
+		});
+
+		if (g_AlwaysDisplayFriendlyFire || splash.friendlyFire)
+			splashDamageTooltip += commaFont(translate(", ")) + sprintf(translate("Friendly Fire: %(enabled)s"), {
 				"enabled": splash.friendlyFire ? translate("Yes") : translate("No")
-			})
-		].join(commaFont(translate(", "))));
+			});
+
+		tooltips.push(splashDamageTooltip);
 	}
 
 	// If multiple attack types deal splash damage, the attack type should be shown to differentiate.
@@ -251,8 +331,8 @@ function getProjectilesTooltip(template)
 	let limit = Math.min(
 		template.buildingAI.maxArrowCount || Infinity,
 		template.buildingAI.defaultArrowCount +
-			template.buildingAI.garrisonArrowMultiplier *
-			template.garrisonHolder.capacity
+			Math.round(template.buildingAI.garrisonArrowMultiplier *
+			template.garrisonHolder.capacity)
 	);
 
 	if (!limit)
@@ -271,7 +351,7 @@ function getProjectilesTooltip(template)
 
 		sprintf(translate("%(label)s: %(value)s"), {
 			"label": headerFont(translateWithContext("projectiles", "Per Unit")),
-			"value": template.buildingAI.garrisonArrowMultiplier
+			"value": +template.buildingAI.garrisonArrowMultiplier.toFixed(2)
 		})
 	].join(commaFont(translate(", ")));
 }
@@ -310,8 +390,9 @@ function getBuildRateTooltip(template)
 function multiplyEntityCosts(template, trainNum)
 {
 	let totalCosts = {};
-	for (let r in template.cost)
-		totalCosts[r] = Math.floor(template.cost[r] * trainNum);
+	for (let r of getCostTypes())
+		if (template.cost[r])
+			totalCosts[r] = Math.floor(template.cost[r] * trainNum);
 
 	return totalCosts;
 }
@@ -319,21 +400,21 @@ function multiplyEntityCosts(template, trainNum)
 /**
  * Helper function for getEntityCostTooltip.
  */
-function getEntityCostComponentsTooltipString(template, trainNum, entity)
+function getEntityCostComponentsTooltipString(template, entity, buildingsCountToTrainFullBatch = 1, fullBatchSize = 1, remainderBatch = 0)
 {
-	if (!trainNum)
-		trainNum = 1;
-
-	let totalCosts = multiplyEntityCosts(template, trainNum);
+	let totalCosts = multiplyEntityCosts(template, buildingsCountToTrainFullBatch * fullBatchSize + remainderBatch);
 	if (template.cost.time)
-		totalCosts.time = Math.ceil(template.cost.time * (entity ? Engine.GuiInterfaceCall("GetBatchTime", { "entity": entity, "batchSize": trainNum }) : 1));
+		totalCosts.time = Math.ceil(template.cost.time * (entity ? Engine.GuiInterfaceCall("GetBatchTime", {
+			"entity": entity,
+			"batchSize": buildingsCountToTrainFullBatch > 0 ? fullBatchSize : remainderBatch
+		}) : 1));
 
 	let costs = [];
-	for (let type in template.cost)
+	for (let type of getCostTypes())
 		// Population bonus is shown in the tooltip
 		if (type != "populationBonus" && totalCosts[type])
 			costs.push(sprintf(translate("%(component)s %(cost)s"), {
-				"component": costIcon(type),
+				"component": resourceIcon(type),
 				"cost": totalCosts[type]
 			}));
 
@@ -342,21 +423,67 @@ function getEntityCostComponentsTooltipString(template, trainNum, entity)
 
 function getGatherTooltip(template)
 {
-	if (!template.gather)
+	if (!template.resourceGatherRates)
+		return "";
+
+	// Average the resource rates (TODO: distinguish between subtypes)
+	let rates = {};
+	for (let resource of g_ResourceData.GetResources())
+	{
+		let types = [resource.code];
+		for (let subtype in resource.subtypes)
+			// We ignore ruins as those are not that common and skew the results
+			if (subtype !== "ruins")
+				types.push(resource.code + "." + subtype);
+
+		let [rate, count] = types.reduce((sum, t) => {
+				let r = template.resourceGatherRates[t];
+				return [sum[0] + (r > 0 ? r : 0), sum[1] + (r > 0 ? 1 : 0)];
+			}, [0, 0]);
+
+		if (rate > 0)
+			rates[resource.code] = +(rate / count).toFixed(1);
+	}
+
+	if (!Object.keys(rates).length)
 		return "";
 
 	return sprintf(translate("%(label)s %(details)s"), {
 		"label": headerFont(translate("Gather Rates:")),
 		"details":
-			Object.keys(template.gather).map(
+			Object.keys(rates).map(
 				type => sprintf(translate("%(resourceIcon)s %(rate)s"), {
-					"resourceIcon": costIcon(type),
-					"rate": template.gather[type]
+					"resourceIcon": resourceIcon(type),
+					"rate": rates[type]
 				})
 			).join("  ")
 	});
 }
 
+function getResourceTrickleTooltip(template)
+{
+	if (!template.resourceTrickle)
+		return "";
+
+	let resCodes = g_ResourceData.GetCodes().filter(res => !!template.resourceTrickle.rates[res]);
+	if (!resCodes.length)
+		return "";
+
+	return sprintf(translate("%(label)s %(details)s"), {
+		"label": headerFont(translate("Resource Trickle:")),
+		"details": sprintf(translate("%(resources)s / %(time)s"), {
+			"resources":
+				resCodes.map(
+					res => sprintf(translate("%(resourceIcon)s %(rate)s"), {
+						"resourceIcon": resourceIcon(res),
+						"rate": template.resourceTrickle.rates[res]
+					})
+				).join("  "),
+			"time": getSecondsString(template.resourceTrickle.interval / 1000)
+		})
+	});
+}
+
 /**
  * Returns an array of strings for a set of wall pieces. If the pieces share
  * resource type requirements, output will be of the form '10 to 30 Stone',
@@ -366,9 +493,7 @@ function getWallPieceTooltip(wallTypes)
 {
 	let out = [];
 	let resourceCount = {};
-
-	// Initialize the acceptable types for '$x to $y $resource' mode.
-	for (let resource in wallTypes[0].cost)
+	for (let resource of getCostTypes())
 		if (wallTypes[0].cost[resource])
 			resourceCount[resource] = [wallTypes[0].cost[resource]];
 
@@ -399,7 +524,7 @@ function getWallPieceTooltip(wallTypes)
 			// Translation: This string is part of the resources cost string on
 			// the tooltip for wall structures.
 			out.push(sprintf(translate("%(resourceIcon)s %(minimum)s to %(resourceIcon)s %(maximum)s"), {
-				"resourceIcon": costIcon(resource),
+				"resourceIcon": resourceIcon(resource),
 				"minimum": Math.min.apply(Math, resourceCount[resource]),
 				"maximum": Math.max.apply(Math, resourceCount[resource])
 			}));
@@ -413,7 +538,7 @@ function getWallPieceTooltip(wallTypes)
 /**
  * Returns the cost information to display in the specified entity's construction button tooltip.
  */
-function getEntityCostTooltip(template, trainNum, entity)
+function getEntityCostTooltip(template, entity, buildingsCountToTrainFullBatch, fullBatchSize, remainderBatch)
 {
 	// Entities with a wallset component are proxies for initiating wall placement and as such do not have a cost of
 	// their own; the individual wall pieces within it do.
@@ -432,11 +557,21 @@ function getEntityCostTooltip(template, trainNum, entity)
 	}
 
 	if (template.cost)
-		return getEntityCostComponentsTooltipString(template, trainNum, entity).join("  ");
+		return getEntityCostComponentsTooltipString(template, entity, buildingsCountToTrainFullBatch, fullBatchSize, remainderBatch).join("  ");
 
 	return "";
 }
 
+function getRequiredTechnologyTooltip(technologyEnabled, requiredTechnology, civ)
+{
+	if (technologyEnabled)
+		return "";
+
+	return sprintf(translate("Requires %(technology)s"), {
+		"technology": getEntityNames(GetTechnologyData(requiredTechnology, civ))
+	});
+}
+
 /**
  * Returns the population bonus information to display in the specified entity's construction button tooltip.
  */
@@ -462,7 +597,7 @@ function getNeededResourcesTooltip(resources)
 	let formatted = [];
 	for (let resource in resources)
 		formatted.push(sprintf(translate("%(component)s %(cost)s"), {
-			"component": '[font="sans-12"]' + costIcon(resource) + '[/font]',
+			"component": '[font="sans-12"]' + resourceIcon(resource) + '[/font]',
 			"cost": resources[resource]
 		}));
 
@@ -532,13 +667,24 @@ function getAurasTooltip(template)
 	if (!template.auras)
 		return "";
 
-	let tooltips = Object.keys(template.auras).map(
-		aura => sprintf(translate("%(auralabel)s %(aurainfo)s"), {
+	let tooltips = [];
+	for (let auraID in template.auras)
+	{
+		let tooltip = sprintf(translate("%(auralabel)s %(aurainfo)s"), {
 			"auralabel": headerFont(sprintf(translate("%(auraname)s:"), {
-				"auraname": translate(template.auras[aura].name)
+				"auraname": translate(template.auras[auraID].name)
 			})),
-			"aurainfo": bodyFont(translate(template.auras[aura].description))
-		}));
+			"aurainfo": bodyFont(translate(template.auras[auraID].description))
+		});
+		let radius = +template.auras[auraID].radius;
+		if (radius)
+			tooltip += " " + sprintf(translatePlural("%(label)s %(val)s %(unit)s", "%(label)s %(val)s %(unit)s", radius), {
+				"label": translateWithContext("aura", "Range:"),
+				"val": radius,
+				"unit": unitFont(translatePlural("meter", "meters", radius))
+			});
+		tooltips.push(tooltip);
+	}
 	return tooltips.join("\n");
 }
 
@@ -593,7 +739,7 @@ function getLootTooltip(template)
 			template.trader && template.trader.goods
 		);
 
-	const lootTypes = ["xp", "food", "wood", "stone", "metal"];
+	const lootTypes = g_ResourceData.GetCodes().concat(["xp"]);
 	let lootLabels = [];
 	for (let type of lootTypes)
 	{
@@ -606,11 +752,14 @@ function getLootTooltip(template)
 
 		// Translation: %(component) will be the icon for the loot type and %(loot) will be the value.
 		lootLabels.push(sprintf(translate("%(component)s %(loot)s"), {
-			"component": costIcon(type),
+			"component": resourceIcon(type),
 			"loot": loot
 		}));
 	}
 
+	if (!lootLabels.length)
+		return "";
+
 	return sprintf(translate("%(label)s %(details)s"), {
 		"label": headerFont(translate("Loot:")),
 		"details": lootLabels.join("  ")
