@@ -120,6 +120,13 @@ AIProxy.prototype.OnUnitIdleChanged = function(msg)
 	this.changes.idle = msg.idle;
 };
 
+AIProxy.prototype.OnUnitStanceChanged = function(msg)
+{
+	if (!this.NotifyChange())
+		return;
+	this.changes.stance = msg.to;
+};
+
 AIProxy.prototype.OnUnitAIStateChanged = function(msg)
 {
 	if (!this.NotifyChange())
@@ -146,7 +153,7 @@ AIProxy.prototype.OnGarrisonedUnitsChanged = function(msg)
 {
 	if (!this.NotifyChange())
 		return;
-	
+
 	let cmpGarrisonHolder = Engine.QueryInterface(this.entity, IID_GarrisonHolder);
 	this.changes.garrisoned = cmpGarrisonHolder.GetEntities();
 
@@ -258,6 +265,8 @@ AIProxy.prototype.GetFullRepresentation = function()
 	{
 		// Updated by OnUnitIdleChanged
 		ret.idle = cmpUnitAI.IsIdle();
+		// Updated by OnUnitStanceChanged
+		ret.stance = cmpUnitAI.GetStanceName();
 		// Updated by OnUnitAIStateChanged
 		ret.unitAIState = cmpUnitAI.GetCurrentState();
 		// Updated by OnUnitAIOrderDataChanged
@@ -329,7 +338,7 @@ AIProxy.prototype.GetFullRepresentation = function()
 AIProxy.prototype.OnOwnershipChanged = function(msg)
 {
 	this.NotifyChange();
-	
+
 	if (msg.from === -1)
 	{
 		this.cmpAIInterface.PushEvent("Create", {"entity" : msg.entity});
@@ -341,7 +350,7 @@ AIProxy.prototype.OnOwnershipChanged = function(msg)
 		this.needsFullGet = true;
 		return;
 	}
-	
+
 	this.changes.owner = msg.to;
 	this.cmpAIInterface.PushEvent("OwnershipChanged", msg);
 };
