@@ -1,49 +1,49 @@
-// True price of 100 units of resource (for case if some resource is more worth).
-// With current bartering system only relative values makes sense
-// so if for example stone is two times more expensive than wood,
-// there will 2:1 exchange rate.
-const TRUE_PRICES = { "food": 100, "wood": 100, "stone": 100, "metal": 100 };
-
-// Constant part of price difference between true price and buy/sell price.
-// In percents.
-// Buy price equal to true price plus constant difference.
-// Sell price equal to true price minus constant difference.
-const CONSTANT_DIFFERENCE = 10;
-
-// Additional difference of prices, added after each deal to specified resource price.
-// In percents.
-const DIFFERENCE_PER_DEAL = 2;
-
-// Price difference which restored each restore timer tick
-// In percents.
-const DIFFERENCE_RESTORE = 0.5;
-
-// Interval of timer which slowly restore prices after deals
-const RESTORE_TIMER_INTERVAL = 5000;
-
-// Array of resource names
-const RESOURCES = ["food", "wood", "stone", "metal"];
-
 function Barter() {}
 
 Barter.prototype.Schema =
 	"<a:component type='system'/><empty/>";
 
+/**
+ * The "true price" is a base price of 100 units of resource (for the case of some resources being of more worth than others).
+ * With current bartering system only relative values makes sense so if for example stone is two times more expensive than wood,
+ * there will 2:1 exchange rate.
+ *
+ * Constant part of price percentage difference between true price and buy/sell price.
+ * Buy price equal to true price plus constant difference.
+ * Sell price equal to true price minus constant difference.
+ */
+Barter.prototype.CONSTANT_DIFFERENCE = 10;
+/**
+ * Additional difference of prices in percents, added after each deal to specified resource price.
+ */
+Barter.prototype.DIFFERENCE_PER_DEAL = 2;
+/**
+ * Price difference percentage which restored each restore timer tick
+ */
+Barter.prototype.DIFFERENCE_RESTORE = 0.5;
+/**
+ * Interval of timer which slowly restore prices after deals
+ */
+Barter.prototype.RESTORE_TIMER_INTERVAL = 5000;
+
 Barter.prototype.Init = function()
 {
 	this.priceDifferences = {};
-	for (var resource of RESOURCES)
+	for (let resource of Resources.GetCodes())
 		this.priceDifferences[resource] = 0;
 	this.restoreTimer = undefined;
 };
 
-Barter.prototype.GetPrices = function()
+Barter.prototype.GetPrices = function(playerEntity)
 {
 	var prices = { "buy": {}, "sell": {} };
-	for (var resource of RESOURCES)
+	let cmpPlayer = Engine.QueryInterface(playerEntity, IID_Player);
+	let multiplier = cmpPlayer.GetBarterMultiplier();
+	for (let resource of Resources.GetCodes())
 	{
-		prices["buy"][resource] = TRUE_PRICES[resource] * (100 + CONSTANT_DIFFERENCE + this.priceDifferences[resource]) / 100;
-		prices["sell"][resource] = TRUE_PRICES[resource] * (100 - CONSTANT_DIFFERENCE + this.priceDifferences[resource]) / 100;
+		let truePrice = Resources.GetResource(resource).truePrice;
+		prices.buy[resource] = truePrice * (100 + this.CONSTANT_DIFFERENCE + this.priceDifferences[resource]) * multiplier.buy[resource] / 100;
+		prices.sell[resource] = truePrice * (100 - this.CONSTANT_DIFFERENCE + this.priceDifferences[resource]) * multiplier.sell[resource] / 100;
 	}
 	return prices;
 };
@@ -71,12 +71,13 @@ Barter.prototype.ExchangeResources = function(playerEntity, resourceToSell, reso
 		warn("ExchangeResources: incorrect amount: " + uneval(amount));
 		return;
 	}
-	if (RESOURCES.indexOf(resourceToSell) == -1)
+	let availResources = Resources.GetCodes();
+	if (availResources.indexOf(resourceToSell) == -1)
 	{
 		warn("ExchangeResources: incorrect resource to sell: " + uneval(resourceToSell));
 		return;
 	}
-	if (RESOURCES.indexOf(resourceToBuy) == -1)
+	if (availResources.indexOf(resourceToBuy) == -1)
 	{
 		warn("ExchangeResources: incorrect resource to buy: " + uneval(resourceToBuy));
 		return;
@@ -86,16 +87,17 @@ Barter.prototype.ExchangeResources = function(playerEntity, resourceToSell, reso
 		warn("ExchangeResources: player has no markets");
 		return;
 	}
+	if (amount != 100 && amount != 500)
+		return;
 
 	var cmpPlayer = Engine.QueryInterface(playerEntity, IID_Player);
-	var prices = this.GetPrices();
+	var prices = this.GetPrices(playerEntity);
 	var amountsToSubtract = {};
 	amountsToSubtract[resourceToSell] = amount;
 	if (cmpPlayer.TrySubtractResources(amountsToSubtract))
 	{
 		var amountToAdd = Math.round(prices["sell"][resourceToSell] / prices["buy"][resourceToBuy] * amount);
 		cmpPlayer.AddResource(resourceToBuy, amountToAdd);
-		var numberOfDeals = Math.round(amount / 100);
 
 		// Display chat message to observers
 		var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
@@ -116,29 +118,29 @@ Barter.prototype.ExchangeResources = function(playerEntity, resourceToSell, reso
 			cmpStatisticsTracker.IncreaseResourcesBoughtCounter(resourceToBuy, amountToAdd);
 		}
 
+		let difference = this.DIFFERENCE_PER_DEAL * amount / 100;
 		// Increase price difference for both exchange resources.
-		// Overal price difference (constant + dynamic) can't exceed +-99%
-		// so both buy/sell prices limited to [1%; 199%] interval.
-		this.priceDifferences[resourceToSell] -= DIFFERENCE_PER_DEAL * numberOfDeals;
-		this.priceDifferences[resourceToSell] = Math.min(99-CONSTANT_DIFFERENCE, Math.max(CONSTANT_DIFFERENCE-99, this.priceDifferences[resourceToSell]));
-		this.priceDifferences[resourceToBuy] += DIFFERENCE_PER_DEAL * numberOfDeals;
-		this.priceDifferences[resourceToBuy] = Math.min(99-CONSTANT_DIFFERENCE, Math.max(CONSTANT_DIFFERENCE-99, this.priceDifferences[resourceToBuy]));
+		// Overall price difference (dynamic +/- constant) can't exceed +-99%.
+		this.priceDifferences[resourceToSell] -= difference;
+		this.priceDifferences[resourceToSell] = Math.min(99 - this.CONSTANT_DIFFERENCE, Math.max(this.CONSTANT_DIFFERENCE - 99, this.priceDifferences[resourceToSell]));
+		this.priceDifferences[resourceToBuy] += difference;
+		this.priceDifferences[resourceToBuy] = Math.min(99 - this.CONSTANT_DIFFERENCE, Math.max(this.CONSTANT_DIFFERENCE - 99, this.priceDifferences[resourceToBuy]));
 	}
 
 	if (this.restoreTimer === undefined)
 	{
 		var cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
-		this.restoreTimer = cmpTimer.SetInterval(this.entity, IID_Barter, "ProgressTimeout", RESTORE_TIMER_INTERVAL, RESTORE_TIMER_INTERVAL, {});
+		this.restoreTimer = cmpTimer.SetInterval(this.entity, IID_Barter, "ProgressTimeout", this.RESTORE_TIMER_INTERVAL, this.RESTORE_TIMER_INTERVAL, {});
 	}
 };
 
 Barter.prototype.ProgressTimeout = function(data)
 {
 	var needRestore = false;
-	for (var resource of RESOURCES)
+	for (let resource of Resources.GetCodes())
 	{
 		// Calculate value to restore, it should be limited to [-DIFFERENCE_RESTORE; DIFFERENCE_RESTORE] interval
-		var differenceRestore = Math.min(DIFFERENCE_RESTORE, Math.max(-DIFFERENCE_RESTORE, this.priceDifferences[resource]));
+		var differenceRestore = Math.min(this.DIFFERENCE_RESTORE, Math.max(-this.DIFFERENCE_RESTORE, this.priceDifferences[resource]));
 		differenceRestore = -differenceRestore;
 		this.priceDifferences[resource] += differenceRestore;
 		// If price difference still exists then set flag to run timer again
