@@ -8,10 +8,10 @@ Builder.prototype.Schema =
 			"\n    structures/{civ}_barracks\n    structures/{civ}_civil_centre\n    structures/pers_apadana\n  " +
 		"</Entities>" +
 	"</a:example>" +
-	"<element name='Rate' a:help='Construction speed multiplier (1.0 is normal speed, higher values are faster)'>" +
+	"<element name='Rate' a:help='Construction speed multiplier (1.0 is normal speed, higher values are faster).'>" +
 		"<ref name='positiveDecimal'/>" +
 	"</element>" +
-	"<element name='Entities' a:help='Space-separated list of entity template names that this unit can build. The special string \"{civ}\" will be automatically replaced by the unit&apos;s four-character civ code. This element can also be empty, in which case no new foundations may be placed by the unit, but they can still repair existing buildings'>" +
+	"<element name='Entities' a:help='Space-separated list of entity template names that this unit can build. The special string \"{civ}\" will be automatically replaced by the unit&apos;s four-character civ code. This element can also be empty, in which case no new foundations may be placed by the unit, but they can still repair existing buildings.'>" +
 		"<attribute name='datatype'>" +
 			"<value>tokens</value>" +
 		"</attribute>" +
@@ -26,31 +26,29 @@ Builder.prototype.Serialize = null; // we have no dynamic state to save
 
 Builder.prototype.GetEntitiesList = function()
 {
-	var entities = [];
-	var string = this.template.Entities._string;
-	if (string)
-	{
-		// Replace the "{civ}" codes with this entity's civ ID
-		var cmpIdentity = Engine.QueryInterface(this.entity, IID_Identity);
-		if (cmpIdentity)
-			string = string.replace(/\{civ\}/g, cmpIdentity.GetCiv());
-		entities = string.split(/\s+/);
-		
-		// Remove disabled entities
-		var cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
-		var disabledEntities = cmpPlayer.GetDisabledTemplates();
-		
-		for (var i = entities.length - 1; i >= 0; --i)
-			if (disabledEntities[entities[i]])
-				entities.splice(i, 1);
-	}
-	return entities;
+	let string = this.template.Entities._string;
+
+	if (!string)
+		return [];
+
+	let cmpIdentity = Engine.QueryInterface(this.entity, IID_Identity);
+	if (cmpIdentity)
+		string = string.replace(/\{civ\}/g, cmpIdentity.GetCiv());
+
+	let entities = string.split(/\s+/);
+
+	let cmpPlayer = QueryOwnerInterface(this.entity);
+	if (!cmpPlayer)
+		return entities;
+
+	let disabledTemplates = cmpPlayer.GetDisabledTemplates();
+	return entities.filter(ent => !disabledTemplates[ent]);
 };
 
 Builder.prototype.GetRange = function()
 {
-	var cmpObstruction = Engine.QueryInterface(this.entity, IID_Obstruction);
-	var max = 2;
+	let max = 2;
+	let cmpObstruction = Engine.QueryInterface(this.entity, IID_Obstruction);
 	if (cmpObstruction)
 		max += cmpObstruction.GetUnitRadius();
 
@@ -60,13 +58,11 @@ Builder.prototype.GetRange = function()
 /**
  * Build/repair the target entity. This should only be called after a successful range check.
  * It should be called at a rate of once per second.
- * Returns obj with obj.finished==true if this is a repair and it's fully repaired.
  */
 Builder.prototype.PerformBuilding = function(target)
 {
 	let rate = ApplyValueModificationsToEntity("Builder/Rate", +this.template.Rate, this.entity);
 
-	// If it's a foundation, then build it
 	let cmpFoundation = Engine.QueryInterface(target, IID_Foundation);
 	if (cmpFoundation)
 	{
@@ -74,7 +70,6 @@ Builder.prototype.PerformBuilding = function(target)
 		return;
 	}
 
-	// Otherwise try to repair it
 	let cmpRepairable = Engine.QueryInterface(target, IID_Repairable);
 	if (cmpRepairable)
 	{
