@@ -121,11 +121,9 @@ BuildingAI.prototype.SetupRangeQuery = function()
 	if (!cmpPlayer)
 		return;
 
-	var enemies = [];
-	var numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
-	for (let i = 1; i < numPlayers; ++i)
-		if (cmpPlayer.IsEnemy(i))
-			enemies.push(i);
+	var enemies = cmpPlayer.GetEnemies();
+	if (enemies.length && enemies[0] == 0)
+		enemies.shift(); // remove gaia
 
 	if (!enemies.length)
 		return;
@@ -224,7 +222,7 @@ BuildingAI.prototype.StartTimer = function()
 BuildingAI.prototype.GetDefaultArrowCount = function()
 {
 	var arrowCount = +this.template.DefaultArrowCount;
-	return ApplyValueModificationsToEntity("BuildingAI/DefaultArrowCount", arrowCount, this.entity);
+	return Math.round(ApplyValueModificationsToEntity("BuildingAI/DefaultArrowCount", arrowCount, this.entity));
 };
 
 BuildingAI.prototype.GetMaxArrowCount = function()
@@ -302,7 +300,7 @@ BuildingAI.prototype.FireArrows = function()
 		arrowsToFire = this.arrowsLeft;
 	else
 		arrowsToFire = Math.min(
-		    Math.round(2 * Math.random() * this.GetArrowCount() / roundCount),
+		    randIntInclusive(0, 2 * this.GetArrowCount() / roundCount),
 		    this.arrowsLeft
 		);
 
