@@ -34,7 +34,7 @@ RallyPoint.prototype.GetPositions = function()
 		// Update the rallypoint coordinates if the target is alive
 		if (!this.data[i] || !this.data[i].target || !this.TargetIsAlive(this.data[i].target))
 			continue;
-		
+
 		// and visible
 		if (cmpRangeManager && cmpOwnership &&
 				cmpRangeManager.GetLosVisibility(this.data[i].target, cmpOwnership.GetOwner()) != "visible")
@@ -69,7 +69,7 @@ RallyPoint.prototype.AddData = function(data)
 };
 
 // Returns an array with the data associated with this rally point.  Each element has the structure:
-// {"type": "walk/gather/garrison/...", "target": targetEntityId, "resourceType": "tree/fruit/ore/..."} where target 
+// {"type": "walk/gather/garrison/...", "target": targetEntityId, "resourceType": "tree/fruit/ore/..."} where target
 // and resourceType (specific resource type) are optional, also target may be an invalid entity, check for existence.
 RallyPoint.prototype.GetData = function()
 {
@@ -106,8 +106,7 @@ RallyPoint.prototype.OnGlobalEntityRenamed = function(msg)
 RallyPoint.prototype.OnOwnershipChanged = function(msg)
 {
 	// No need to reset when constructing or destructing the entity
-	// Don't reset upon defeat to improve performance
-	if (msg.from == -1 || msg.to < 1)
+	if (msg.from == -1 || msg.to == -1)
 		return;
 
 	this.Reset();
