@@ -25,35 +25,10 @@ ResourceGatherer.prototype.Schema =
 		"<ref name='positiveDecimal'/>" +
 	"</element>" +
 	"<element name='Rates' a:help='Per-resource-type gather rate multipliers. If a resource type is not specified then it cannot be gathered by this unit'>" +
-		"<interleave>" +
-			"<optional><element name='food' a:help='Food gather rate (may be overridden by \"food.*\" subtypes)'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='wood' a:help='Wood gather rate'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='stone' a:help='Stone gather rate'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='metal' a:help='Metal gather rate'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='treasure' a:help='Treasure gather rate (only presense on value makes sense, size is only used to determine the delay before gathering, so it should be set to 1)'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='food.fish' a:help='Fish gather rate (overrides \"food\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='food.fruit' a:help='Fruit gather rate (overrides \"food\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='food.grain' a:help='Grain gather rate (overrides \"food\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='food.meat' a:help='Meat gather rate (overrides \"food\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='food.milk' a:help='Milk gather rate (overrides \"food\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='wood.tree' a:help='Tree gather rate (overrides \"wood\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='wood.ruins' a:help='Tree gather rate (overrides \"wood\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='stone.rock' a:help='Rock gather rate (overrides \"stone\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='stone.ruins' a:help='Rock gather rate (overrides \"stone\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='metal.ore' a:help='Ore gather rate (overrides \"metal\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='treasure.food' a:help='Food treasure gather rate (overrides \"treasure\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='treasure.wood' a:help='Wood treasure gather rate (overrides \"treasure\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='treasure.stone' a:help='Stone treasure gather rate (overrides \"treasure\")'><ref name='positiveDecimal'/></element></optional>" +
-			"<optional><element name='treasure.metal' a:help='Metal treasure gather rate (overrides \"treasure\")'><ref name='positiveDecimal'/></element></optional>" +
-		"</interleave>" +
+		Resources.BuildSchema("positiveDecimal", ["treasure"], true) +
 	"</element>" +
 	"<element name='Capacities' a:help='Per-resource-type maximum carrying capacity'>" +
-		"<interleave>" +
-			"<element name='food' a:help='Food capacity'><ref name='positiveDecimal'/></element>" +
-			"<element name='wood' a:help='Wood capacity'><ref name='positiveDecimal'/></element>" +
-			"<element name='stone' a:help='Stone capacity'><ref name='positiveDecimal'/></element>" +
-			"<element name='metal' a:help='Metal capacity'><ref name='positiveDecimal'/></element>" +
-		"</interleave>" +
+		Resources.BuildSchema("positiveDecimal") +
 	"</element>";
 
 ResourceGatherer.prototype.Init = function()
@@ -93,7 +68,7 @@ ResourceGatherer.prototype.GetCarryingStatus = function()
  */
 ResourceGatherer.prototype.GiveResources = function(resources)
 {
-	for each (let resource in resources)
+	for (let resource of resources)
 	{
 		this.carrying[resource.type] = +(resource.amount);
 	}
@@ -137,6 +112,14 @@ ResourceGatherer.prototype.RecalculateGatherRatesAndCapacities = function()
 	this.rates = {};
 	for (let r in this.template.Rates)
 	{
+		let type = r.split(".");
+
+		if (type[0] != "treasure" && type.length > 1 && !Resources.GetResource(type[0]).subtypes[type[1]])
+		{
+			error("Resource subtype not found: " + type[0] + "." + type[1]);
+			continue;
+		}
+
 		let rate = ApplyValueModificationsToEntity("ResourceGatherer/Rates/" + r, +this.template.Rates[r], this.entity);
 		this.rates[r] = rate * this.baseSpeed;
 	}
@@ -174,7 +157,7 @@ ResourceGatherer.prototype.GetRange = function()
 
 /**
  * Try to gather treasure
- * @return 'true' if treasure is successfully gathered and 'false' in the other case
+ * @return 'true' if treasure is successfully gathered, otherwise 'false'
  */
 ResourceGatherer.prototype.TryInstantGather = function(target)
 {
@@ -194,9 +177,9 @@ ResourceGatherer.prototype.TryInstantGather = function(target)
 	if (cmpStatisticsTracker)
 		cmpStatisticsTracker.IncreaseTreasuresCollectedCounter();
 
-	let cmpTrigger = Engine.QueryInterface(SYSTEM_ENTITY, IID_Trigger);  
-	if (cmpTrigger && cmpPlayer) 
-		cmpTrigger.CallEvent("TreasureCollected", {"player": cmpPlayer.GetPlayerID(), "type": type.specific, "amount": status.amount });  
+	let cmpTrigger = Engine.QueryInterface(SYSTEM_ENTITY, IID_Trigger);
+	if (cmpTrigger && cmpPlayer)
+		cmpTrigger.CallEvent("TreasureCollected", {"player": cmpPlayer.GetPlayerID(), "type": type.specific, "amount": status.amount });
 
 	return true;
 };
@@ -272,27 +255,12 @@ ResourceGatherer.prototype.GetTargetGatherRate = function(target)
 	if ("Mirages" in cmpResourceSupply)
 		return rate;
 
-	// Apply diminishing returns with more gatherers, for e.g. infinite farms. For most resources this has no effect. (GetDiminishingReturns will return null.)
-	// We can assume that for resources that are miraged this is the case. (else just add the diminishing returns data to the mirage data and remove the
-	// early return above)
-	// Note to people looking to change <DiminishingReturns> in a template: This is a bit complicated. Basically, the lower that number is
-	// the steeper diminishing returns will be. I suggest playing around with Wolfram Alpha or a graphing calculator a bit.
-	// In each of the following links, replace 0.65 with the gather rate of your worker for the resource with diminishing returns and
-	// 14 with the constant you wish to use to control the diminishing returns.
-	// (In this case 0.65 is the women farming rate, in resources/second, and 14 is a good constant for farming.)
-	// This is the gather rate in resources/second of each individual worker as the total number of workers goes up:
-	// http://www.wolframalpha.com/input/?i=plot+%281%2F2+cos%28%28x-1%29*pi%2F14%29+%2B+1%2F2%29+*+0.65+from+1+to+5
-	// This is the total output of the resource in resources/second:
-	// http://www.wolframalpha.com/input/?i=plot+x%281%2F2+cos%28%28x-1%29*pi%2F14%29+%2B+1%2F2%29+*+0.65+from+1+to+5
-	// This is the fraction of a worker each new worker is worth (the 5th worker in this example is only producing about half as much as the first one):
-	// http://www.wolframalpha.com/input/?i=plot+x%281%2F2+cos%28%28x-1%29*pi%2F14%29+%2B+1%2F2%29+-++%28x-1%29%281%2F2+cos%28%28x-2%29*pi%2F14%29+%2B+1%2F2%29+from+x%3D1+to+5+and+y%3D0+to+1
-	// Here's how this technically works:
-	// The cosine function is an oscillating curve, normally between -1 and 1. Multiplying by 0.5 squishes that down to
-	// between -0.5 and 0.5. Adding 0.5 to that changes the range to 0 to 1. The diminishingReturns constant
-	// adjusts the period of the curve.
+	// Apply diminishing returns with more gatherers, for e.g. infinite farms. For most resources this has no effect
+	// (GetDiminishingReturns will return null). We can assume that for resources that are miraged this is the case
+	// (else just add the diminishing returns data to the mirage data and remove the early return above)
 	let diminishingReturns = cmpResourceSupply.GetDiminishingReturns();
 	if (diminishingReturns)
-		rate = (0.5 * Math.cos((cmpResourceSupply.GetNumGatherers() - 1) * Math.PI / diminishingReturns) + 0.5) * rate;
+		rate *= diminishingReturns;
 
 	return rate;
 };
@@ -339,7 +307,7 @@ ResourceGatherer.prototype.CommitResources = function(types)
 	let cmpPlayer = QueryOwnerInterface(this.entity);
 
 	if (cmpPlayer)
-		for each (let type in types)
+		for (let type of types)
 			if (type in this.carrying)
 			{
 				cmpPlayer.AddResource(type, this.carrying[type]);
