@@ -1,14 +1,16 @@
 function StatisticsTracker() {}
 
+const g_UpdateSequenceInterval = 30 * 1000;
+
 StatisticsTracker.prototype.Schema =
-	"<a:component type='system'/><empty/>";
+	"<empty/>";
 
 StatisticsTracker.prototype.Init = function()
 {
 	this.unitsClasses = [
 		"Infantry",
 		"Worker",
-		"Female",
+		"FemaleCitizen",
 		"Cavalry",
 		"Champion",
 		"Hero",
@@ -19,7 +21,7 @@ StatisticsTracker.prototype.Init = function()
 	this.unitsTrained = {
 		"Infantry": 0,
 		"Worker": 0,
-		"Female": 0,
+		"FemaleCitizen": 0,
 		"Cavalry": 0,
 		"Champion": 0,
 		"Hero": 0,
@@ -28,10 +30,11 @@ StatisticsTracker.prototype.Init = function()
 		"Trader": 0,
 		"total": 0
 	};
+	this.domesticUnitsTrainedValue = 0;
 	this.unitsLost = {
 		"Infantry": 0,
 		"Worker": 0,
-		"Female": 0,
+		"FemaleCitizen": 0,
 		"Cavalry": 0,
 		"Champion": 0,
 		"Hero": 0,
@@ -44,7 +47,7 @@ StatisticsTracker.prototype.Init = function()
 	this.enemyUnitsKilled = {
 		"Infantry": 0,
 		"Worker": 0,
-		"Female": 0,
+		"FemaleCitizen": 0,
 		"Cavalry": 0,
 		"Champion": 0,
 		"Hero": 0,
@@ -57,7 +60,7 @@ StatisticsTracker.prototype.Init = function()
 	this.unitsCaptured = {
 		"Infantry": 0,
 		"Worker": 0,
-		"Female": 0,
+		"FemaleCitizen": 0,
 		"Cavalry": 0,
 		"Champion": 0,
 		"Hero": 0,
@@ -122,30 +125,18 @@ StatisticsTracker.prototype.Init = function()
 	this.buildingsCapturedValue = 0;
 
 	this.resourcesGathered = {
-		"food": 0,
-		"wood": 0,
-		"metal": 0,
-		"stone": 0,
 		"vegetarianFood": 0
 	};
-	this.resourcesUsed = {
-		"food": 0,
-		"wood": 0,
-		"metal": 0,
-		"stone": 0
-	};
-	this.resourcesSold = {
-		"food": 0,
-		"wood": 0,
-		"metal": 0,
-		"stone": 0
-	};
-	this.resourcesBought = {
-		"food": 0,
-		"wood": 0,
-		"metal": 0,
-		"stone": 0
-	};
+	this.resourcesUsed = {};
+	this.resourcesSold = {};
+	this.resourcesBought = {};
+	for (let res of Resources.GetCodes())
+	{
+		this.resourcesGathered[res] = 0;
+		this.resourcesUsed[res] = 0;
+		this.resourcesSold[res] = 0;
+		this.resourcesBought[res] = 0;
+	}
 
 	this.tributesSent = 0;
 	this.tributesReceived = 0;
@@ -154,6 +145,16 @@ StatisticsTracker.prototype.Init = function()
 	this.lootCollected = 0;
 	this.peakPercentMapControlled = 0;
 	this.teamPeakPercentMapControlled = 0;
+
+	let cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
+	this.updateTimer = cmpTimer.SetInterval(
+		this.entity, IID_StatisticsTracker, "UpdateSequences", 0, g_UpdateSequenceInterval);
+};
+
+StatisticsTracker.prototype.OnGlobalInitGame = function()
+{
+	this.sequences = clone(this.GetStatistics());
+	this.sequences.time = [];
 };
 
 /**
@@ -176,6 +177,7 @@ StatisticsTracker.prototype.GetStatistics = function()
 {
 	return {
 		"unitsTrained": this.unitsTrained,
+		"domesticUnitsTrainedValue": this.domesticUnitsTrainedValue,
 		"unitsLost": this.unitsLost,
 		"unitsLostValue": this.unitsLostValue,
 		"enemyUnitsKilled": this.enemyUnitsKilled,
@@ -207,11 +209,38 @@ StatisticsTracker.prototype.GetStatistics = function()
 	};
 };
 
+StatisticsTracker.prototype.GetSequences = function()
+{
+	let ret = clone(this.sequences);
+	let cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
+
+	ret.time.push(cmpTimer.GetTime() / 1000);
+	this.PushValue(this.GetStatistics(), ret);
+	return ret;
+};
+
+/**
+ * Used to print statistics for non-visual autostart games.
+ * @return The player's statistics as a JSON string beautified with some indentations.
+ */
+StatisticsTracker.prototype.GetStatisticsJSON = function()
+{
+	let cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
+
+	let playerStatistics = {
+		"playerID": cmpPlayer.GetPlayerID(),
+		"playerState": cmpPlayer.GetState(),
+		"statistics": this.GetStatistics()
+	};
+
+	return JSON.stringify(playerStatistics, null, "\t");
+};
+
 /**
  * Increments counter associated with certain entity/counter and type of given entity.
- * @param cmpIdentity The entity identity component
- * @param counter The name of the counter to increment (e.g. "unitsTrained")
- * @param type The type of the counter (e.g. "workers")
+ * @param cmpIdentity - the entity identity component.
+ * @param counter - the name of the counter to increment (e.g. "unitsTrained").
+ * @param type - the type of the counter (e.g. "workers").
  */
 StatisticsTracker.prototype.CounterIncrement = function(cmpIdentity, counter, type)
 {
@@ -229,15 +258,23 @@ StatisticsTracker.prototype.CounterIncrement = function(cmpIdentity, counter, ty
  */
 StatisticsTracker.prototype.IncreaseTrainedUnitsCounter = function(trainedUnit)
 {
-	var cmpUnitEntityIdentity = Engine.QueryInterface(trainedUnit, IID_Identity);
+	let cmpUnitEntityIdentity = Engine.QueryInterface(trainedUnit, IID_Identity);
 
 	if (!cmpUnitEntityIdentity)
 		return;
 
+	let cmpCost = Engine.QueryInterface(trainedUnit, IID_Cost);
+	let costs = cmpCost && cmpCost.GetResourceCosts();
+
 	for (let type of this.unitsClasses)
 		this.CounterIncrement(cmpUnitEntityIdentity, "unitsTrained", type);
 
 	++this.unitsTrained.total;
+
+	if (cmpUnitEntityIdentity.HasClass("Domestic") && costs)
+		for (let type in costs)
+			this.domesticUnitsTrainedValue += costs[type];
+
 };
 
 /**
@@ -260,11 +297,12 @@ StatisticsTracker.prototype.IncreaseConstructedBuildingsCounter = function(const
 StatisticsTracker.prototype.KilledEntity = function(targetEntity)
 {
 	var cmpTargetEntityIdentity = Engine.QueryInterface(targetEntity, IID_Identity);
-	var cmpCost = Engine.QueryInterface(targetEntity, IID_Cost);
-	var costs = cmpCost.GetResourceCosts();
 	if (!cmpTargetEntityIdentity)
 		return;
 
+	var cmpCost = Engine.QueryInterface(targetEntity, IID_Cost);
+	var costs = cmpCost && cmpCost.GetResourceCosts();
+
 	var cmpTargetOwnership = Engine.QueryInterface(targetEntity, IID_Ownership);
 
 	// Ignore gaia
@@ -278,8 +316,9 @@ StatisticsTracker.prototype.KilledEntity = function(targetEntity)
 
 		++this.enemyUnitsKilled.total;
 
-		for (let type in costs)
-			this.enemyUnitsKilledValue += costs[type];
+		if (costs)
+			for (let type in costs)
+				this.enemyUnitsKilledValue += costs[type];
 	}
 
 	let cmpFoundation = Engine.QueryInterface(targetEntity, IID_Foundation);
@@ -290,19 +329,21 @@ StatisticsTracker.prototype.KilledEntity = function(targetEntity)
 
 		++this.enemyBuildingsDestroyed.total;
 
-		for (let type in costs)
-			this.enemyBuildingsDestroyedValue += costs[type];
+		if (costs)
+			for (let type in costs)
+				this.enemyBuildingsDestroyedValue += costs[type];
 	}
 };
 
 StatisticsTracker.prototype.LostEntity = function(lostEntity)
 {
 	var cmpLostEntityIdentity = Engine.QueryInterface(lostEntity, IID_Identity);
-	var cmpCost = Engine.QueryInterface(lostEntity, IID_Cost);
-	var costs = cmpCost.GetResourceCosts();
 	if (!cmpLostEntityIdentity)
 		return;
 
+	var cmpCost = Engine.QueryInterface(lostEntity, IID_Cost);
+	var costs = cmpCost && cmpCost.GetResourceCosts();
+
 	if (cmpLostEntityIdentity.HasClass("Unit") && !cmpLostEntityIdentity.HasClass("Domestic"))
 	{
 		for (let type of this.unitsClasses)
@@ -310,8 +351,9 @@ StatisticsTracker.prototype.LostEntity = function(lostEntity)
 
 		++this.unitsLost.total;
 
-		for (let type in costs)
-			this.unitsLostValue += costs[type];
+		if (costs)
+			for (let type in costs)
+				this.unitsLostValue += costs[type];
 	}
 
 	let cmpFoundation = Engine.QueryInterface(lostEntity, IID_Foundation);
@@ -322,8 +364,9 @@ StatisticsTracker.prototype.LostEntity = function(lostEntity)
 
 		++this.buildingsLost.total;
 
-		for (let type in costs)
-			this.buildingsLostValue += costs[type];
+		if (costs)
+			for (let type in costs)
+				this.buildingsLostValue += costs[type];
 	}
 };
 
@@ -333,6 +376,9 @@ StatisticsTracker.prototype.CapturedEntity = function(capturedEntity)
 	if (!cmpCapturedEntityIdentity)
 		return;
 
+	let cmpCost = Engine.QueryInterface(capturedEntity, IID_Cost);
+	let costs = cmpCost && cmpCost.GetResourceCosts();
+
 	if (cmpCapturedEntityIdentity.HasClass("Unit"))
 	{
 		for (let type of this.unitsClasses)
@@ -340,13 +386,9 @@ StatisticsTracker.prototype.CapturedEntity = function(capturedEntity)
 
 		++this.unitsCaptured.total;
 
-		let cmpCost = Engine.QueryInterface(capturedEntity, IID_Cost);
-		if (!cmpCost)
-			return;
-
-		let costs = cmpCost.GetResourceCosts();
-		for (let type in costs)
-			this.unitsCapturedValue += costs[type];
+		if (costs)
+			for (let type in costs)
+				this.unitsCapturedValue += costs[type];
 	}
 
 	if (cmpCapturedEntityIdentity.HasClass("Structure"))
@@ -356,20 +398,16 @@ StatisticsTracker.prototype.CapturedEntity = function(capturedEntity)
 
 		++this.buildingsCaptured.total;
 
-		let cmpCost = Engine.QueryInterface(capturedEntity, IID_Cost);
-		if (!cmpCost)
-			return;
-
-		let costs = cmpCost.GetResourceCosts();
-		for (let type in costs)
-			this.buildingsCapturedValue += costs[type];
+		if (costs)
+			for (let type in costs)
+				this.buildingsCapturedValue += costs[type];
 	}
 };
 
 /**
- * @param type Generic type of resource (string)
- * @param amount Amount of resource, whick should be added (integer)
- * @param specificType Specific type of resource (string, optional)
+ * @param {string} type - generic type of resource.
+ * @param {number} amount - amount of resource, whick should be added.
+ * @param {string} specificType - specific type of resource.
  */
 StatisticsTracker.prototype.IncreaseResourceGatheredCounter = function(type, amount, specificType)
 {
@@ -380,8 +418,8 @@ StatisticsTracker.prototype.IncreaseResourceGatheredCounter = function(type, amo
 };
 
 /**
- * @param type Generic type of resource (string)
- * @param amount Amount of resource, which should be added (integer)
+ * @param {string} type - generic type of resource.
+ * @param {number} amount - amount of resource, which should be added.
  */
 StatisticsTracker.prototype.IncreaseResourceUsedCounter = function(type, amount)
 {
@@ -504,4 +542,32 @@ StatisticsTracker.prototype.OnTerritoriesChanged = function(msg)
 		this.teamPeakPercentMapControlled = newPercent;
 };
 
+/**
+ * Adds the values of fromData to the end of the arrays of toData.
+ * If toData misses the needed array, one will be created.
+ *
+ * @param fromData - an object of values or a value.
+ * @param toData - an object of arrays or an array.
+**/
+StatisticsTracker.prototype.PushValue = function(fromData, toData)
+{
+	if (typeof fromData == "object")
+		for (let prop in fromData)
+		{
+			if (typeof toData[prop] != "object")
+				toData[prop] = [fromData[prop]];
+			else
+				this.PushValue(fromData[prop], toData[prop]);
+		}
+	else
+		toData.push(fromData);
+};
+
+StatisticsTracker.prototype.UpdateSequences = function()
+{
+	let cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
+	this.sequences.time.push(cmpTimer.GetTime() / 1000);
+	this.PushValue(this.GetStatistics(), this.sequences);
+};
+
 Engine.RegisterComponentType(IID_StatisticsTracker, "StatisticsTracker", StatisticsTracker);
