@@ -10,6 +10,7 @@ Engine.LoadComponentScript("interfaces/DamageReceiver.js");
 Engine.LoadComponentScript("interfaces/EndGameManager.js");
 Engine.LoadComponentScript("interfaces/EntityLimits.js");
 Engine.LoadComponentScript("interfaces/Foundation.js");
+Engine.LoadComponentScript("interfaces/Garrisonable.js");
 Engine.LoadComponentScript("interfaces/GarrisonHolder.js");
 Engine.LoadComponentScript("interfaces/Gate.js");
 Engine.LoadComponentScript("interfaces/Guard.js");
@@ -24,6 +25,7 @@ Engine.LoadComponentScript("interfaces/RallyPoint.js");
 Engine.LoadComponentScript("interfaces/Repairable.js");
 Engine.LoadComponentScript("interfaces/ResourceDropsite.js");
 Engine.LoadComponentScript("interfaces/ResourceGatherer.js");
+Engine.LoadComponentScript("interfaces/ResourceTrickle.js");
 Engine.LoadComponentScript("interfaces/ResourceSupply.js");
 Engine.LoadComponentScript("interfaces/TechnologyManager.js");
 Engine.LoadComponentScript("interfaces/Trader.js");
@@ -34,6 +36,21 @@ Engine.LoadComponentScript("interfaces/Upgrade.js");
 Engine.LoadComponentScript("interfaces/BuildingAI.js");
 Engine.LoadComponentScript("GuiInterface.js");
 
+Resources = {
+	"GetCodes": () => ["food", "metal", "stone", "wood"],
+	"GetNames": () => ({
+		"food": "Food",
+		"metal": "Metal",
+		"stone": "Stone",
+		"wood": "Wood"
+	}),
+	"GetResource": resource => ({
+		"aiAnalysisInfluenceGroup":
+			resource == "food" ? "ignore" :
+			resource == "wood" ? "abundant" : "sparse"
+	})
+};
+
 var cmp = ConstructComponent(SYSTEM_ENTITY, "GuiInterface");
 
 
@@ -43,31 +60,33 @@ AddMock(SYSTEM_ENTITY, IID_Barter, {
 			"buy": { "food": 150 },
 			"sell": { "food": 25 }
 		};
-	}
+	},
+	PlayerHasMarket: function () { return false; }
 });
 
 AddMock(SYSTEM_ENTITY, IID_EndGameManager, {
-	GetGameType: function() { return "conquest"; }
+	GetGameType: function() { return "conquest"; },
+	GetAlliedVictory: function() { return false; }
 });
 
 AddMock(SYSTEM_ENTITY, IID_PlayerManager, {
 	GetNumPlayers: function() { return 2; },
-	GetPlayerByID: function(id) { TS_ASSERT(id === 0 || id === 1); return 100+id; },
+	GetPlayerByID: function(id) { TS_ASSERT(id === 0 || id === 1); return 100+id; }
 });
 
 AddMock(SYSTEM_ENTITY, IID_RangeManager, {
 	GetLosVisibility: function(ent, player) { return "visible"; },
-	GetLosCircular: function() { return false; },
+	GetLosCircular: function() { return false; }
 });
 
 AddMock(SYSTEM_ENTITY, IID_TemplateManager, {
 	GetCurrentTemplateName: function(ent) { return "example"; },
-	GetTemplate: function(name) { return ""; },
+	GetTemplate: function(name) { return ""; }
 });
 
 AddMock(SYSTEM_ENTITY, IID_Timer, {
 	GetTime: function() { return 0; },
-	SetTimeout: function(ent, iid, funcname, time, data) { return 0; },
+	SetTimeout: function(ent, iid, funcname, time, data) { return 0; }
 });
 
 AddMock(100, IID_Player, {
@@ -79,7 +98,7 @@ AddMock(100, IID_Player, {
 	GetPopulationLimit: function() { return 20; },
 	GetMaxPopulation: function() { return 200; },
 	GetResourceCounts: function() { return { food: 100 }; },
-	GetHeroes: function() { return []; },
+	GetPanelEntities: function() { return []; },
 	IsTrainingBlocked: function() { return false; },
 	GetState: function() { return "active"; },
 	GetTeam: function() { return -1; },
@@ -91,8 +110,10 @@ AddMock(100, IID_Player, {
 	IsNeutral: function() { return false; },
 	IsEnemy: function() { return true; },
 	GetDisabledTemplates: function() { return {}; },
+	GetDisabledTechnologies: function() { return {}; },
+	GetSpyCostMultiplier: function() { return 1; },
 	HasSharedDropsites: function() { return false; },
-	HasSharedLos: function() { return false; },
+	HasSharedLos: function() { return false; }
 });
 
 AddMock(100, IID_EntityLimits, {
@@ -104,54 +125,54 @@ AddMock(100, IID_EntityLimits, {
 AddMock(100, IID_TechnologyManager, {
 	IsTechnologyResearched: function(tech) { if (tech == "phase_village") return true; else return false; },
 	GetQueuedResearch: function() { return {}; },
-	GetStartedResearch: function() { return {}; },
+	GetStartedTechs: function() { return {}; },
 	GetResearchedTechs: function() { return {}; },
 	GetClassCounts: function() { return {}; },
 	GetTypeCountsByClass: function() { return {}; },
-	GetTechModifications: function() { return {}; },
+	GetTechModifications: function() { return {}; }
 });
 
 AddMock(100, IID_StatisticsTracker, {
-	GetBasicStatistics: function() { 
+	GetBasicStatistics: function() {
 		return {
 			"resourcesGathered": {
-				"food": 100,	
-				"wood": 0,	
-				"metal": 0,	
+				"food": 100,
+				"wood": 0,
+				"metal": 0,
 				"stone": 0,
-				"vegetarianFood": 0, 
+				"vegetarianFood": 0
 			},
 			"percentMapExplored": 10
 		};
 	},
-	GetStatistics: function() { 
+	GetSequences: function() {
 		return {
-			"unitsTrained": 10,
-			"unitsLost": 9,
-			"buildingsConstructed": 5,
-			"buildingsCaptured": 7,
-			"buildingsLost": 4,
-			"civCentresBuilt": 1,
+			"unitsTrained": [0, 10],
+			"unitsLost": [0, 42],
+			"buildingsConstructed": [1, 3],
+			"buildingsCaptured": [3, 7],
+			"buildingsLost": [3, 10],
+			"civCentresBuilt": [4, 10],
 			"resourcesGathered": {
-				"food": 100,	
-				"wood": 0,	
-				"metal": 0,	
-				"stone": 0,
-				"vegetarianFood": 0, 
+				"food": [5, 100],
+				"wood": [0, 0],
+				"metal": [0, 0],
+				"stone": [0, 0],
+				"vegetarianFood": [0, 0]
 			},
-			"treasuresCollected": 0,
-			"lootCollected": 0,
-			"percentMapExplored": 10,
-			"teamPercentMapExplored": 10,
-			"percentMapControlled": 10,
-			"teamPercentMapControlled": 10,
-			"peakPercentOfMapControlled": 10,
-			"teamPeakPercentOfMapControlled": 10
+			"treasuresCollected": [1, 20],
+			"lootCollected": [0, 2],
+			"percentMapExplored": [0, 10],
+			"teamPercentMapExplored": [0, 10],
+			"percentMapControlled": [0, 10],
+			"teamPercentMapControlled": [0, 10],
+			"peakPercentOfMapControlled": [0, 10],
+			"teamPeakPercentOfMapControlled": [0, 10]
 		};
 	},
 	IncreaseTrainedUnitsCounter: function() { return 1; },
 	IncreaseConstructedBuildingsCounter: function() { return 1; },
-	IncreaseBuiltCivCentresCounter: function() { return 1; },
+	IncreaseBuiltCivCentresCounter: function() { return 1; }
 });
 
 AddMock(101, IID_Player, {
@@ -163,7 +184,7 @@ AddMock(101, IID_Player, {
 	GetPopulationLimit: function() { return 30; },
 	GetMaxPopulation: function() { return 300; },
 	GetResourceCounts: function() { return { food: 200 }; },
-	GetHeroes: function() { return []; },
+	GetPanelEntities: function() { return []; },
 	IsTrainingBlocked: function() { return false; },
 	GetState: function() { return "active"; },
 	GetTeam: function() { return -1; },
@@ -175,8 +196,10 @@ AddMock(101, IID_Player, {
 	IsNeutral: function() { return false; },
 	IsEnemy: function() { return false; },
 	GetDisabledTemplates: function() { return {}; },
+	GetDisabledTechnologies: function() { return {}; },
+	GetSpyCostMultiplier: function() { return 1; },
 	HasSharedDropsites: function() { return false; },
-	HasSharedLos: function() { return false; },
+	HasSharedLos: function() { return false; }
 });
 
 AddMock(101, IID_EntityLimits, {
@@ -188,54 +211,54 @@ AddMock(101, IID_EntityLimits, {
 AddMock(101, IID_TechnologyManager, {
 		IsTechnologyResearched: function(tech) { if (tech == "phase_village") return true; else return false; },
 		GetQueuedResearch: function() { return {}; },
-		GetStartedResearch: function() { return {}; },
+		GetStartedTechs: function() { return {}; },
 		GetResearchedTechs: function() { return {}; },
 		GetClassCounts: function() { return {}; },
 		GetTypeCountsByClass: function() { return {}; },
-		GetTechModifications: function() { return {}; },
+		GetTechModifications: function() { return {}; }
 });
 
 AddMock(101, IID_StatisticsTracker, {
-	GetBasicStatistics: function() { 
+	GetBasicStatistics: function() {
 		return {
 			"resourcesGathered": {
-				"food": 100,	
-				"wood": 0,	
-				"metal": 0,	
+				"food": 100,
+				"wood": 0,
+				"metal": 0,
 				"stone": 0,
-				"vegetarianFood": 0, 
+				"vegetarianFood": 0
 			},
 			"percentMapExplored": 10
 		};
 	},
-	GetStatistics: function() { 
+	GetSequences: function() {
 		return {
-			"unitsTrained": 10,
-			"unitsLost": 9,
-			"buildingsConstructed": 5,
-			"buildingsCaptured": 7,
-			"buildingsLost": 4,
-			"civCentresBuilt": 1,
+			"unitsTrained": [0, 10],
+			"unitsLost": [0, 9],
+			"buildingsConstructed": [0, 5],
+			"buildingsCaptured": [0, 7],
+			"buildingsLost": [0, 4],
+			"civCentresBuilt": [0, 1],
 			"resourcesGathered": {
-				"food": 100,	
-				"wood": 0,	
-				"metal": 0,	
-				"stone": 0,
-				"vegetarianFood": 0, 
+				"food": [0, 100],
+				"wood": [0, 0],
+				"metal": [0, 0],
+				"stone": [0, 0],
+				"vegetarianFood": [0, 0]
 			},
-			"treasuresCollected": 0,
-			"lootCollected": 0,
-			"percentMapExplored": 10,
-			"teamPercentMapExplored": 10,
-			"percentMapControlled": 10,
-			"teamPercentMapControlled": 10,
-			"peakPercentOfMapControlled": 10,
-			"teamPeakPercentOfMapControlled": 10
+			"treasuresCollected": [0, 0],
+			"lootCollected": [0, 0],
+			"percentMapExplored": [0, 10],
+			"teamPercentMapExplored": [0, 10],
+			"percentMapControlled": [0, 10],
+			"teamPercentMapControlled": [0, 10],
+			"peakPercentOfMapControlled": [0, 10],
+			"teamPeakPercentOfMapControlled": [0, 10]
 		};
 	},
 	IncreaseTrainedUnitsCounter: function() { return 1; },
 	IncreaseConstructedBuildingsCounter: function() { return 1; },
-	IncreaseBuiltCivCentresCounter: function() { return 1; },
+	IncreaseBuiltCivCentresCounter: function() { return 1; }
 });
 
 // Note: property order matters when using TS_ASSERT_UNEVAL_EQUALS,
@@ -251,7 +274,7 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			popCount: 10,
 			popLimit: 20,
 			popMax: 200,
-			heroes: [],
+			panelEntities: [],
 			resourceCounts: { food: 100 },
 			trainingBlocked: false,
 			state: "active",
@@ -259,8 +282,10 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			teamsLocked: false,
 			cheatsEnabled: false,
 			disabledTemplates: {},
+			disabledTechnologies: {},
 			hasSharedDropsites: false,
 			hasSharedLos: false,
+			spyCostMultiplier: 1,
 			phase: "village",
 			isAlly: [false, false],
 			isMutualAlly: [false, false],
@@ -274,16 +299,21 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			researchedTechs: {},
 			classCounts: {},
 			typeCountsByClass: {},
+			canBarter: false,
+			barterPrices: {
+				"buy": { "food": 150 },
+				"sell": { "food": 25 }
+			},
 			statistics: {
 				resourcesGathered: {
 					food: 100,
 					wood: 0,
 					metal: 0,
 					stone: 0,
-					vegetarianFood: 0, 
+					vegetarianFood: 0
 				},
 				percentMapExplored: 10
-			},
+			}
 		},
 		{
 			name: "Player 2",
@@ -293,7 +323,7 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			popCount: 40,
 			popLimit: 30,
 			popMax: 300,
-			heroes: [],
+			panelEntities: [],
 			resourceCounts: { food: 200 },
 			trainingBlocked: false,
 			state: "active",
@@ -301,8 +331,10 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			teamsLocked: false,
 			cheatsEnabled: false,
 			disabledTemplates: {},
+			disabledTechnologies: {},
 			hasSharedDropsites: false,
 			hasSharedLos: false,
+			spyCostMultiplier: 1,
 			phase: "village",
 			isAlly: [true, true],
 			isMutualAlly: [false, false],
@@ -316,141 +348,210 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetSimulationState(), {
 			researchedTechs: {},
 			classCounts: {},
 			typeCountsByClass: {},
+			canBarter: false,
+			barterPrices: {
+				"buy": { "food": 150 },
+				"sell": { "food": 25 }
+			},
 			statistics: {
 				resourcesGathered: {
 					food: 100,
 					wood: 0,
 					metal: 0,
 					stone: 0,
-					vegetarianFood: 0, 
+					vegetarianFood: 0
 				},
 				percentMapExplored: 10
-			},
+			}
 		}
 	],
 	circularMap: false,
 	timeElapsed: 0,
 	gameType: "conquest",
-	barterPrices: {buy: {food: 150}, sell: {food: 25}}
+	alliedVictory: false,
+	"resources": {
+		"codes": ["food", "metal", "stone", "wood"],
+		"names": {
+			"food": "Food",
+			"metal": "Metal",
+			"stone": "Stone",
+			"wood": "Wood"
+		},
+		"aiInfluenceGroups": {
+			"food": "ignore",
+			"metal": "sparse",
+			"stone": "sparse",
+			"wood": "abundant"
+		}
+	}
 });
 
 TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedSimulationState(), {
-	players: [
+	"players": [
 		{
-			name: "Player 1",
-			civ: "gaia",
-			color: { r:1, g:1, b:1, a:1 },
-			controlsAll: false,
-			popCount: 10,
-			popLimit: 20,
-			popMax: 200,
-			heroes: [],
-			resourceCounts: { food: 100 },
-			trainingBlocked: false,
-			state: "active",
-			team: -1,
-			teamsLocked: false,
-			cheatsEnabled: false,
-			disabledTemplates: {},
-			hasSharedDropsites: false,
-			hasSharedLos: false,
-			phase: "village",
-			isAlly: [false, false],
-			isMutualAlly: [false, false],
-			isNeutral: [false, false],
-			isEnemy: [true, true],
-			entityLimits: {"Foo": 10},
-			entityCounts: {"Foo": 5},
-			entityLimitChangers: {"Foo": {}},
-			researchQueued: {},
-			researchStarted: {},
-			researchedTechs: {},
-			classCounts: {},
-			typeCountsByClass: {},
-			statistics: {
-				unitsTrained: 10,
-				unitsLost: 9,
-				buildingsConstructed: 5,
-				buildingsCaptured: 7,
-				buildingsLost: 4,
-				civCentresBuilt: 1,
-				resourcesGathered: {
-					food: 100,
-					wood: 0,
-					metal: 0,
-					stone: 0,
-					vegetarianFood: 0, 
-				},
-				treasuresCollected: 0,
-				lootCollected: 0,
-				percentMapExplored: 10,
-				teamPercentMapExplored: 10,
-				percentMapControlled: 10,
-				teamPercentMapControlled: 10,
-				peakPercentOfMapControlled: 10,
-				teamPeakPercentOfMapControlled: 10
+			"name": "Player 1",
+			"civ": "gaia",
+			"color": { "r":1, "g":1, "b":1, "a":1 },
+			"controlsAll": false,
+			"popCount": 10,
+			"popLimit": 20,
+			"popMax": 200,
+			"panelEntities": [],
+			"resourceCounts": { "food": 100 },
+			"trainingBlocked": false,
+			"state": "active",
+			"team": -1,
+			"teamsLocked": false,
+			"cheatsEnabled": false,
+			"disabledTemplates": {},
+			"disabledTechnologies": {},
+			"hasSharedDropsites": false,
+			"hasSharedLos": false,
+			"spyCostMultiplier": 1,
+			"phase": "village",
+			"isAlly": [false, false],
+			"isMutualAlly": [false, false],
+			"isNeutral": [false, false],
+			"isEnemy": [true, true],
+			"entityLimits": {"Foo": 10},
+			"entityCounts": {"Foo": 5},
+			"entityLimitChangers": {"Foo": {}},
+			"researchQueued": {},
+			"researchStarted": {},
+			"researchedTechs": {},
+			"classCounts": {},
+			"typeCountsByClass": {},
+			"canBarter": false,
+			"barterPrices": {
+				"buy": { "food": 150 },
+				"sell": { "food": 25 }
 			},
+			"statistics": {
+				"resourcesGathered": {
+					"food": 100,
+					"wood": 0,
+					"metal": 0,
+					"stone": 0,
+					"vegetarianFood": 0
+				},
+				"percentMapExplored": 10
+			},
+			"sequences": {
+				"unitsTrained": [0, 10],
+				"unitsLost": [0, 42],
+				"buildingsConstructed": [1, 3],
+				"buildingsCaptured": [3, 7],
+				"buildingsLost": [3, 10],
+				"civCentresBuilt": [4, 10],
+				"resourcesGathered": {
+					"food": [5, 100],
+					"wood": [0, 0],
+					"metal": [0, 0],
+					"stone": [0, 0],
+					"vegetarianFood": [0, 0]
+				},
+				"treasuresCollected": [1, 20],
+				"lootCollected": [0, 2],
+				"percentMapExplored": [0, 10],
+				"teamPercentMapExplored": [0, 10],
+				"percentMapControlled": [0, 10],
+				"teamPercentMapControlled": [0, 10],
+				"peakPercentOfMapControlled": [0, 10],
+				"teamPeakPercentOfMapControlled": [0, 10]
+			}
 		},
 		{
-			name: "Player 2",
-			civ: "mace",
-			color: { r:1, g:0, b:0, a:1 },
-			controlsAll: true,
-			popCount: 40,
-			popLimit: 30,
-			popMax: 300,
-			heroes: [],
-			resourceCounts: { food: 200 },
-			trainingBlocked: false,
-			state: "active",
-			team: -1,
-			teamsLocked: false,
-			cheatsEnabled: false,
-			disabledTemplates: {},
-			hasSharedDropsites: false,
-			hasSharedLos: false,
-			phase: "village",
-			isAlly: [true, true],
-			isMutualAlly: [false, false],
-			isNeutral: [false, false],
-			isEnemy: [false, false],
-			entityLimits: {"Bar": 20},
-			entityCounts: {"Bar": 0},
-			entityLimitChangers: {"Bar": {}},
-			researchQueued: {},
-			researchStarted: {},
-			researchedTechs: {},
-			classCounts: {},
-			typeCountsByClass: {},
-			statistics: {
-				unitsTrained: 10,
-				unitsLost: 9,
-				buildingsConstructed: 5,
-				buildingsCaptured: 7,
-				buildingsLost: 4,
-				civCentresBuilt: 1,
-				resourcesGathered: {
-					food: 100,
-					wood: 0,
-					metal: 0,
-					stone: 0,
-					vegetarianFood: 0, 
-				},
-				treasuresCollected: 0,
-				lootCollected: 0,
-				percentMapExplored: 10,
-				teamPercentMapExplored: 10,
-				percentMapControlled: 10,
-				teamPercentMapControlled: 10,
-				peakPercentOfMapControlled: 10,
-				teamPeakPercentOfMapControlled: 10
+			"name": "Player 2",
+			"civ": "mace",
+			"color": { "r":1, "g":0, "b":0, "a":1 },
+			"controlsAll": true,
+			"popCount": 40,
+			"popLimit": 30,
+			"popMax": 300,
+			"panelEntities": [],
+			"resourceCounts": { "food": 200 },
+			"trainingBlocked": false,
+			"state": "active",
+			"team": -1,
+			"teamsLocked": false,
+			"cheatsEnabled": false,
+			"disabledTemplates": {},
+			"disabledTechnologies": {},
+			"hasSharedDropsites": false,
+			"hasSharedLos": false,
+			"spyCostMultiplier": 1,
+			"phase": "village",
+			"isAlly": [true, true],
+			"isMutualAlly": [false, false],
+			"isNeutral": [false, false],
+			"isEnemy": [false, false],
+			"entityLimits": {"Bar": 20},
+			"entityCounts": {"Bar": 0},
+			"entityLimitChangers": {"Bar": {}},
+			"researchQueued": {},
+			"researchStarted": {},
+			"researchedTechs": {},
+			"classCounts": {},
+			"typeCountsByClass": {},
+			"canBarter": false,
+			"barterPrices": {
+				"buy": { "food": 150 },
+				"sell": { "food": 25 }
 			},
+			"statistics": {
+				"resourcesGathered": {
+					"food": 100,
+					"wood": 0,
+					"metal": 0,
+					"stone": 0,
+					"vegetarianFood": 0
+				},
+				"percentMapExplored": 10
+			},
+			"sequences": {
+				"unitsTrained": [0, 10],
+				"unitsLost": [0, 9],
+				"buildingsConstructed": [0, 5],
+				"buildingsCaptured": [0, 7],
+				"buildingsLost": [0, 4],
+				"civCentresBuilt": [0, 1],
+				"resourcesGathered": {
+					"food": [0, 100],
+					"wood": [0, 0],
+					"metal": [0, 0],
+					"stone": [0, 0],
+					"vegetarianFood": [0, 0]
+				},
+				"treasuresCollected": [0, 0],
+				"lootCollected": [0, 0],
+				"percentMapExplored": [0, 10],
+				"teamPercentMapExplored": [0, 10],
+				"percentMapControlled": [0, 10],
+				"teamPercentMapControlled": [0, 10],
+				"peakPercentOfMapControlled": [0, 10],
+				"teamPeakPercentOfMapControlled": [0, 10]
+			}
 		}
 	],
-	circularMap: false,
-	timeElapsed: 0,
-	gameType: "conquest",
-	barterPrices: {buy: {food: 150}, sell: {food: 25}}
+	"circularMap": false,
+	"timeElapsed": 0,
+	"gameType": "conquest",
+	"alliedVictory": false,
+	"resources": {
+		"codes": ["food", "metal", "stone", "wood"],
+		"names": {
+			"food": "Food",
+			"metal": "Metal",
+			"stone": "Stone",
+			"wood": "Wood"
+		},
+		"aiInfluenceGroups": {
+			"food": "ignore",
+			"metal": "sparse",
+			"stone": "sparse",
+			"wood": "abundant"
+		}
+	}
 });
 
 
@@ -465,7 +566,7 @@ AddMock(10, IID_Health, {
 	GetMaxHitpoints: function() { return 60; },
 	IsRepairable: function() { return false; },
 	IsUnhealable: function() { return false; },
-	IsUndeletable: function() { return false; },
+	IsUndeletable: function() { return false; }
 });
 
 AddMock(10, IID_Identity, {
@@ -473,7 +574,7 @@ AddMock(10, IID_Identity, {
 	GetVisibleClassesList: function() { return ["class3", "class4"]; },
 	GetRank: function() { return "foo"; },
 	GetSelectionGroupName: function() { return "Selection Group Name"; },
-	HasClass: function() { return true; },
+	HasClass: function() { return true; }
 });
 
 AddMock(10, IID_Position, {
@@ -486,7 +587,17 @@ AddMock(10, IID_Position, {
 	},
 	IsInWorld: function() {
 		return true;
-	},
+	}
+});
+
+AddMock(10, IID_ResourceTrickle, {
+	"GetTimer": () => 1250,
+	"GetRates": () => ({
+		"food": 2,
+		"wood": 3,
+		"stone": 5,
+		"metal": 9
+	})
 });
 
 // Note: property order matters when using TS_ASSERT_UNEVAL_EQUALS,
@@ -497,11 +608,12 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetEntityState(-1, 10), {
 	template: "example",
 	alertRaiser: null,
 	builder: true,
+	canGarrison: false,
 	identity: {
 		rank: "foo",
 		classes: ["class1", "class2"],
 		visibleClasses: ["class3", "class4"],
-		selectionGroupName: "Selection Group Name",
+		selectionGroupName: "Selection Group Name"
 	},
 	fogging: null,
 	foundation: null,
@@ -525,17 +637,15 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetEntityState(-1, 10), {
 	maxHitpoints: 60,
 	needsRepair: false,
 	needsHeal: true,
-	canDelete: true,
+	canDelete: true
 });
 
 TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedEntityState(-1, 10), {
 	armour: null,
 	attack: null,
-	barterMarket: {
-		prices: { "buy": {"food":150}, "sell": {"food":25} },
-	},
 	buildingAI: null,
 	heal: null,
+	isBarterMarket: true,
 	loot: null,
 	obstruction: null,
 	turretParent: null,
@@ -545,5 +655,14 @@ TS_ASSERT_UNEVAL_EQUALS(cmp.GetExtendedEntityState(-1, 10), {
 	resourceDropsite: null,
 	resourceGatherRates: null,
 	resourceSupply: null,
-	speed: null,
+	resourceTrickle: {
+		"interval": 1250,
+		"rates": {
+			"food": 2,
+			"wood": 3,
+			"stone": 5,
+			"metal": 9
+		}
+	},
+	speed: null
 });
