@@ -76,13 +76,6 @@ var g_Commands = {
 		Cheat(cmd);
 	},
 
-	"quit": function(player, cmd, data)
-	{
-		// Let the AI exit the game for testing purposes
-		var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-		cmpGuiInterface.PushNotification({ "type": "quit", "players": [player] });
-	},
-
 	"diplomacy": function(player, cmd, data)
 	{
 		let cmpCeasefireManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_CeasefireManager);
@@ -121,6 +114,9 @@ var g_Commands = {
 
 	"control-all": function(player, cmd, data)
 	{
+		if (!data.cmpPlayer.GetCheatsEnabled())
+			return;
+
 		var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 		cmpGuiInterface.PushNotification({
 			"type": "aichat",
@@ -133,11 +129,14 @@ var g_Commands = {
 
 	"reveal-map": function(player, cmd, data)
 	{
+		if (!data.cmpPlayer.GetCheatsEnabled())
+			return;
+
 		var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 		cmpGuiInterface.PushNotification({
 			"type": "aichat",
 			"players": [player],
-			"message": "(Cheat - reveal map)" // TODO: translate me!
+			"message": markForTranslation("(Cheat - reveal map)")
 		});
 
 		// Reveal the map for all players, not just the current player,
@@ -173,11 +172,12 @@ var g_Commands = {
 
 	"attack": function(player, cmd, data)
 	{
-		if (g_DebugCommands && !(IsOwnedByEnemyOfPlayer(player, cmd.target) || IsOwnedByNeutralOfPlayer(player, cmd.target)))
-			warn("Invalid command: attack target is not owned by enemy of player "+player+": "+uneval(cmd));
-
 		let allowCapture = cmd.allowCapture || cmd.allowCapture == null;
 
+		if (g_DebugCommands && !allowCapture &&
+		   !(IsOwnedByEnemyOfPlayer(player, cmd.target) || IsOwnedByNeutralOfPlayer(player, cmd.target)))
+			warn("Invalid command: attack target is not owned by enemy of player "+player+": "+uneval(cmd));
+
 		GetFormationUnitAIs(data.entities, player).forEach(cmpUnitAI => {
 			cmpUnitAI.Attack(cmd.target, cmd.queued, allowCapture);
 		});
@@ -260,9 +260,14 @@ var g_Commands = {
 
 	"train": function(player, cmd, data)
 	{
+		if (!Number.isInteger(cmd.count) || cmd.count <= 0)
+		{
+			warn("Invalid command: can't train " + uneval(cmd.count) + " units");
+			return;
+		}
+
 		// Check entity limits
-		var cmpTempMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-		var template = cmpTempMan.GetTemplate(cmd.template);
+		var template = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager).GetTemplate(cmd.template);
 		var unitCategory = null;
 		if (template.TrainingRestrictions)
 			unitCategory = template.TrainingRestrictions.Category;
@@ -270,7 +275,7 @@ var g_Commands = {
 		// Verify that the building(s) can be controlled by the player
 		if (data.entities.length <= 0)
 		{
-			 if (g_DebugCommands)
+			if (g_DebugCommands)
 				warn("Invalid command: training building(s) cannot be controlled by player "+player+": "+uneval(cmd));
 			return;
 		}
@@ -503,10 +508,6 @@ var g_Commands = {
 
 	"unload-template": function(player, cmd, data)
 	{
-		var index = cmd.template.indexOf("&");  // Templates for garrisoned units are extended
-		if (index == -1)
-			return;
-
 		var entities = FilterEntityListWithAllies(cmd.garrisonHolders, player, data.controlAllUnits);
 		for (let garrisonHolder of entities)
 		{
@@ -515,10 +516,10 @@ var g_Commands = {
 			{
 				// Only the owner of the garrisonHolder may unload entities from any owners
 				if (!IsOwnedByPlayer(player, garrisonHolder) && !data.controlAllUnits
-				    && player != +cmd.template.slice(1,index))
+				    && player != +cmd.owner)
 						continue;
 
-				if (!cmpGarrisonHolder.UnloadTemplate(cmd.template, cmd.all))
+				if (!cmpGarrisonHolder.UnloadTemplate(cmd.template, cmd.owner, cmd.all))
 					notifyUnloadFailure(player, garrisonHolder);
 			}
 		}
@@ -575,6 +576,9 @@ var g_Commands = {
 
 	"promote": function(player, cmd, data)
 	{
+		if (!data.cmpPlayer.GetCheatsEnabled())
+			return;
+
 		var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 		cmpGuiInterface.PushNotification({
 			"type": "aichat",
@@ -636,6 +640,10 @@ var g_Commands = {
 
 	"set-shading-color": function(player, cmd, data)
 	{
+		// Prevent multiplayer abuse
+		if (!data.cmpPlayer.IsAI())
+			return;
+
 		// Debug command to make an entity brightly colored
 		for (let ent of cmd.entities)
 		{
@@ -747,7 +755,7 @@ var g_Commands = {
 			"players": [player],
 			"message": "/allies " + markForTranslation("Attack against %(_player_)s requested."),
 			"translateParameters": ["_player_"],
-			"parameters": { "_player_": cmd.target }
+			"parameters": { "_player_": cmd.player }
 		});
 
 		// And send an attackRequest event to the AIs
@@ -756,6 +764,47 @@ var g_Commands = {
 			cmpAIInterface.PushEvent("AttackRequest", cmd);
 	},
 
+	"spy-request": function(player, cmd, data)
+	{
+		let cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
+		let ent = pickRandom(cmpRangeManager.GetEntitiesByPlayer(cmd.player).filter(ent => {
+			let cmpVisionSharing = Engine.QueryInterface(ent, IID_VisionSharing);
+			return cmpVisionSharing && cmpVisionSharing.IsBribable() && !cmpVisionSharing.ShareVisionWith(player);
+		}));
+
+		let cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
+		if (ent)
+		{
+			Engine.QueryInterface(ent, IID_VisionSharing).AddSpy(cmd.source);
+			cmpGUIInterface.PushNotification({
+				"type": "spy-response",
+				"players": [player],
+				"entity": ent
+			});
+		}
+		else
+			cmpGUIInterface.PushNotification({
+				"type": "text",
+				"players": [player],
+				"message": markForTranslation("There are no bribable units"),
+				"translateMessage": true
+			});
+	},
+
+	"diplomacy-request": function(player, cmd, data)
+	{
+		let cmpAIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_AIInterface);
+		if (cmpAIInterface)
+			cmpAIInterface.PushEvent("DiplomacyRequest", cmd);
+	},
+
+	"tribute-request": function(player, cmd, data)
+	{
+		let cmpAIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_AIInterface);
+		if (cmpAIInterface)
+			cmpAIInterface.PushEvent("TributeRequest", cmd);
+	},
+
 	"dialog-answer": function(player, cmd, data)
 	{
 		// Currently nothing. Triggers can read it anyway, and send this
@@ -967,10 +1016,9 @@ function TryConstructBuilding(player, cmpPlayer, controlAllUnits, cmd)
 	}
 
 	// If it's a dock, get the right angle.
-	var cmpTemplateMgr = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-	var template = cmpTemplateMgr.GetTemplate(cmd.template);
+	var template = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager).GetTemplate(cmd.template);
 	var angle = cmd.angle;
-	if (template.BuildRestrictions.Category === "Dock")
+	if (template.BuildRestrictions.PlacementType === "shore")
 	{
 		let angleDock = GetDockAngle(template, cmd.x, cmd.z);
 		if (angleDock !== undefined)
@@ -1372,6 +1420,7 @@ function GetFormationUnitAIs(ents, player, formationTemplate)
 			return [];
 
 		RemoveFromFormation(ents);
+		cmpUnitAI.SetLastFormationTemplate("formations/null");
 
 		return [ cmpUnitAI ];
 	}
@@ -1397,7 +1446,10 @@ function GetFormationUnitAIs(ents, player, formationTemplate)
 		else
 		{
 			if (nullFormation)
+			{
+				RemoveFromFormation([ent]);
 				cmpUnitAI.SetLastFormationTemplate("formations/null");
+			}
 			nonformedUnitAIs.push(cmpUnitAI);
 		}
 	}
@@ -1566,8 +1618,7 @@ function ClusterEntities(ents, separationDistance)
 
 function GetFormationRequirements(formationTemplate)
 {
-	var cmpTempManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-	var template = cmpTempManager.GetTemplate(formationTemplate);
+	var template = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager).GetTemplate(formationTemplate);
 	if (!template.Formation)
 		return false;
 
