@@ -45,8 +45,12 @@ function LoadMapSettings(settings)
 
 	let cmpEndGameManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_EndGameManager);
 	let gameTypeSettings = {};
-	if (settings.WonderDuration)
-		gameTypeSettings.wonderDuration = settings.WonderDuration * 60 * 1000;
+	if (settings.GameType && settings.GameType == "capture_the_relic")
+		gameTypeSettings.relicCount = settings.RelicCount;
+	if (settings.VictoryDuration)
+		gameTypeSettings.victoryDuration = settings.VictoryDuration * 60 * 1000;
+	if (settings.GameType && settings.GameType == "regicide")
+		gameTypeSettings.regicideGarrison = settings.RegicideGarrison;
 	if (settings.GameType)
 		cmpEndGameManager.SetGameType(settings.GameType, gameTypeSettings);
 
@@ -63,7 +67,7 @@ function LoadMapSettings(settings)
 			else
 				cmpGarrisonHolder.initGarrison = settings.Garrison[holder];
 		}
-	
+
 	let cmpCeasefireManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_CeasefireManager);
 	if (settings.Ceasefire)
 		cmpCeasefireManager.StartCeasefire(settings.Ceasefire * 60 * 1000);
