@@ -192,7 +192,7 @@ function generateModsList(listObjectName, mods)
 	});
 
 	// Update the list
-	var obj  = Engine.GetGUIObjectByName(listObjectName);
+	var obj = Engine.GetGUIObjectByName(listObjectName);
 	obj.list_name = names;
 	obj.list_modFolderName = folders;
 	obj.list_modLabel = labels;
@@ -269,19 +269,32 @@ function disableMod()
 
 function resetFilters()
 {
-	// Reset states of gui objects.
-	Engine.GetGUIObjectByName("modTypeFilter").selected = 0;
-	Engine.GetGUIObjectByName("negateFilter").checked = false;
 	Engine.GetGUIObjectByName("modGenericFilter").caption = "";
+	Engine.GetGUIObjectByName("negateFilter").checked = false;
 
 	// NOTE: Calling generateModsLists() is not needed as the selection changes and that calls applyFilters()
+	Engine.GetGUIObjectByName("modTypeFilter").selected = 0;
 }
 
 function applyFilters()
 {
-	Engine.GetGUIObjectByName("modsAvailableList").selected = -1;
-	Engine.GetGUIObjectByName("modsEnabledList").selected = -1;
+	// Save selected rows
+	let modsAvailableList = Engine.GetGUIObjectByName("modsAvailableList");
+	let modsEnabledList = Engine.GetGUIObjectByName("modsEnabledList");
+	let selectedModAvailableFolder = modsAvailableList.list_modFolderName[modsAvailableList.selected];
+	let selectedModEnabledFolder = modsEnabledList.list_modFolderName[modsEnabledList.selected];
+
+	// Remove selected rows to prevent a link to a non existing item
+	modsAvailableList.selected = -1;
+	modsEnabledList.selected = -1;
+
 	generateModsLists();
+
+	// Restore previously selected rows
+	modsAvailableList.selected = modsAvailableList.list_modFolderName.indexOf(selectedModAvailableFolder);
+	modsEnabledList.selected = modsEnabledList.list_modFolderName.indexOf(selectedModEnabledFolder);
+
+	Engine.GetGUIObjectByName("globalModDescription").caption = "";
 }
 
 /**
@@ -360,7 +373,7 @@ function moveCurrItem(objectName, up)
 function areDependenciesMet(mod)
 {
 	var guiObject = Engine.GetGUIObjectByName("message");
-	for each (var dependency in g_mods[mod].dependencies)
+	for (var dependency of g_mods[mod].dependencies)
 	{
 		if (isDependencyMet(dependency))
 			continue;
@@ -397,7 +410,7 @@ function isDependencyMet(dependency_idAndVersion, modsEnabled = null)
 		var dependency_id = dependency_idAndVersion;
 
 	// modsEnabled_key currently is the mod folder name.
-	for each (var modsEnabled_key in modsEnabled)
+	for (var modsEnabled_key of modsEnabled)
 	{
 		var modJson = g_mods[modsEnabled_key];
 		if (modJson.name != dependency_id)
