@@ -8,7 +8,7 @@ function init(data)
 	g_Controls = {};
 
 	var options = Engine.ReadJSONFile("gui/options/options.json");
-	for (let category of Object.keys(options))
+	for (let category in options)
 	{
 		let lastSize;
 		for (let i = 0; i < options[category].length; ++i)
@@ -21,6 +21,7 @@ function init(data)
 			let config = option.parameters.config;
 			g_Controls[config] = {
 				"control": setupControl(option, i, category),
+				"label": label,
 				"type": option.type,
 				"dependencies": option.dependencies || undefined,
 				"parameters": option.parameters
@@ -83,7 +84,7 @@ function setupControl(option, i, category)
 		let checked;
 		let keyRenderer;
 
-		for (let param of Object.keys(option.parameters))
+		for (let param in option.parameters)
 		{
 			switch (param)
 			{
@@ -130,8 +131,63 @@ function setupControl(option, i, category)
 		control.checked = checked;
 		control.onPress = onUpdate;
 		break;
+	case "slider":
+		control = Engine.GetGUIObjectByName(category + "Slider[" + i + "]");
+		let value;
+		let callbackFunction;
+		let minvalue;
+		let maxvalue;
+
+		for (let param in option.parameters)
+		{
+			switch (param)
+			{
+			case "config":
+				value = +Engine.ConfigDB_GetValue("user", key);
+				break;
+			case "function":
+				if (Engine[option.parameters.function])
+					callbackFunction = option.parameters.function;
+				break;
+			case "min":
+				minvalue = +option.parameters.min;
+				break;
+			case "max":
+				maxvalue = +option.parameters.max;
+				break;
+			default:
+				warn("Unknown option source type '" + param + "'");
+			}
+		}
+
+		onUpdate = function(key, callbackFunction, minvalue, maxvalue)
+		{
+			return function()
+			{
+				this.tooltip =
+					(option.tooltip ? translate(option.tooltip) + "\n" : "") +
+					sprintf(translateWithContext("slider number", "Value: %(val)s (min: %(min)s, max: %(max)s)"), {
+						"val": +this.value.toFixed(2),
+						"min": +minvalue.toFixed(2),
+						"max": +maxvalue.toFixed(2)
+					});
+
+				if (+Engine.ConfigDB_GetValue("user", key) === this.value)
+					return;
+				Engine.ConfigDB_CreateValue("user", key, this.value);
+				Engine.ConfigDB_SetChanges("user", true);
+				if (callbackFunction)
+					Engine[callbackFunction](+this.value);
+				updateOptionPanel();
+			};
+		}(key, callbackFunction, minvalue, maxvalue);
+
+		control.value = value;
+		control.max_value = maxvalue;
+		control.min_value = minvalue;
+		control.onValueChange = onUpdate;
+		break;
 	case "number":
-		// TODO: Slider
 	case "string":
 		control = Engine.GetGUIObjectByName(category + "Input[" + i + "]");
 		let caption;
@@ -139,7 +195,7 @@ function setupControl(option, i, category)
 		let minval;
 		let maxval;
 
-		for (let param of Object.keys(option.parameters))
+		for (let param in option.parameters)
 		{
 			switch (param)
 			{
@@ -190,22 +246,20 @@ function setupControl(option, i, category)
 	case "dropdown":
 		control = Engine.GetGUIObjectByName(category + "Dropdown[" + i + "]");
 		control.onSelectionChange = function(){};  // just the time to setup the value
+		let config;
 
-		for (let param of Object.keys(option.parameters))
+		for (let param in option.parameters)
 		{
 			switch (param)
 			{
 			case "config":
-				let val = +Engine.ConfigDB_GetValue("user", key);
-				if (key === "materialmgr.quality")
-					val = val > 5 ? 2 : val > 2 ? 1 : 0;
-				control.selected = val;
+				config = Engine.ConfigDB_GetValue("user", key);
 				break;
 			case "list":
-				control.list = option.parameters.list.map(e => translate(e));
-				break;
-			case "list_data":
-				control.list_data = option.parameters.list_data;
+				control.list = option.parameters.list.map(e => translate(e.label));
+				let values = option.parameters.list.map(e => e.value);
+				control.list_data = values;
+				control.selected = values.indexOf(config);
 				break;
 			default:
 				warn("Unknown option source type '" + param + "'");
@@ -216,10 +270,7 @@ function setupControl(option, i, category)
 		{
 			return function()
 			{
-				let val = this.selected;
-				if (key === "materialmgr.quality")
-					val = val == 0 ? 2 : val == 1 ? 5 : 8;
-				Engine.ConfigDB_CreateValue("user", key, val);
+				Engine.ConfigDB_CreateValue("user", key, this.list_data[this.selected]);
 				Engine.ConfigDB_SetChanges("user", true);
 				updateOptionPanel();
 			};
@@ -233,7 +284,12 @@ function setupControl(option, i, category)
 		break;
 	}
 	control.hidden = false;
-	control.tooltip = option.tooltip ? translate(option.tooltip) : "";
+
+	if (option.type == "slider")
+		control.onValueChange();
+	else
+		control.tooltip = option.tooltip ? translate(option.tooltip) : "";
+
 	return control;
 }
 
@@ -247,7 +303,10 @@ function updateOptionPanel()
 			continue;
 
 		for (let dependency of control.dependencies)
+		{
 			g_Controls[dependency].control.enabled = control.control.checked;
+			g_Controls[dependency].label.enabled = control.control.checked;
+		}
 	}
 
 	// And main buttons
@@ -306,7 +365,7 @@ function revertChanges()
 		// and the possible function calls (which are of number or string types)
 		if (control.parameters.function)
 		{
-			if (control.type !== "string" && control.type !== "number")
+			if (control.type !== "string" && control.type !== "number" && control.type !== "slider")
 			{
 				warn("Invalid type option " + control.type + " defined with function for " + item + ": will not be reverted");
 				continue;
