@@ -1,13 +1,21 @@
 var userReportEnabledText; // contains the original version with "$status" placeholder
 var currentSubmenuType; // contains submenu type
 const MARGIN = 4; // menu border size
-var g_BackgroundCode; // Background type.
 
 var g_ShowSplashScreens;
+
+/**
+ * Available backdrops
+ */
 var g_BackgroundLayerData = [];
 
-var g_T0 = +(new Date());
-var g_LastTickTime = new Date();
+/**
+ * Chosen backdrop
+ */
+var g_BackgroundLayerset;
+
+var g_T0 = Date.now();
+var g_LastTickTime = Date.now();
 
 function init(initData, hotloadData)
 {
@@ -26,13 +34,12 @@ function init(initData, hotloadData)
 	g_ShowSplashScreens = hotloadData ? hotloadData.showSplashScreens : initData && initData.isStartup;
 
 	// Pick a random background and initialise it
-	g_BackgroundCode = Math.floor(Math.random() * g_BackgroundLayerData.length);
-	var layerset = g_BackgroundLayerData[g_BackgroundCode];
-	for (var i = 0; i < layerset.length; ++i)
+	g_BackgroundLayerset = pickRandom(g_BackgroundLayerData);
+	for (let i = 0; i < g_BackgroundLayerset.length; ++i)
 	{
 		var guiObj = Engine.GetGUIObjectByName("background["+i+"]");
 		guiObj.hidden = false;
-		guiObj.sprite = layerset[i].sprite;
+		guiObj.sprite = g_BackgroundLayerset[i].sprite;
 		guiObj.z = i;
 	}
 }
@@ -44,10 +51,8 @@ function getHotloadData()
 
 function scrollBackgrounds()
 {
-	var layerset = g_BackgroundLayerData[g_BackgroundCode];
-	for (var i = 0; i < layerset.length; ++i)
+	for (let i = 0; i < g_BackgroundLayerset.length; ++i)
 	{
-		var layer = layerset[i];
 		var guiObj = Engine.GetGUIObjectByName("background["+i+"]");
 
 		var screen = guiObj.parent.getComputedSize();
@@ -55,9 +60,9 @@ function scrollBackgrounds()
 		var w = h * 16/9;
 		var iw = h * 2;
 
-		var time = (new Date() - g_T0) / 1000;
-		var offset = layer.offset(time, w);
-		if (layer.tiling)
+		var time = (Date.now() - g_T0) / 1000;
+		var offset = g_BackgroundLayerset[i].offset(time, w);
+		if (g_BackgroundLayerset[i].tiling)
 		{
 			var left = offset % iw;
 			if (left >= 0)
@@ -108,8 +113,8 @@ function formatUserReportStatus(status)
 
 function onTick()
 {
-	var now = new Date();
-	var tickLength = new Date() - g_LastTickTime;
+	let now = Date.now();
+	let tickLength = Date.now() - g_LastTickTime;
 	g_LastTickTime = now;
 
 	scrollBackgrounds();
@@ -117,19 +122,20 @@ function onTick()
 	updateMenuPosition(tickLength);
 
 	if (Engine.IsUserReportEnabled())
-	{
 		Engine.GetGUIObjectByName("userReportEnabledText").caption =
-			userReportEnabledText.replace(/\$status/,
-				formatUserReportStatus(Engine.GetUserReportStatus()));
-	}
+			'[font="sans-bold-16"]' + translate("Thank you for helping improve 0 A.D.!") + "[/font]\n\n" +
+			translate("Anonymous feedback is currently enabled.") + "\n" +
+			sprintf(translate("Status: %(status)s."), {
+				"status": formatUserReportStatus(Engine.GetUserReportStatus())
+			});
 
 	// Show splash screens here, so we don't interfere with main menu hotloading
 	if (g_ShowSplashScreens)
 	{
 		g_ShowSplashScreens = false;
 
-		if (Engine.ConfigDB_GetValue("user", "splashscreendisable") !== "true" &&
-		    Engine.ConfigDB_GetValue("user", "splashscreenversion") < Engine.GetFileMTime("gui/splashscreen/splashscreen.txt"))
+		if (Engine.ConfigDB_GetValue("user", "gui.splashscreen.enable") === "true" ||
+		    Engine.ConfigDB_GetValue("user", "gui.splashscreen.version") < Engine.GetFileMTime("gui/splashscreen/splashscreen.txt"))
 			Engine.PushGuiPage("page_splashscreen.xml", { "page": "splashscreen", callback : "SplashScreenClosedCallback" } );
 		else
 			ShowRenderPathMessage();
@@ -206,7 +212,7 @@ function openMenu(newSubmenu, position, buttonHeight, numButtons)
 	var submenu = Engine.GetGUIObjectByName("submenu");
 	var top = position - MARGIN;
 	var bottom = position + ((buttonHeight + MARGIN) * numButtons);
-	submenu.size = submenu.size.left + " " + top + " " + submenu.size.right + " " + bottom;
+	submenu.size = new GUISize(submenu.size.left, top, submenu.size.right, bottom);
 
 	// Blend in right border of main menu into the left border of the submenu
 	blendSubmenuIntoMain(top, bottom);
