@@ -166,21 +166,8 @@ for (var i = 0; i < numPlayers; i++)
 		false, clBaseResource, tX, tZ
 	);
 	createObjectGroup(group, 0, avoidClasses(clBaseResource,2));
-	var hillSize = PI * radius * radius;
-	// create grass tufts
-	var num = hillSize / 250;
-	for (var j = 0; j < num; j++)
-	{
-		var gAngle = randFloat(0, TWO_PI);
-		var gDist = radius - (5 + randInt(7));
-		var gX = round(fx + gDist * cos(gAngle));
-		var gZ = round(fz + gDist * sin(gAngle));
-		group = new SimpleGroup(
-			[new SimpleObject(aGrassShort, 2,5, 0,1, -PI/8,PI/8)],
-			false, clBaseResource, gX, gZ
-		);
-		createObjectGroup(group, 0);
-	}
+
+	placeDefaultDecoratives(fx, fz, aGrassShort, clBaseResource, radius);
 }
 
 RMS.SetProgress(10);
@@ -286,7 +273,7 @@ createFood
   [new SimpleObject(oBerryBush, 5,7, 0,4)]
  ],
  [
-  randInt(3, 12) * numPlayers + 2
+  randIntInclusive(3, 12) * numPlayers + 2
  ],
  avoidClasses(clForest, 0, clPlayer, 20, clHill, 1, clFood, 10)
 );
@@ -317,7 +304,7 @@ RMS.SetProgress(80);
 
 // create straggler trees
 var types = [oCarob, oAleppoPine];	// some variation
-createStragglerTrees(types, avoidClasses(clForest, 1, clHill, 1, clPlayer, 10, clMetal, 1, clRock, 1, clTreasure, 1));
+createStragglerTrees(types, avoidClasses(clForest, 1, clHill, 1, clPlayer, 10, clMetal, 6, clRock, 6, clTreasure, 4));
 
 // create hill trees
 log("Creating hill trees...");
@@ -344,4 +331,4 @@ setPPSaturation(0.56);
 setPPBloom(0.1);
 
 // Export map data
-ExportMap();
\ No newline at end of file
+ExportMap();
