@@ -106,7 +106,7 @@ for (var ix = 0; ix < mapSize; ix++)
 }
 
 // let's decide if we swap
-var swap = Math.round(Math.random());	// should return about 50/50 a 0 or a 1
+var swap = randBool();
 // let's create Corsica
 log("Creating Corsica");
 var CorsicaX = fractionToTiles(0.99);
@@ -181,7 +181,7 @@ var islandZ = [SardiniaZ,CorsicaZ];
 for (var island = 0; island <= 1; island++)
 	for (var i = 0; i <= nbCreeks; i++)
 	{
-		var radius = fractionToTiles( (Math.random()/17) + 0.49);
+		var radius = fractionToTiles(randFloat(0.49, 0.55));
 		var angle = PI*island + i*(PI/(nbCreeks*2));
 		if (swap)
 			angle += PI/2;
@@ -189,11 +189,8 @@ for (var island = 0; island <= 1; island++)
 		fz = radius * sin(angle);
 		fx = round(islandX[island] + fx);
 		fz = round(islandZ[island] + fz);
-		var size = scaleByMapSize(75,100);
-		if (Math.random() > 0.5)
-			size = Math.random() * 40 + 10;
-		else
-			size += Math.random() * 20;
+
+		var size = randBool() ? randFloat(10, 50) : scaleByMapSize(75, 100) + randFloat(0, 20);
 		var placer = new ClumpPlacer(size, 0.4, 0.01, 10, fx,fz);
 		var terrainPainter = new TerrainPainter(tSteepCliffs);
 		var elevationPainter = new SmoothElevationPainter(ELEVATION_SET, -5,0);	// base height is -10
@@ -224,6 +221,11 @@ for (var island = 0; island <= 1; island++)
 		var elevationPainter = new SmoothElevationPainter(ELEVATION_SET, -1,5);	// base height is -10
 		createArea(placer, [elevationPainter], null);
 
+		startX = Math.max(0, Math.min(startX, mapSize));
+		startZ = Math.max(0, Math.min(startZ, mapSize));
+		endX = Math.max(0, Math.min(endX, mapSize));
+		endZ = Math.max(0, Math.min(endZ, mapSize));
+
 		straightPassageMaker(startX, startZ,endX,endZ, 25, 18, 4,clShore,null);
 	}
 }
@@ -336,7 +338,7 @@ for (var i = 0; i < numPlayers; i++)
 	placeDefaultChicken(fx, fz, clBaseResource);
 
 	// create berry bushes
-	var bbAngle = randFloat(0, TWO_PI);
+	var bbAngle = randFloat(0, 2 * PI);
 	var bbDist = 11;
 	var bbX = round(fx + bbDist * cos(bbAngle));
 	var bbZ = round(fz + bbDist * sin(bbAngle));
@@ -591,10 +593,9 @@ createObjectGroups( group, 0, [avoidClasses(clCreek,3,clShore,3),stayClasses(clW
 RMS.SetProgress(90);
 
 RMS.SetProgress(95);
-if (randFloat(0,1) > 0.5)
-	setSkySet("cumulus");
-else
-	setSkySet("sunny");
+
+setSkySet(pickRandom(["cumulus", "sunny"]));
+
 setSunColor(0.8,0.66,0.48);
 setSunElevation(0.828932);
 if (!swap)
