@@ -180,7 +180,7 @@ for (var i = 0; i < numPlayers; i++)
 		true, clBaseResource, mX, mZ
 	);
 	createObjectGroup(group, 0);
-	var hillSize = PI * radius * radius;
+
 	// create starting trees
 	var num = 2;
 	var tAngle = randFloat(0, TWO_PI);
@@ -193,20 +193,7 @@ for (var i = 0; i < numPlayers; i++)
 	);
 	createObjectGroup(group, 0, avoidClasses(clBaseResource,2));
 
-	// create grass tufts
-	var num = hillSize / 250;
-	for (var j = 0; j < num; j++)
-	{
-		var gAngle = randFloat(0, TWO_PI);
-		var gDist = radius - (5 + randInt(7));
-		var gX = round(fx + gDist * cos(gAngle));
-		var gZ = round(fz + gDist * sin(gAngle));
-		group = new SimpleGroup(
-			[new SimpleObject(aGrassShort, 2,5, 0,1, -PI/8,PI/8)],
-			false, clBaseResource, gX, gZ
-		);
-		createObjectGroup(group, 0);
-	}
+	placeDefaultDecoratives(fx, fz, aGrassShort, clBaseResource, radius);
 }
 
 RMS.SetProgress(20);
@@ -301,7 +288,7 @@ var elevationPainter = new SmoothElevationPainter(
 	4,				// elevation
 	4				// blend radius
 );
-createArea(placer, [terrainPainter, elevationPainter], null);
+createArea(placer, [terrainPainter, elevationPainter, unPaintClass(clWater)], null);
 
 var fx = fractionToTiles(0.5);
 var fz = fractionToTiles(0.5);
@@ -320,7 +307,7 @@ var elevationPainter = new SmoothElevationPainter(
 	-2,				// elevation
 	3				// blend radius
 );
-createArea(placer, [terrainPainter, elevationPainter], null);
+createArea(placer, [terrainPainter, elevationPainter, paintClass(clWater)], null);
 
 var placer = new ClumpPlacer((mapSize - 50) * (mapSize - 50) * 0.09, 1, 1, 10, ix, iz);
 var terrainPainter = new LayeredPainter(
@@ -367,7 +354,7 @@ for (var i = 0; i < numPlayers; i++)
 	createArea(placer, painter, null);
 }
 
-if (randInt(1,2) == 1)
+if (randBool())
 	createHills([tMainTerrain, tCliff, tHill], avoidClasses(clPlayer, 20, clHill, 15, clWater, 2), clHill, scaleByMapSize(1, 4) * numPlayers);
 else
 	createMountains(tCliff, avoidClasses(clPlayer, 20, clHill, 15, clWater, 2), clHill, scaleByMapSize(1, 4) * numPlayers);
@@ -417,6 +404,14 @@ createMines(
  clMetal
 );
 
+log("Creating fish...");
+createObjectGroups(
+	new SimpleGroup([new SimpleObject(oFish, 1,1, 0,3)], true, clFood),
+	0,
+	[stayClasses(clWater, 8), avoidClasses(clFood, 14)],
+	scaleByMapSize(400, 2000),
+	100);
+
 RMS.SetProgress(65);
 
 // create decoration
@@ -473,8 +468,8 @@ createFood
 
 // create straggler trees
 var types = [oTree1, oTree2, oTree4, oTree3];	// some variation
-createStragglerTrees(types, avoidClasses(clWater, 5, clForest, 7, clHill, 1, clPlayer, 12, clMetal, 1, clRock, 1));
+createStragglerTrees(types, avoidClasses(clWater, 5, clForest, 7, clHill, 1, clPlayer, 12, clMetal, 6, clRock, 6));
 
 
 // Export map data
-ExportMap();
\ No newline at end of file
+ExportMap();
