@@ -42,7 +42,7 @@ SimpleTerrain.prototype.constructor = SimpleTerrain;
 SimpleTerrain.prototype.placeNew = function(x, z)
 {
 	if (this.treeType !== undefined && g_Map.validT(round(x), round(z), MAP_BORDER_WIDTH))
-		g_Map.terrainObjects[x][z] = new Entity(this.treeType, 0, x+0.5, z+0.5, randFloat()*TWO_PI);
+		g_Map.terrainObjects[x][z] = new Entity(this.treeType, 0, x + 0.5, z + 0.5, randFloat(0, 2 * PI));
 
 	g_Map.texture[x][z] = g_Map.getTextureID(this.texture);
 };
@@ -68,5 +68,5 @@ RandomTerrain.prototype = new Terrain();
 RandomTerrain.prototype.constructor = RandomTerrain;
 RandomTerrain.prototype.placeNew = function(x, z)
 {
-	this.terrains[randInt(this.terrains.length)].placeNew(x, z);
+	pickRandom(this.terrains).placeNew(x, z);
 };
