@@ -81,7 +81,7 @@ for (var ix = 0; ix < mapSize; ix++)
 	}
 }
 
-var md = randInt(2,13);
+var md = randIntInclusive(2,13);
 var needsAdditionalWood = false;
 //*****************************************************************************************************************************
 if (md == 2) //continent
@@ -146,7 +146,7 @@ if (md == 2) //continent
 
 	var clPeninsulaSteam = createTileClass();
 
-	if (randInt(1,3)==1) // peninsula
+	if (randBool(1/3)) // peninsula
 	{
 		var angle = randFloat(0, TWO_PI);
 
@@ -195,7 +195,7 @@ playerIDs = primeSortPlayers(sortPlayers(playerIDs));
 	var playerPos = new Array(numPlayers);
 	var iop = 0;
 
-	var mdd1 = randInt(1,2);
+	var mdd1 = randIntInclusive(1,2);
 	if (mdd1 == 1) //vertical
 	{
 		for (var i = 0; i < numPlayers; i++)
@@ -351,11 +351,11 @@ playerIDs = primeSortPlayers(sortPlayers(playerIDs));
 	// linked
 	if (mdd1 == 1) //vertical
 	{
-		var placer = new PathPlacer(1, fractionToTiles(0.5), fractionToTiles(0.99), fractionToTiles(0.5), scaleByMapSize(randInt(16,24),randInt(100,140)), 0.5, 3*(scaleByMapSize(1,4)), 0.1, 0.01);
+		var placer = new PathPlacer(1, fractionToTiles(0.5), fractionToTiles(0.99), fractionToTiles(0.5), scaleByMapSize(randIntInclusive(16,24),randIntInclusive(100,140)), 0.5, 3*(scaleByMapSize(1,4)), 0.1, 0.01);
 	}
 	else
 	{
-		var placer = new PathPlacer(fractionToTiles(0.5), 1, fractionToTiles(0.5), fractionToTiles(0.99), scaleByMapSize(randInt(16,24),randInt(100,140)), 0.5, 3*(scaleByMapSize(1,4)), 0.1, 0.01);
+		var placer = new PathPlacer(fractionToTiles(0.5), 1, fractionToTiles(0.5), fractionToTiles(0.99), scaleByMapSize(randIntInclusive(16,24),randIntInclusive(100,140)), 0.5, 3*(scaleByMapSize(1,4)), 0.1, 0.01);
 	}
 	var terrainPainter = new LayeredPainter(
 		[tMainTerrain, tMainTerrain, tMainTerrain],		// terrains
@@ -368,7 +368,7 @@ playerIDs = primeSortPlayers(sortPlayers(playerIDs));
 	);
 	createArea(placer, [terrainPainter, elevationPainter, unPaintClass(clWater)], null);
 
-	var mdd2 = randInt(1,7);
+	var mdd2 = randIntInclusive(1,7);
 	if (mdd2 == 1)
 	{
 		// create islands
@@ -383,14 +383,14 @@ playerIDs = primeSortPlayers(sortPlayers(playerIDs));
 			placer,
 			[terrainPainter, elevationPainter, paintClass(clLand)],
 			avoidClasses(clLand, 3, clPlayer, 3),
-			scaleByMapSize(2, 5)*randInt(8,14)
+			scaleByMapSize(2, 5)*randIntInclusive(8,14)
 		);
 	}
 	else if (mdd2 == 2)
 	{
 		// create extentions
 		log("Creating extentions...");
-		placer = new ClumpPlacer(randInt(scaleByMapSize(13,24),scaleByMapSize(24,45))*randInt(scaleByMapSize(13,24),scaleByMapSize(24,45)), 0.80, 0.1, 10);
+		placer = new ClumpPlacer(randIntInclusive(scaleByMapSize(13,24),scaleByMapSize(24,45))*randIntInclusive(scaleByMapSize(13,24),scaleByMapSize(24,45)), 0.80, 0.1, 10);
 		terrainPainter = new LayeredPainter(
 			[tMainTerrain, tMainTerrain],		// terrains
 			[2]								// widths
@@ -400,7 +400,7 @@ playerIDs = primeSortPlayers(sortPlayers(playerIDs));
 			placer,
 			[terrainPainter, elevationPainter, paintClass(clLand)],
 			null,
-			scaleByMapSize(2, 5)*randInt(8,14)
+			scaleByMapSize(2, 5)*randIntInclusive(8,14)
 		);
 	}
 }
@@ -433,7 +433,7 @@ else if (md == 4) //central river
 	var playerAngle = new Array(numPlayers);
 	var playerPos = new Array(numPlayers);
 	var iop = 0;
-	var mdd1 = randInt(1,2);
+	var mdd1 = randIntInclusive(1,2);
 	if (mdd1 == 1) //horizontal
 	{
 		for (var i = 0; i < numPlayers; i++)
@@ -523,7 +523,7 @@ else if (md == 4) //central river
 	// create the shallows of the main river
 	log("Creating the shallows of the main river");
 
-	for (var i = 0; i <= randInt(1, scaleByMapSize(4,8)); i++)
+	for (var i = 0; i <= randIntInclusive(1, scaleByMapSize(4,8)); i++)
 	{
 		var cLocation = randFloat(0.15,0.85);
 		if (mdd1 == 1)
@@ -532,7 +532,7 @@ else if (md == 4) //central river
 			passageMaker(floor(fractionToTiles(0.35)), floor(fractionToTiles(cLocation)), floor(fractionToTiles(0.65)), floor(fractionToTiles(cLocation)), scaleByMapSize(4,8), -2, -2, 2, clShallow, undefined, -4);
 	}
 
-	if (randInt(1,2) == 1)
+	if (randBool())
 	{
 		for (var i = 0; i < numPlayers; i++)
 		{
@@ -549,10 +549,10 @@ else if (md == 4) //central river
 		// create tributaries
 		log("Creating tributaries");
 
-		for (var i = 0; i <= randInt(8, (scaleByMapSize(12,20))); i++)
+		for (var i = 0; i <= randIntInclusive(8, (scaleByMapSize(12,20))); i++)
 		{
 			var cLocation = randFloat(0.05,0.95);
-			var tang = randFloat(PI*0.2, PI*0.8)*((randInt(2)-0.5)*2);
+			var tang = randFloat(PI*0.2, PI*0.8)*((randIntInclusive(0, 1)-0.5)*2);
 			if (tang > 0)
 			{
 				var cDistance = 0.05;
@@ -597,17 +597,6 @@ else if (md == 4) //central river
 				}
 			}
 		}
-
-		if (mdd1 == 1)
-		{
-			passageMaker(floor(fractionToTiles(0.2)), floor(fractionToTiles(0.25)), floor(fractionToTiles(0.8)), floor(fractionToTiles(0.25)), scaleByMapSize(4,8), -2, -2, 2, clShallow, undefined, -4);
-			passageMaker(floor(fractionToTiles(0.2)), floor(fractionToTiles(0.75)), floor(fractionToTiles(0.8)), floor(fractionToTiles(0.75)), scaleByMapSize(4,8), -2, -2, 2, clShallow, undefined, -4);
-		}
-		else
-		{
-			passageMaker(floor(fractionToTiles(0.25)), floor(fractionToTiles(0.2)), floor(fractionToTiles(0.25)), floor(fractionToTiles(0.8)), scaleByMapSize(4,8), -2, -2, 2, clShallow, undefined, -4);
-			passageMaker(floor(fractionToTiles(0.75)), floor(fractionToTiles(0.2)), floor(fractionToTiles(0.75)), floor(fractionToTiles(0.8)), scaleByMapSize(4,8), -2, -2, 2, clShallow, undefined, -4);
-		}
 	}
 }
 //********************************************************************************************************
@@ -657,7 +646,7 @@ else if (md == 5) //rivers and lake
 				setHeight(ix, iz, 3);
 		}
 	}
-	var mdd1 = randInt(1,2);
+	var mdd1 = randIntInclusive(1,2);
 	if (mdd1 == 1) //lake
 	{
 		var fx = fractionToTiles(0.5);
@@ -696,7 +685,7 @@ else if (md == 5) //rivers and lake
 		);
 	}
 
-	if ((randInt(1,3) == 1)&&(mdd1 == 1))//island
+	if (randBool(1/3) &&(mdd1 == 1))//island
 	{
 		var placer = new ClumpPlacer(mapArea * 0.006 * lSize, 0.7, 0.1, 10, ix, iz);
 		var terrainPainter = new LayeredPainter(
@@ -725,7 +714,7 @@ else if (md == 6) //edge seas
 		}
 	}
 
-	var mdd1 = randInt(1,2);
+	var mdd1 = randIntInclusive(1,2);
 
 	// randomize player order
 	var playerIDs = [];
@@ -766,7 +755,7 @@ else if (md == 6) //edge seas
 		addToClass(ix, iz-5, clPlayer);
 	}
 
-	var mdd2 = randInt(1,3);
+	var mdd2 = randIntInclusive(1,3);
 	var fadedistance = 7;
 
 	if (mdd1 == 1)
@@ -901,7 +890,7 @@ else if (md == 6) //edge seas
 		scaleByMapSize(12, 130) * 2, 150
 	);
 
-	var mdd3 = randInt(1,5);
+	var mdd3 = randIntInclusive(1,5);
 	if (mdd3 == 1)
 	{
 		// create islands
@@ -916,7 +905,7 @@ else if (md == 6) //edge seas
 			placer,
 			[terrainPainter, elevationPainter, paintClass(clLand)],
 			avoidClasses(clLand, 3, clPlayer, 3),
-			scaleByMapSize(2, 5)*randInt(8,14)
+			scaleByMapSize(2, 5)*randIntInclusive(8,14)
 		);
 	}
 	else if (mdd3 == 2)
@@ -933,7 +922,7 @@ else if (md == 6) //edge seas
 			placer,
 			[terrainPainter, elevationPainter, paintClass(clLand)],
 			null,
-			scaleByMapSize(2, 5)*randInt(8,14)
+			scaleByMapSize(2, 5)*randIntInclusive(8,14)
 		);
 	}
 }
@@ -1115,7 +1104,7 @@ else if (md == 8) //lakes
 		if (!lakeAreaLen)
 			break;
 
-		chosenPoint = lakeAreas[randInt(lakeAreaLen)];
+		chosenPoint = pickRandom(lakeAreas);
 
 		placer = new ChainPlacer(1, floor(scaleByMapSize(4, 8)), floor(scaleByMapSize(40, 180)), 0.7, chosenPoint[0], chosenPoint[1]);
 		terrainPainter = new LayeredPainter(
@@ -1205,7 +1194,7 @@ else if (md == 9) //passes
 		var elevationPainter = new SmoothElevationPainter(ELEVATION_SET, 3, 2);
 		createArea(placer, [painter, elevationPainter], null);
 	}
-	var mdd1 = randInt (1,3);
+	var mdd1 = randIntInclusive (1,3);
 	if (mdd1 <= 2)
 	{
 		var fx = fractionToTiles(0.5);
@@ -1501,20 +1490,7 @@ for (var i = 0; i < numPlayers; i++)
 	);
 	createObjectGroup(group, 0, avoidClasses(clBaseResource,2));
 
-	// create grass tufts
-	var num = hillSize / 250;
-	for (var j = 0; j < num; j++)
-	{
-		var gAngle = randFloat(0, TWO_PI);
-		var gDist = radius - (5 + randInt(7));
-		var gX = round(fx + gDist * cos(gAngle));
-		var gZ = round(fz + gDist * sin(gAngle));
-		group = new SimpleGroup(
-			[new SimpleObject(aGrassShort, 2,5, 0,1, -PI/8,PI/8)],
-			false, clBaseResource, gX, gZ
-		);
-		createObjectGroup(group, 0);
-	}
+	placeDefaultDecoratives(fx, fz, aGrassShort, clBaseResource, radius);
 }
 
 for (var i = 0; i < numPlayers; i++)
@@ -1538,7 +1514,7 @@ createAreas(
 	placer,
 	painter,
 	[avoidClasses(clWater, 2, clPlayer, 10), stayClasses(clLand, 3)],
-	randInt(0,scaleByMapSize(200, 400))
+	randIntInclusive(0,scaleByMapSize(200, 400))
 );
 
 // create hills
@@ -1552,8 +1528,8 @@ elevationPainter = new SmoothElevationPainter(ELEVATION_SET, 18, 2);
 createAreas(
 	placer,
 	[terrainPainter, elevationPainter, paintClass(clHill)],
-	[avoidClasses(clPlayer, 20, clHill, randInt(6, 18)), stayClasses(clLand, 0)],
-	randInt(0, scaleByMapSize(4, 8))*randInt(1, scaleByMapSize(4, 9))
+	[avoidClasses(clPlayer, 20, clHill, randIntInclusive(6, 18)), stayClasses(clLand, 0)],
+	randIntInclusive(0, scaleByMapSize(4, 8))*randIntInclusive(1, scaleByMapSize(4, 9))
 );
 
 var multiplier = sqrt(randFloat(0.5,1.2)*randFloat(0.5,1.2));
@@ -1601,7 +1577,7 @@ if (random_terrain != g_BiomeSavanna)
 		createAreas(
 			placer,
 			[painter, paintClass(clForest)],
-			[avoidClasses(clPlayer, 20, clForest, randInt(5, 15), clHill, 0), stayClasses(clLand, 4)],
+			[avoidClasses(clPlayer, 20, clForest, randIntInclusive(5, 15), clHill, 0), stayClasses(clLand, 4)],
 			num
 		);
 	}
@@ -1651,14 +1627,14 @@ log("Creating stone mines...");
 group = new SimpleGroup([new SimpleObject(oStoneSmall, 0,2, 0,4), new SimpleObject(oStoneLarge, 1,1, 0,4)], true, clRock);
 createObjectGroups(group, 0,
 	[avoidClasses(clForest, 1, clPlayer, 20, clRock, 10, clHill, 1), stayClasses(clLand, 4)],
-	randInt(scaleByMapSize(2,9),scaleByMapSize(9,40)), 100
+	randIntInclusive(scaleByMapSize(2,9),scaleByMapSize(9,40)), 100
 );
 
 // create small stone quarries
 group = new SimpleGroup([new SimpleObject(oStoneSmall, 2,5, 1,3)], true, clRock);
 createObjectGroups(group, 0,
 	[avoidClasses(clForest, 1, clPlayer, 20, clRock, 10, clHill, 1), stayClasses(clLand, 4)],
-	randInt(scaleByMapSize(2,9),scaleByMapSize(9,40)), 100
+	randIntInclusive(scaleByMapSize(2,9),scaleByMapSize(9,40)), 100
 );
 
 log("Creating metal mines...");
@@ -1666,7 +1642,7 @@ log("Creating metal mines...");
 group = new SimpleGroup([new SimpleObject(oMetalLarge, 1,1, 0,4)], true, clMetal);
 createObjectGroups(group, 0,
 	[avoidClasses(clForest, 1, clPlayer, 20, clMetal, 10, clRock, 5, clHill, 1), stayClasses(clLand, 4)],
-	randInt(scaleByMapSize(2,9),scaleByMapSize(9,40)), 100
+	randIntInclusive(scaleByMapSize(2,9),scaleByMapSize(9,40)), 100
 );
 
 RMS.SetProgress(65);
@@ -1706,7 +1682,7 @@ group = new SimpleGroup(
 );
 createObjectGroups(group, 0,
 	[avoidClasses(clWater, 0, clForest, 0, clPlayer, 20, clHill, 1, clFood, 20), stayClasses(clLand, 4)],
-	randInt(numPlayers+3, 5*numPlayers+4), 50
+	randIntInclusive(numPlayers+3, 5*numPlayers+4), 50
 );
 
 // create berry bush
@@ -1717,7 +1693,7 @@ group = new SimpleGroup(
 );
 createObjectGroups(group, 0,
 	[avoidClasses(clWater, 0, clForest, 0, clPlayer, 20, clHill, 1, clFood, 20), stayClasses(clLand, 4)],
-	randInt(1, 4) * numPlayers + 2, 50
+	randIntInclusive(1, 4) * numPlayers + 2, 50
 );
 
 RMS.SetProgress(75);
@@ -1730,7 +1706,7 @@ group = new SimpleGroup(
 );
 createObjectGroups(group, 0,
 	[avoidClasses(clWater, 0, clForest, 0, clPlayer, 20, clHill, 1, clFood, 20), stayClasses(clLand, 4)],
-	randInt(numPlayers+3, 5*numPlayers+4), 50
+	randIntInclusive(numPlayers+3, 5*numPlayers+4), 50
 );
 
 // create fish
@@ -1741,7 +1717,7 @@ group = new SimpleGroup(
 );
 createObjectGroups(group, 0,
 	avoidClasses(clLand, 5, clForest, 0, clPlayer, 0, clHill, 0, clFood, 20),
-	randInt(15, 40) * numPlayers, 60
+	randIntInclusive(15, 40) * numPlayers, 60
 );
 
 RMS.SetProgress(85);
@@ -1758,7 +1734,7 @@ for (var i = 0; i < types.length; ++i)
 		true, clForest
 	);
 	createObjectGroups(group, 0,
-		[avoidClasses(clWater, 1, clForest, 7, clHill, 1, clPlayer, 0, clMetal, 1, clRock, 1), stayClasses(clLand, 4)],
+		[avoidClasses(clWater, 1, clForest, 7, clHill, 1, clPlayer, 0, clMetal, 6, clRock, 6), stayClasses(clLand, 4)],
 		num
 	);
 }
@@ -1811,20 +1787,7 @@ createObjectGroups(group, 0,
 	planetm * scaleByMapSize(13, 200), 50
 );
 
-random_terrain = randInt(1,6);
-if (random_terrain == 1)
-	setSkySet("cirrus");
-else if (random_terrain == 2)
-	setSkySet("cumulus");
-else if (random_terrain == 3)
-	setSkySet("sunny");
-else if (random_terrain == 4)
-	setSkySet("sunny 1");
-else if (random_terrain == 5)
-	setSkySet("mountainous");
-else if (random_terrain == 6)
-	setSkySet("stratus");
-
+setSkySet(pickRandom(["cirrus", "cumulus", "sunny", "sunny 1", "mountainous", "stratus"]));
 setSunRotation(randFloat(0, TWO_PI));
 setSunElevation(randFloat(PI/ 5, PI / 3));
 
