@@ -57,8 +57,12 @@ function startReplay()
  */
 function reallyStartVisualReplay(replayDirectory)
 {
-	// TODO: enhancement: restore filter settings and selected replay when returning from the summary screen.
-	Engine.StartVisualReplay(replayDirectory);
+	if (!Engine.StartVisualReplay(replayDirectory))
+	{
+		warn('Replay "' + escapeText(Engine.GetReplayDirectoryName(replayDirectory)) + '" not found! Please click on reload cache.');
+		return;
+	}
+
 	Engine.SwitchGuiPage("page_loading.xml", {
 		"attribs": Engine.GetReplayAttributes(replayDirectory),
 		"isNetworked": false,
@@ -127,6 +131,12 @@ function showReplaySummary()
 	});
 }
 
+function reloadCache()
+{
+	let selected = Engine.GetGUIObjectByName("replaySelection").selected;
+	loadReplays(selected > -1 ? createReplaySelectionData(g_ReplaysFiltered[selected].directory) : "", true);
+}
+
 /**
  * Callback.
  */
@@ -154,7 +164,8 @@ function deleteReplay()
 
 	messageBox(
 		500, 200,
-		translate("Are you sure you want to delete this replay permanently?") + "\n" + escapeText(replay.file),
+		translate("Are you sure you want to delete this replay permanently?") + "\n" +
+			escapeText(Engine.GetReplayDirectoryName(replay.directory)),
 		translate("Delete replay"),
 		[translate("No"), translate("Yes")],
 		[null, function() { reallyDeleteReplay(replay.directory); }]
@@ -182,7 +193,7 @@ function reallyDeleteReplay(replayDirectory)
 	var selectedIndex = replaySelection.selected;
 
 	if (!Engine.DeleteReplay(replayDirectory))
-		error(sprintf("Could not delete replay '%(id)s'", { "id": replayDirectory }));
+		error("Could not delete replay!");
 
 	// Refresh replay list
 	init();
