@@ -7,49 +7,105 @@ const g_CivData = loadCivData();
 
 function init()
 {
-	let gameSelection = Engine.GetGUIObjectByName("gameSelection");
-
-	let savedGames = Engine.GetSavedGames().sort(sortDecreasingDate);
-	if (!savedGames.length)
-	{
-		gameSelection.list = [translate("No saved games found")];
-		gameSelection.selected = -1;
-		selectionChanged();
-		Engine.GetGUIObjectByName("loadGameButton").enabled = false;
-		Engine.GetGUIObjectByName("deleteGameButton").enabled = false;
-		return;
-	}
+	let savedGames = Engine.GetSavedGames();
 
 	// Get current game version and loaded mods
 	let engineInfo = Engine.GetEngineInfo();
 
-	g_SavedGamesMetadata = savedGames.map(game => game.metadata);
+	if (Engine.GetGUIObjectByName("compatibilityFilter").checked)
+		savedGames = savedGames.filter(game => isCompatibleSavegame(game.metadata, engineInfo));
 
-	gameSelection.list = savedGames.map(game => generateLabel(game.metadata, engineInfo));
-	gameSelection.list_data = savedGames.map(game => game.id);
+	let gameSelection = Engine.GetGUIObjectByName("gameSelection");
+	gameSelection.enabled = !!savedGames.length;
+	Engine.GetGUIObjectByName("gameSelectionFeedback").hidden = !!savedGames.length;
 
-	if (gameSelection.selected == -1)
+	let selectedGameId = gameSelection.list_data[gameSelection.selected];
+
+	// Save metadata for the detailed view
+	g_SavedGamesMetadata = savedGames.map(game =>
+	{
+		game.metadata.id = game.id;
+		return game.metadata;
+	});
+
+	const sortKey = gameSelection.selected_column;
+	const sortOrder = gameSelection.selected_column_order;
+	g_SavedGamesMetadata = g_SavedGamesMetadata.sort((a, b) =>
+	{
+		let cmpA, cmpB;
+		switch (sortKey)
+		{
+		case 'date':
+			cmpA = +a.time;
+			cmpB = +b.time;
+			break;
+		case 'mapName':
+			cmpA = translate(a.initAttributes.settings.Name);
+			cmpB = translate(b.initAttributes.settings.Name);
+			break;
+		case 'mapType':
+			cmpA = translateMapType(a.initAttributes.mapType);
+			cmpB = translateMapType(b.initAttributes.mapType);
+			break;
+		case 'description':
+			cmpA = a.description;
+			cmpB = b.description;
+			break;
+		}
+
+		if (cmpA < cmpB)
+			return -sortOrder;
+		else if (cmpA > cmpB)
+			return +sortOrder;
+
+		return 0;
+	});
+
+	let list = g_SavedGamesMetadata.map(metadata => {
+		let isCompatible = isCompatibleSavegame(metadata, engineInfo);
+		return {
+			"date": generateSavegameDateString(metadata, engineInfo),
+			"mapName": compatibilityColor(translate(metadata.initAttributes.settings.Name), isCompatible),
+			"mapType": compatibilityColor(translateMapType(metadata.initAttributes.mapType), isCompatible),
+			"description": compatibilityColor(metadata.description, isCompatible)
+		};
+	});
+
+	if (list.length)
+		list = prepareForDropdown(list);
+
+	gameSelection.list_date = list.date || [];
+	gameSelection.list_mapName = list.mapName || [];
+	gameSelection.list_mapType = list.mapType || [];
+	gameSelection.list_description = list.description || [];
+
+	// Change these last, otherwise crash
+	// list strings used in the delete dialog
+	gameSelection.list = g_SavedGamesMetadata.map(metadata => generateSavegameLabel(metadata, engineInfo));
+	gameSelection.list_data = g_SavedGamesMetadata.map(metadata => metadata.id);
+
+	let selectedGameIndex = g_SavedGamesMetadata.findIndex(metadata => metadata.id == selectedGameId);
+	if (selectedGameIndex != -1)
+		gameSelection.selected = selectedGameIndex;
+	else if (gameSelection.selected >= g_SavedGamesMetadata.length) // happens when deleting the last saved game
+		gameSelection.selected = g_SavedGamesMetadata.length - 1;
+	else if (gameSelection.selected == -1 && g_SavedGamesMetadata.length)
 		gameSelection.selected = 0;
-	else if (gameSelection.selected >= savedGames.length) // happens when deleting the last saved game
-		gameSelection.selected = savedGames.length - 1;
-	else
-		selectionChanged();
 
 	Engine.GetGUIObjectByName("deleteGameButton").tooltip = deleteTooltip();
 }
 
 function selectionChanged()
 {
-	let gameSelection = Engine.GetGUIObjectByName("gameSelection");
-	let selectionEmpty = gameSelection.selected == -1;
-	Engine.GetGUIObjectByName("invalidGame").hidden = !selectionEmpty;
-	Engine.GetGUIObjectByName("validGame").hidden = selectionEmpty;
+	let metadata = g_SavedGamesMetadata[Engine.GetGUIObjectByName("gameSelection").selected];
+	Engine.GetGUIObjectByName("invalidGame").hidden = !!metadata;
+	Engine.GetGUIObjectByName("validGame").hidden = !metadata;
+	Engine.GetGUIObjectByName("loadGameButton").enabled = !!metadata;
+	Engine.GetGUIObjectByName("deleteGameButton").enabled = !!metadata;
 
-	if (selectionEmpty)
+	if (!metadata)
 		return;
 
-	let metadata = g_SavedGamesMetadata[gameSelection.selected];
-
 	Engine.GetGUIObjectByName("savedMapName").caption = translate(metadata.initAttributes.settings.Name);
 	let mapData = getMapDescriptionAndPreview(metadata.initAttributes.mapType, metadata.initAttributes.map);
 	setMapPreviewImage("savedInfoPreview", mapData.preview);
@@ -160,25 +216,3 @@ function reallyLoadGame(gameId)
 		"savedGUIData": metadata.gui
 	});
 }
-
-function deleteGame()
-{
-	let gameSelection = Engine.GetGUIObjectByName("gameSelection");
-	let gameID = gameSelection.list_data[gameSelection.selected];
-
-	if (!gameID)
-		return;
-
-	if (Engine.HotkeyIsPressed("session.savedgames.noconfirmation"))
-		reallyDeleteGame(gameID);
-	else
-		messageBox(
-			500, 200,
-			sprintf(translate("\"%(label)s\""), {
-				"label": gameSelection.list[gameSelection.selected]
-			}) + "\n" + translate("Saved game will be permanently deleted, are you sure?"),
-			translate("DELETE"),
-			[translate("No"), translate("Yes")],
-			[null, function(){ reallyDeleteGame(gameID); }]
-		);
-}
