@@ -4,11 +4,12 @@ var g_SavedGameData;
 function selectDescription()
 {
 	let gameSelection = Engine.GetGUIObjectByName("gameSelection");
-	if (gameSelection.selected == -1)
+	let gameID = gameSelection.list_data[gameSelection.selected];
+	Engine.GetGUIObjectByName("deleteGameButton").enabled = !!gameID;
+
+	if (!gameID)
 		return;
 
-	let gameID = gameSelection.list_data[gameSelection.selected];
-	Engine.GetGUIObjectByName("deleteGameButton").enabled = true;
 	Engine.GetGUIObjectByName("saveGameDesc").caption = g_Descriptions[gameID];
 }
 
@@ -19,10 +20,11 @@ function init(data)
 	g_SavedGameData.timeElapsed = simulationState.timeElapsed;
 	g_SavedGameData.states = simulationState.players.map(pState => pState.state);
 
-	let gameSelection = Engine.GetGUIObjectByName("gameSelection");
-	Engine.GetGUIObjectByName("deleteGameButton").enabled = false;
-
 	let savedGames = Engine.GetSavedGames().sort(sortDecreasingDate);
+
+	let gameSelection = Engine.GetGUIObjectByName("gameSelection");
+	gameSelection.enabled = savedGames.length;
+
 	if (!savedGames.length)
 	{
 		gameSelection.list = [translate("No saved games found")];
@@ -34,7 +36,8 @@ function init(data)
 	for (let game of savedGames)
 		g_Descriptions[game.id] = game.metadata.description || "";
 
-	gameSelection.list = savedGames.map(game => generateLabel(game.metadata));
+	let engineInfo = Engine.GetEngineInfo();
+	gameSelection.list = savedGames.map(game => generateSavegameLabel(game.metadata, engineInfo));
 	gameSelection.list_data = savedGames.map(game => game.id);
 	gameSelection.selected = -1;
 
@@ -49,7 +52,7 @@ function saveGame()
 	let desc = Engine.GetGUIObjectByName("saveGameDesc").caption;
 	let name = gameID || "savegame";
 
-	if (gameSelection.selected == -1)
+	if (!gameID)
 	{
 		reallySaveGame(name, desc, true);
 		return;
@@ -80,22 +83,6 @@ function closeSave()
 	Engine.PopGuiPageCB(0);
 }
 
-function deleteGame()
-{
-	let gameSelection = Engine.GetGUIObjectByName("gameSelection");
-	let gameLabel = gameSelection.list[gameSelection.selected];
-	let gameID = gameSelection.list_data[gameSelection.selected];
-
-	messageBox(
-		500, 200,
-		sprintf(translate("\"%(label)s\""), { "label": gameLabel }) + "\n" +
-			translate("Saved game will be permanently deleted, are you sure?"),
-		translate("DELETE"),
-		[translate("No"), translate("Yes")],
-		[null, function(){ reallyDeleteGame(gameID); }]
-	);
-}
-
 // HACK: Engine.SaveGame* expects this function to be defined on the current page.
 // That's why we have to pass the data to this page even if we don't need it.
 function getSavedGameData()
