@@ -1,3 +1,8 @@
+/**
+ * Which enemy entity types will be attacked on sight when patroling.
+ */
+var g_PatrolTargets = ["Unit"];
+
 /**
  * List of different actions units can execute,
  * this is mostly used to determine which actions can be executed
@@ -131,9 +136,10 @@ var unitActions =
 				return false;
 
 			return {
-				"possible": Engine.GuiInterfaceCall("CanCapture", {
+				"possible": Engine.GuiInterfaceCall("CanAttack", {
 					"entity": entState.id,
-					"target": targetState.id
+					"target": targetState.id,
+					"types": ["Capture"]
 				})
 			};
 		},
@@ -178,7 +184,8 @@ var unitActions =
 			return {
 				"possible": Engine.GuiInterfaceCall("CanAttack", {
 					"entity": entState.id,
-					"target": targetState.id
+					"target": targetState.id,
+					"types": ["!Capture"]
 				})
 			};
 		},
@@ -218,7 +225,7 @@ var unitActions =
 				"x": target.x,
 				"z": target.z,
 				"target": action.target,
-				"targetClasses": { "attack": ["Unit"] }, // patrol should only attack units
+				"targetClasses": { "attack": g_PatrolTargets },
 				"queued": queued,
 				"allowCapture": false
 			});
@@ -227,11 +234,14 @@ var unitActions =
 		},
 		"getActionInfo": function(entState, targetState)
 		{
+			if (!entState.unitAI || !entState.unitAI.canPatrol)
+				return false;
+
 			return { "possible": true };
 		},
 		"hotkeyActionCheck": function(target, selection)
 		{
-			if (!someUnitAI(selection) ||
+			if (!someCanPatrol(selection) ||
 			    !Engine.HotkeyIsPressed("session.patrol") ||
 			    !getActionInfo("patrol", target).possible)
 				return false;
@@ -435,7 +445,7 @@ var unitActions =
 
 			return true;
 		},
-		"getActionInfo":  function(entState, targetState)
+		"getActionInfo": function(entState, targetState)
 		{
 			if (!targetState.resourceSupply)
 				return false;
@@ -547,7 +557,7 @@ var unitActions =
 		"getActionInfo": function(entState, targetState)
 		{
 			if (targetState.foundation || !entState.trader || !targetState.market ||
-			    !playerCheck(entState, targetState, ["Player", "Ally"]) ||
+			    playerCheck(entState, targetState, ["Enemy"]) ||
 			    !(targetState.market.land && hasClass(entState, "Organic") ||
 			      targetState.market.naval && hasClass(entState, "Ship")))
 				return false;
@@ -637,8 +647,7 @@ var unitActions =
 		},
 		"getActionInfo": function(entState, targetState)
 		{
-			if (!hasClass(entState, "Unit") ||
-			    !targetState.garrisonHolder ||
+			if (!entState.canGarrison || !targetState.garrisonHolder ||
 			    !playerCheck(entState, targetState, ["Player", "MutualAlly"]))
 				return false;
 
@@ -1026,18 +1035,15 @@ var unitActions =
 var g_EntityCommands =
 {
 	"unload-all": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			if (!entState.garrisonHolder)
-				return false;
-
 			let count = 0;
-			for (let ent in g_Selection.selected)
-			{
-				let state = GetEntityState(+ent);
-				if (state.garrisonHolder)
-					count += state.garrisonHolder.entities.length;
-			}
+			for (let entState of entStates)
+				if (entState.garrisonHolder)
+					count += entState.garrisonHolder.entities.length;
+
+			if (!count)
+				return false;
 
 			return {
 				"tooltip": colorizeHotkey("%(hotkey)s" + " ", "session.unload") +
@@ -1046,20 +1052,16 @@ var g_EntityCommands =
 				"count": count,
 			};
 		},
-		"execute": function(entState)
+		"execute": function()
 		{
 			unloadAll();
 		},
 	},
+
 	"delete": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			let deleteReason = isUndeletable(entState);
-			return deleteReason ?
-				{
-					"tooltip": deleteReason,
-					"icon": "kill_small_disabled.png"
-				} :
+			return entStates.some(entState => !isUndeletable(entState)) ?
 				{
 					"tooltip":
 						colorizeHotkey("%(hotkey)s" + " ", "session.kill") +
@@ -1069,30 +1071,35 @@ var g_EntityCommands =
 							"session.noconfirmation"
 						),
 					"icon": "kill_small.png"
+				} :
+				{
+					// Get all delete reasons and remove duplications
+					"tooltip": entStates.map(entState => isUndeletable(entState))
+						.filter((reason, pos, self) =>
+							self.indexOf(reason) == pos && reason
+						).join("\n"),
+					"icon": "kill_small_disabled.png"
 				};
 		},
-		"execute": function(entState)
+		"execute": function(entStates)
 		{
-			if (isUndeletable(entState))
-				return;
-
-			let selection = g_Selection.toList();
-			if (!selection.length)
+			if (!entStates.length || entStates.every(entState => isUndeletable(entState)))
 				return;
 
 			if (Engine.HotkeyIsPressed("session.noconfirmation"))
 				Engine.PostNetworkCommand({
 					"type": "delete-entities",
-					"entities": selection
+					"entities": entStates.map(entState => entState.id)
 				});
 			else
-				openDeleteDialog(selection);
+				openDeleteDialog(entStates.map(entState => entState.id));
 		},
 	},
+
 	"stop": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			if (!entState.unitAI)
+			if (entStates.every(entState => !entState.unitAI))
 				return false;
 
 			return {
@@ -1101,18 +1108,17 @@ var g_EntityCommands =
 				"icon": "stop.png"
 			};
 		},
-		"execute": function(entState)
+		"execute": function(entStates)
 		{
-			let selection = g_Selection.toList();
-			if (selection.length)
-				stopUnits(selection);
+			if (entStates.length)
+				stopUnits(entStates.map(entState => entState.id));
 		},
 	},
 
 	"garrison": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			if (!entState.unitAI || entState.turretParent)
+			if (entStates.every(entState => !entState.unitAI || entState.turretParent))
 				return false;
 
 			return {
@@ -1121,7 +1127,7 @@ var g_EntityCommands =
 				"icon": "garrison.png"
 			};
 		},
-		"execute": function(entState)
+		"execute": function()
 		{
 			inputState = INPUT_PRESELECTEDACTION;
 			preSelectedAction = ACTION_GARRISON;
@@ -1129,13 +1135,14 @@ var g_EntityCommands =
 	},
 
 	"unload": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			if (!entState.unitAI || !entState.turretParent)
-				return false;
-
-			let p = GetEntityState(entState.turretParent);
-			if (!p.garrisonHolder || p.garrisonHolder.entities.indexOf(entState.id) == -1)
+			if (entStates.every(entState => {
+				if (!entState.unitAI || !entState.turretParent)
+					return true;
+				let parent = GetEntityState(entState.turretParent);
+				return !parent || !parent.garrisonHolder || parent.garrisonHolder.entities.indexOf(entState.id) == -1;
+			}))
 				return false;
 
 			return {
@@ -1143,16 +1150,16 @@ var g_EntityCommands =
 				"icon": "garrison-out.png"
 			};
 		},
-		"execute": function(entState)
+		"execute": function()
 		{
 			unloadSelection();
 		},
 	},
 
 	"repair": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			if (!entState.builder)
+			if (entStates.every(entState => !entState.builder))
 				return false;
 
 			return {
@@ -1161,7 +1168,7 @@ var g_EntityCommands =
 				"icon": "repair.png"
 			};
 		},
-		"execute": function(entState)
+		"execute": function()
 		{
 			inputState = INPUT_PRESELECTEDACTION;
 			preSelectedAction = ACTION_REPAIR;
@@ -1169,9 +1176,9 @@ var g_EntityCommands =
 	},
 
 	"focus-rally": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			if (!entState.rallyPoint)
+			if (entStates.every(entState => !entState.rallyPoint))
 				return false;
 
 			return {
@@ -1180,13 +1187,23 @@ var g_EntityCommands =
 				"icon": "focus-rally.png"
 			};
 		},
-		"execute": function(entState)
+		"execute": function(entStates)
 		{
+			// TODO: Would be nicer to cycle between the rallypoints of multiple entities instead of just using the first
 			let focusTarget;
-			if (entState.rallyPoint && entState.rallyPoint.position)
-				focusTarget = entState.rallyPoint.position;
-			else if (entState.position)
-				focusTarget = entState.position;
+			for (let entState of entStates)
+				if (entState.rallyPoint && entState.rallyPoint.position)
+				{
+					focusTarget = entState.rallyPoint.position;
+					break;
+				}
+			if (!focusTarget)
+				for (let entState of entStates)
+					if (entState.position)
+					{
+						focusTarget = entState.position;
+						break;
+					}
 
 			if (focusTarget)
 				Engine.CameraMoveTo(focusTarget.x, focusTarget.z);
@@ -1194,9 +1211,9 @@ var g_EntityCommands =
 	},
 
 	"back-to-work": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			if (!entState.unitAI || !entState.unitAI.hasWorkOrders)
+			if (entStates.every(entState => !entState.unitAI || !entState.unitAI.hasWorkOrders))
 				return false;
 
 			return {
@@ -1205,17 +1222,17 @@ var g_EntityCommands =
 				"icon": "production.png"
 			};
 		},
-		"execute": function(entState)
+		"execute": function()
 		{
 			backToWork();
 		},
 	},
 
 	"add-guard": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			if (!entState.unitAI || !entState.unitAI.canGuard ||
-			    entState.unitAI.isGuarding)
+			if (entStates.every(entState =>
+				!entState.unitAI || !entState.unitAI.canGuard || entState.unitAI.isGuarding))
 				return false;
 
 			return {
@@ -1224,7 +1241,7 @@ var g_EntityCommands =
 				"icon": "add-guard.png"
 			};
 		},
-		"execute": function(entState)
+		"execute": function()
 		{
 			inputState = INPUT_PRESELECTEDACTION;
 			preSelectedAction = ACTION_GUARD;
@@ -1232,9 +1249,9 @@ var g_EntityCommands =
 	},
 
 	"remove-guard": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			if (!entState.unitAI || !entState.unitAI.isGuarding)
+			if (entStates.every(entState => !entState.unitAI || !entState.unitAI.isGuarding))
 				return false;
 
 			return {
@@ -1242,34 +1259,35 @@ var g_EntityCommands =
 				"icon": "remove-guard.png"
 			};
 		},
-		"execute": function(entState)
+		"execute": function()
 		{
 			removeGuard();
 		},
 	},
 
 	"select-trading-goods": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			if (!entState.market)
+			if (entStates.every(entState => !entState.market))
 				return false;
 
 			return {
-				"tooltip": translate("Select trading goods"),
+				"tooltip": translate("Barter & Trade"),
 				"icon": "economics.png"
 			};
 		},
-		"execute": function(entState)
+		"execute": function()
 		{
 			toggleTrade();
 		},
 	},
 
 	"patrol": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			if (g_Selection.toList().every(ent => !GetEntityState(ent).unitAI))
+			if (!entStates.some(entState => entState.unitAI && entState.unitAI.canPatrol))
 				return false;
+
 			return {
 				"tooltip": colorizeHotkey("%(hotkey)s" + " ", "session.patrol") +
 				           translate("Patrol") + "\n" +
@@ -1277,7 +1295,7 @@ var g_EntityCommands =
 				"icon": "patrol.png"
 			};
 		},
-		"execute": function(entState)
+		"execute": function()
 		{
 			inputState = INPUT_PRESELECTEDACTION;
 			preSelectedAction = ACTION_PATROL;
@@ -1285,32 +1303,36 @@ var g_EntityCommands =
 	},
 
 	"share-dropsite": {
-		"getInfo": function(entState)
+		"getInfo": function(entStates)
 		{
-			if (!entState.resourceDropsite || !entState.resourceDropsite.sharable)
+			let sharableEntities = entStates.filter(
+				entState => entState.resourceDropsite && entState.resourceDropsite.sharable);
+			if (!sharableEntities.length)
 				return false;
 
-			let playerState = GetSimState().players[entState.player];
-			if (!playerState.isMutualAlly.some((e, i) => e && i != entState.player))
+			// Returns if none of the entities belong to a player with a mutual ally
+			if (entStates.every(entState => !GetSimState().players[entState.player].isMutualAlly.some(
+				(isAlly, playerId) => isAlly && playerId != entState.player)))
 				return false;
 
-			if (entState.resourceDropsite.shared)
-				return {
+			return sharableEntities.some(entState => !entState.resourceDropsite.shared) ?
+				{
+					"tooltip": translate("Press to allow allies to use this dropsite"),
+					"icon": "locked_small.png"
+				} :
+				{
 					"tooltip": translate("Press to prevent allies from using this dropsite"),
 					"icon": "unlocked_small.png"
 				};
-
-			return {
-				"tooltip": translate("Press to allow allies to use this dropsite"),
-				"icon": "locked_small.png"
-			};
 		},
-		"execute": function(entState)
+		"execute": function(entStates)
 		{
+			let sharableEntities = entStates.filter(
+				entState => entState.resourceDropsite && entState.resourceDropsite.sharable);
 			Engine.PostNetworkCommand({
 				"type": "set-dropsite-sharing",
-				"entities": g_Selection.toList(),
-				"shared": !entState.resourceDropsite.shared
+				"entities": sharableEntities.map(entState => entState.id),
+				"shared": sharableEntities.some(entState => !entState.resourceDropsite.shared)
 			});
 		},
 	}
@@ -1350,7 +1372,7 @@ var g_AllyEntityCommands =
 		},
 		"execute": function(entState)
 		{
-			unloadAllByOwner();
+			unloadAll();
 		},
 	},
 
@@ -1393,6 +1415,12 @@ function playerCheck(entState, targetState, validPlayers)
 	return false;
 }
 
+function hasClass(entState, className)
+{
+	// note: use the functions in globalscripts/Templates.js for more versatile matching
+	return entState.identity && entState.identity.classes.indexOf(className) != -1;
+}
+
 /**
  * Work out whether at least part of the selected entities have UnitAI.
  */
@@ -1420,6 +1448,14 @@ function someGuarding(entities)
 	});
 }
 
+function someCanPatrol(entities)
+{
+	return entities.some(ent => {
+		let entState = GetEntityState(ent);
+		return entState && entState.unitAI && entState.unitAI.canPatrol;
+	});
+}
+
 /**
  * Keep in sync with Commands.js.
  */
