@@ -1,9 +1,7 @@
 Trigger.prototype.SpawnAndAttack = function()
 {
-	var rand = Math.random();
-	// randomize spawn points
-	var spawnPoint = rand > 0.5 ? "B" : "C";
-	var intruders = TriggerHelper.SpawnUnitsFromTriggerPoints(spawnPoint, "units/rome_legionnaire_marian", this.attackSize, 0);
+	var intruders = TriggerHelper.SpawnUnitsFromTriggerPoints(
+			pickRandom(["B", "C"]), "units/rome_legionnaire_marian", this.attackSize, 0);
 
 	for (var origin in intruders)
 	{
@@ -30,8 +28,7 @@ Trigger.prototype.SpawnAndAttack = function()
 	}
 
 	// enlarge the attack time and size
-	// multiply with a number between 1 and 3
-	rand = Math.random() * 2 + 1;
+	var rand = randFloat(1, 3);
 	this.attackTime *= rand;
 	this.attackSize = Math.round(this.attackSize * rand);
 	this.DoAfterDelay(this.attackTime, "SpawnAndAttack", {});
