@@ -1,5 +1,24 @@
 var g_DrawLimits = {}; // GUI limits. Populated by predraw()
 
+var g_TooltipFunctions = [
+	getEntityNamesFormatted,
+	getEntityCostTooltip,
+	getEntityTooltip,
+	getAurasTooltip,
+	getHealthTooltip,
+	getHealerTooltip,
+	getAttackTooltip,
+	getSplashDamageTooltip,
+	getArmorTooltip,
+	getGarrisonTooltip,
+	getProjectilesTooltip,
+	getSpeedTooltip,
+	getGatherTooltip,
+	getPopulationBonusTooltip,
+	getResourceTrickleTooltip,
+	getLootTooltip
+];
+
 /**
  * Draw the structree
  *
@@ -11,17 +30,20 @@ function draw()
 	if (!Object.keys(g_DrawLimits).length)
 		predraw();
 
-	let defWidth = 96;
-	let defMargin = 4;
+	const leftMargin = Engine.GetGUIObjectByName("tree_display").size.left;
+	const defWidth = 96;
+	const defMargin = 4;
+
 	let phaseList = g_ParsedData.phaseList;
 
-	Engine.GetGUIObjectByName("civEmblem").sprite = "stretched:"+g_CivData[g_SelectedCiv].Emblem;
+	Engine.GetGUIObjectByName("civEmblem").sprite = "stretched:" + g_CivData[g_SelectedCiv].Emblem;
 	Engine.GetGUIObjectByName("civName").caption = g_CivData[g_SelectedCiv].Name;
 	Engine.GetGUIObjectByName("civHistory").caption = g_CivData[g_SelectedCiv].History;
 
 	let i = 0;
 	for (let pha of phaseList)
 	{
+		let prodBarWidth = 0;
 		let s = 0;
 		let y = 0;
 
@@ -30,16 +52,24 @@ function draw()
 			let thisEle = Engine.GetGUIObjectByName("phase["+i+"]_struct["+s+"]");
 			if (thisEle === undefined)
 			{
-				error("\""+g_SelectedCiv+"\" has more structures in phase "+pha+" than can be supported by the current GUI layout");
+				error("\""+g_SelectedCiv+"\" has more structures in phase " +
+				      pha + " than can be supported by the current GUI layout");
 				break;
 			}
 
 			let c = 0;
 			let rowCounts = [];
 			stru = g_ParsedData.structures[stru];
-			Engine.GetGUIObjectByName("phase["+i+"]_struct["+s+"]_icon").sprite = "stretched:session/portraits/"+stru.icon;
-			Engine.GetGUIObjectByName("phase["+i+"]_struct["+s+"]_icon").tooltip = assembleTooltip(stru);
-			Engine.GetGUIObjectByName("phase["+i+"]_struct["+s+"]_name").caption = translate(stru.name.specific);
+
+			Engine.GetGUIObjectByName("phase["+i+"]_struct["+s+"]_icon").sprite =
+				"stretched:session/portraits/"+stru.icon;
+
+			Engine.GetGUIObjectByName("phase["+i+"]_struct["+s+"]_icon").tooltip =
+				assembleTooltip(stru);
+
+			Engine.GetGUIObjectByName("phase["+i+"]_struct["+s+"]_name").caption =
+				translate(stru.name.specific);
+
 			thisEle.hidden = false;
 
 			for (let r in g_DrawLimits[pha].prodQuant)
@@ -47,39 +77,54 @@ function draw()
 				let p = 0;
 				r = +r; // force int
 				let prod_pha = phaseList[phaseList.indexOf(pha) + r];
+
 				if (stru.production.units[prod_pha])
-				{
 					for (let prod of stru.production.units[prod_pha])
 					{
 						prod = g_ParsedData.units[prod];
 						if (!drawProdIcon(i, s, r, p, prod))
 							break;
-						p++;
+						++p;
 					}
-				}
+
+				if (stru.upgrades[prod_pha])
+					for (let upgrade of stru.upgrades[prod_pha])
+					{
+						if (!drawProdIcon(i, s, r, p, upgrade))
+							break;
+						++p;
+					}
+
 				if (stru.wallset && prod_pha == pha)
 				{
-					for (let prod of [stru.wallset.gate, stru.wallset.tower])
-					{
-						if (!drawProdIcon(i, s, r, p, prod))
-							break;
-						p++;
-					}
+					if (!drawProdIcon(i, s, r, p, stru.wallset.tower))
+						break;
+					++p;
 				}
+
 				if (stru.production.technology[prod_pha])
-				{
 					for (let prod of stru.production.technology[prod_pha])
 					{
-						prod = (depath(prod).slice(0,5) == "phase") ? g_ParsedData.phases[prod] : g_ParsedData.techs[prod];
+						prod = clone(basename(prod).startsWith("phase") ?
+							g_ParsedData.phases[prod] :
+							g_ParsedData.techs[g_SelectedCiv][prod]);
+
+						for (let res in stru.techCostMultiplier)
+							if (prod.cost[res])
+								prod.cost[res] *= stru.techCostMultiplier[res];
+
 						if (!drawProdIcon(i, s, r, p, prod))
 							break;
-						p++;
+
+						++p;
 					}
-				}
+
 				rowCounts[r] = p;
+
 				if (p>c)
 					c = p;
-				hideRemaining("phase["+i+"]_struct["+s+"]_row["+r+"]_prod[", p, "]");
+
+				hideRemaining("phase["+i+"]_struct["+s+"]_row["+r+"]", p);
 			}
 
 			let size = thisEle.size;
@@ -99,13 +144,26 @@ function draw()
 				phaEle.size = size;
 			}
 			++r;
-			hideRemaining("phase["+i+"]_struct["+s+"]_row[", r, "]");
+			hideRemaining("phase["+i+"]_struct["+s+"]_rows", r);
 			++s;
+			prodBarWidth += eleWidth + defMargin;
 		}
-		hideRemaining("phase["+i+"]_struct[", s, "]");
+
+		hideRemaining("phase["+i+"]", s);
+
+		let offset = getPositionOffset(i);
+		// Resize phase bars
+		for (let j = 1; j < phaseList.length - i; ++j)
+		{
+			let prodBar = Engine.GetGUIObjectByName("phase["+i+"]_bar["+(j-1)+"]");
+			let prodBarSize = prodBar.size;
+			prodBarSize.right = leftMargin + prodBarWidth;
+			prodBar.size = prodBarSize;
+		}
+
 		++i;
 	}
-	
+
 	let t = 0;
 	for (let trainer of g_CivData[g_SelectedCiv].trainList)
 	{
@@ -121,19 +179,43 @@ function draw()
 		Engine.GetGUIObjectByName("trainer["+t+"]_icon").tooltip = assembleTooltip(trainer);
 		Engine.GetGUIObjectByName("trainer["+t+"]_name").caption = translate(trainer.name.specific);
 		thisEle.hidden = false;
-		
+
 		let p = 0;
-		for (let prod of trainer.trainer)
-		{
-			prod = g_ParsedData.units[prod];
-			if (!drawProdIcon(null, t, null, p, prod))
-				break;
-			p++;
-		}
-		hideRemaining("trainer["+t+"]_prod[", p, "]");
+		if (trainer.production)
+			for (let prodType in trainer.production)
+				for (let prod of trainer.production[prodType])
+				{
+					switch (prodType)
+					{
+					case "units":
+						prod = g_ParsedData.units[prod];
+						break;
+					case "techs":
+						prod = clone(g_ParsedData.techs[g_SelectedCiv][prod]);
+						for (let res in trainer.techCostMultiplier)
+							if (prod.cost[res])
+								prod.cost[res] *= trainer.techCostMultiplier[res];
+						break;
+					default:
+						continue;
+					}
+					if (!drawProdIcon(null, t, null, p, prod))
+						break;
+					++p;
+				}
+
+		if (trainer.upgrades)
+			for (let upgrade of trainer.upgrades)
+			{
+				if (!drawProdIcon(null, t, null, p, upgrade))
+					break;
+				++p;
+			}
+
+		hideRemaining("trainer["+t+"]_row", p);
 
 		let size = thisEle.size;
-		size.right = size.left + ((p*24 < defWidth)?defWidth:p*24)+4;
+		size.right = size.left + Math.max(p*24, defWidth) + 4;
 		thisEle.size = size;
 
 		let eleWidth = size.right - size.left;
@@ -144,7 +226,7 @@ function draw()
 		phaEle.size = size;
 		++t;
 	}
-	hideRemaining("trainer[", t, "]");
+	hideRemaining("trainers", t);
 
 	let size = Engine.GetGUIObjectByName("display_tree").size;
 	size.right = t > 0 ? -124 : -4;
@@ -156,12 +238,14 @@ function draw()
 function drawProdIcon(pha, s, r, p, prod)
 {
 	let prodEle = Engine.GetGUIObjectByName("phase["+pha+"]_struct["+s+"]_row["+r+"]_prod["+p+"]");
+
 	if (pha === null)
 		prodEle = Engine.GetGUIObjectByName("trainer["+s+"]_prod["+p+"]");
 
 	if (prodEle === undefined)
 	{
-		error("The "+(pha === null ? "trainer units" : "structures")+" of \""+g_SelectedCiv+"\" have more production icons than can be supported by the current GUI layout");
+		error("The "+(pha === null ? "trainer units" : "structures") + " of \"" + g_SelectedCiv +
+		      "\" have more production icons than can be supported by the current GUI layout");
 		return false;
 	}
 
@@ -184,22 +268,10 @@ function getPositionOffset(idx)
 	return size;
 }
 
-function hideRemaining(prefix, idx, suffix)
-{
-	let obj = Engine.GetGUIObjectByName(prefix+idx+suffix);
-	while (obj)
-	{
-		obj.hidden = true;
-		++idx;
-		obj = Engine.GetGUIObjectByName(prefix+idx+suffix);
-	}
-}
-
-
 /**
  * Positions certain elements that only need to be positioned once
  * (as <repeat> does not reposition automatically).
- * 
+ *
  * Also detects limits on what the GUI can display by iterating through the set
  * elements of the GUI. These limits are then used by draw().
  */
@@ -214,25 +286,21 @@ function predraw()
 	{
 		let offset = getPositionOffset(i);
 		// Align the phase row
-		Engine.GetGUIObjectByName("phase["+i+"]").size = "8 16+"+offset+" 100% 100%";
+		Engine.GetGUIObjectByName("phase["+i+"]").size = "8 16+" + offset + " 100% 100%";
 
 		// Set phase icon
 		let phaseIcon = Engine.GetGUIObjectByName("phase["+i+"]_phase");
-		phaseIcon.sprite = "stretched:session/portraits/"+g_ParsedData.phases[pha].icon;
 		phaseIcon.size = "16 32+"+offset+" 48+16 48+32+"+offset;
 
-		// Position prod bars
+		// Set initial prod bar size
 		let j = 1;
-		for (; j < phaseCount - i; ++j)
+		for (; j < phaseList.length - i; ++j)
 		{
 			let prodBar = Engine.GetGUIObjectByName("phase["+i+"]_bar["+(j-1)+"]");
-			prodBar.size = "40 1+"+(24*j)+"+98+"+offset+" 100%-8 1+"+(24*j)+"+98+"+offset+"+22";
-			// Set phase icon
-			let prodBarIcon = Engine.GetGUIObjectByName("phase["+i+"]_bar["+(j-1)+"]_icon");
-			prodBarIcon.sprite = "stretched:session/portraits/"+g_ParsedData.phases[phaseList[i+j]].icon;
+			prodBar.size = "40 1+"+(24*j)+"+98+"+offset+" 0 1+"+(24*j)+"+98+"+offset+"+22";
 		}
 		// Hide remaining prod bars
-		hideRemaining("phase["+i+"]_bar[", j-1, "]");
+		hideRemaining("phase["+i+"]_bars", j-1);
 
 		let s = 0;
 		let ele = Engine.GetGUIObjectByName("phase["+i+"]_struct["+s+"]");
@@ -283,16 +351,16 @@ function predraw()
 		g_DrawLimits[pha].structQuant = s;
 		++i;
 	}
-	hideRemaining("phase[", i, "]");
-	hideRemaining("phase[", i, "]_bar");
-	
+	hideRemaining("phase_rows", i);
+	hideRemaining("phase_ident", i);
+
 	let t = 0;
 	let ele = Engine.GetGUIObjectByName("trainer["+t+"]");
 	g_DrawLimits.trainer = {
 		"trainerQuant": 0,
 		"prodQuant": 0
 	};
-	
+
 	let x = 4;
 	do
 	{
@@ -310,18 +378,18 @@ function predraw()
 		} while (prodEle !== undefined);
 		Engine.GetGUIObjectByName("trainer["+t+"]_row").size = "4 100%-24"+" 100%-4 100%";
 		g_DrawLimits.trainer.prodQuant = p;
-		
+
 		let size = ele.size;
 		size.top += x;
 		size.bottom += x + 24;
 		x += size.bottom - size.top + 8;
 		ele.size = size;
-		
+
 		t++;
 		ele = Engine.GetGUIObjectByName("trainer["+t+"]");
-		
+
 	} while (ele !== undefined);
-	
+
 	g_DrawLimits.trainer.trainerQuant = t;
 }
 
@@ -329,54 +397,30 @@ function predraw()
  * Assemble a tooltip text
  *
  * @param  template Information about a Unit, a Structure or a Technology
- *
  * @return  The tooltip text, formatted.
  */
 function assembleTooltip(template)
 {
-	let txt = getEntityNamesFormatted(template);
-	txt += '\n' + getEntityCostTooltip(template, 1);
+	return g_TooltipFunctions.map(func => func(template)).filter(tip => tip).join("\n");
+}
 
-	if (template.tooltip)
-		txt += '\n' + g_TooltipTextFormats.body[0] +  translate(template.tooltip) + g_TooltipTextFormats.body[1];
-
-	if (template.auras)
-		txt += getAurasTooltip(template);
-
-	if (template.health)
-		txt += '\n' + sprintf(translate("%(label)s %(details)s"), {
-			"label": g_TooltipTextFormats.header[0] + translate("Health:") + g_TooltipTextFormats.header[1],
-			"details": template.health
-		});
-
-	if (template.healer)
-		txt += '\n' + getHealerTooltip(template);
-
-	if (template.attack)
-		txt += '\n' + getAttackTooltip(template);
-
-	if (template.armour)
-		txt += '\n' + getArmorTooltip(template.armour);
-
-	if (template.speed)
-		txt += '\n' + getSpeedTooltip(template);
-
-	if (template.gather)
+function drawPhaseIcons()
+{
+	for (let i = 0; i < g_ParsedData.phaseList.length; ++i)
 	{
-		let rates = [];
-		for (let type in template.gather)
-			rates.push(sprintf(translate("%(resourceIcon)s %(rate)s"), {
-				"resourceIcon": getCostComponentDisplayName(type),
-				"rate": template.gather[type]
-			}));
+		drawPhaseIcon("phase["+i+"]_phase", i);
 
-		txt += '\n' + sprintf(translate("%(label)s %(details)s"), {
-			"label": g_TooltipTextFormats.header[0] + translate("Gather Rates:") + g_TooltipTextFormats.header[1],
-			"details": rates.join("  ")
-		});
+		for (let j = 1; j < g_ParsedData.phaseList.length - i; ++j)
+			drawPhaseIcon("phase["+i+"]_bar["+(j-1)+"]_icon", j+i);
 	}
+}
 
-	txt += getPopulationBonusTooltip(template);
+function drawPhaseIcon(guiObjectName, phaseIndex)
+{
+	let phaseName = g_ParsedData.phaseList[phaseIndex];
+	let prodPhaseTemplate = g_ParsedData.phases[phaseName + "_" + g_SelectedCiv] || g_ParsedData.phases[phaseName];
 
-	return txt;
+	let phaseIcon = Engine.GetGUIObjectByName(guiObjectName);
+	phaseIcon.sprite = "stretched:session/portraits/" + prodPhaseTemplate.icon;
+	phaseIcon.tooltip = getEntityNamesFormatted(prodPhaseTemplate);
 }
